/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications;

import java.util.List;
import java.util.Map;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.platform.integration.applications.exceptions.ItemException;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectResource;

public interface ApplicationProjectContent {

    public interface STANDARD_PROPERTIES {
        String APPLICATION = "application";
        String GROUP = "group";
        String VERSION = "version";
        String CREATED_ON = "createdOn";
        String CREATED_BY = "createdBy";
        String MODIFIED_ON = "modifiedOn";
        String MODIFIED_BY = "modifiedBy";
    }

    List<ApplicationProjectItem> getItems();

    <T extends ApplicationProjectItem> List<T> getItems(Class<T> itemClass);

    void addItem(ApplicationProjectItem item) throws ItemException;

    void ignoreItem(ApplicationProjectItem item) throws ItemException;

    void removeItem(ApplicationProjectItem item) throws ItemException;

    default boolean isConsistent() {
        return getItems().stream().filter(item -> (item.isRequired() && !item.isIgnored())).count() == 0;
    }

    ApplicationProjectItem addObject(EntaxyRuntimeObject runtimeObject);

    ApplicationProjectItem addResource(EntaxyRuntimeObjectResource runtimeObjectResource);

    ApplicationProjectItem addResource(EntaxyResource entaxyResource);

    Map<String, Object> getProperties();

}
