/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

import ru.entaxy.platform.integration.applications.exceptions.ApplicationException;
import ru.entaxy.platform.integration.applications.exceptions.ProjectException;
import ru.entaxy.platform.integration.applications.exceptions.StorageException;

public interface ApplicationStorage {

    // storage itself

    String getName();

    String getType();

    Map<String, Object> getInfo();

    boolean isEnabled();

    void enable();

    void disable();

    // projects

    List<EntaxyApplicationProject> getProjects();

    EntaxyApplicationProject getProject(String applicationName);

    void addProject(EntaxyApplicationProject project) throws ProjectException, StorageException;

    void removeProject(EntaxyApplicationProject project) throws ProjectException, StorageException;

    EntaxyApplicationProject createProject(String applicationName)
            throws ProjectException, StorageException;

    // applications

    public interface ApplicationImportContent {

        Map<String, ApplicationImportItem> getItems();

        void clear();
    }

    public interface ApplicationImportItem {
        String getName();

        String getLocation();

        String getPath();

        InputStream getInputStream() throws IOException;
    }

    List<EntaxyApplication> getApplications();

    EntaxyApplication getApplication(String applicationName);

    void removeApplication(EntaxyApplication application) throws ApplicationException, StorageException;

    EntaxyApplication createApplication(ApplicationImportContent content) throws ApplicationException, StorageException;

}
