/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications;

import java.util.List;

import ru.entaxy.esb.resources.EntaxyResource.RESOURCE_SCOPE;
import ru.entaxy.platform.base.objects.EntaxyObject.OBJECT_RELATION_SCOPE;

public interface EntaxyApplication {

    String DESCRIPTOR_NAME = "descriptor.json";

    String DEFAULT_GROUP_ID = "entaxy.application";

    enum ITEM_TYPE {
        OBJECT,
        CONFIG,
        BUNDLE,
        RESOURCE,
        FEATURES
    }

    enum ITEM_SCOPE {
        DESIGN,
        RUNTIME,
        UNKNOWN;

        public static ITEM_SCOPE convert(OBJECT_RELATION_SCOPE originScope) {
            try {
                return ITEM_SCOPE.valueOf(originScope.name().toUpperCase());
            } catch (Exception e) {
                return UNKNOWN;
            }
        }

        public static ITEM_SCOPE convert(RESOURCE_SCOPE originScope) {
            try {
                return ITEM_SCOPE.valueOf(originScope.name().toUpperCase());
            } catch (Exception e) {
                return UNKNOWN;
            }
        }

    }

    String getName();

    ApplicationStorage getStorage();

    List<EntaxyApplicationVersion> getVersions();

    EntaxyApplicationVersion getLatestVersion();

    EntaxyApplicationVersion getVersion(String version);

    void removeVersion(String version);


}
