/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications;

import java.util.List;

import ru.entaxy.platform.integration.applications.exceptions.ApplicationVersionException;
import ru.entaxy.platform.integration.applications.exceptions.RevisionException;
import ru.entaxy.platform.integration.applications.exceptions.RevisionIllegalNumber;

public interface EntaxyApplicationVersion extends ApplicationContent {

    EntaxyApplication getApplication();

    String getVersionNumber();

    List<EntaxyApplicationRevision> getRevisions();

    EntaxyApplicationRevision getLastRevision();

    default int compareTo(EntaxyApplicationVersion otherVersion) {
        String[] thisV = otherVersion.getVersionNumber().split("\\.");
        String[] otherV = otherVersion.getVersionNumber().split("\\.");

        for (int i = 0; i < Math.max(thisV.length, otherV.length); i++) {
            try {
                int thisValue = Integer.parseInt(thisV[i]);
                int otherValue = Integer.parseInt(otherV[i]);
                if (thisValue > otherValue)
                    return 1;
                if (thisValue < otherValue)
                    return -1;
            } catch (Exception ignore) {
                continue;
            }
        }

        return 0;
    }

    default EntaxyApplicationRevision getRevision(String revision) throws RevisionException {
        int rev = -1;
        try {
            rev = Integer.parseInt(revision);
        } catch (Exception ignore) {

        }
        if (rev < 0)
            throw new RevisionIllegalNumber(getApplication().getName(), getVersionNumber(), revision);
        return getRevision(rev);
    };

    EntaxyApplicationRevision getRevision(int revision);

    // creates next revision from last revision
    default EntaxyApplicationRevision createRevision() throws ApplicationVersionException {
        if (getLastRevision() == null)
            // create initial revision
            return createRevision(-1);
        return createRevision(getLastRevision().getRevisionNumber());
    };

    // creates next revision from given revision
    // -1 means create origin revision
    EntaxyApplicationRevision createRevision(int fromRevision) throws ApplicationVersionException;

}
