/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.descriptor;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.EntaxyApplication.ITEM_TYPE;

public class ApplicationDescriptor {

    public String name;
    public String group;
    public String version;
    public String buildOn;
    public String buildBy;

    public String mavenUrl;
    public String mavenAsset;

    public ApplicationFeatures features;

    public volatile List<ApplicationRequirement> requirements = new ArrayList<>();

    public volatile List<ApplicationContentItem> items = new ArrayList<>();

    public volatile List<ApplicationComponent> components = new ArrayList<>();

    public String getName() {
        return name;
    }

    public String getGroup() {
        return group;
    }

    public String getVersion() {
        return version;
    }

    public String getBuildOn() {
        return buildOn;
    }

    public String getBuildBy() {
        return buildBy;
    }

    public String getMavenUrl() {
        return mavenUrl;
    }

    public String getMavenAsset() {
        return mavenAsset;
    }

    public List<ApplicationRequirement> getRequirements() {
        return requirements;
    }

    public List<ApplicationContentItem> getItems() {
        return items;
    }

    public List<ApplicationComponent> getComponents() {
        return components;
    }

    public List<ApplicationComponent> getComponents(EntaxyApplication.ITEM_TYPE type) {
        if (type == null)
            return getComponents();
        return components.stream().filter(comp -> type.equals(comp.type)).collect(Collectors.toList());
    }

    public List<ApplicationComponent> getBundleComponents() {
        return getComponents(ITEM_TYPE.BUNDLE);
    }

    public List<ApplicationComponent> getConfigComponents() {
        return getComponents(ITEM_TYPE.CONFIG);
    }

    public ApplicationFeatures getFeatures() {
        return features;
    }

}
