/*-
 * ~~~~~~licensing~~~~~~
 * application-api
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.storage;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import ru.entaxy.platform.integration.applications.ApplicationStorage;

public abstract class AbstractApplicationStorage implements ApplicationStorage {

    protected String name;

    protected String type = "unknown";

    protected boolean isEnabled = true;

    protected Map<String, Object> storageInfo = new HashMap<>();

    protected AbstractApplicationStorage() {
        super();
        if (this.getClass().isAnnotationPresent(ApplicationStorageInfo.class)) {
            ApplicationStorageInfo anno = this.getClass().getAnnotation(ApplicationStorageInfo.class);
            setName(anno.name());
            setEnabled(anno.defaultEnabled());
            type = anno.type();
        }
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setEnabled(boolean isEnabled) {
        this.isEnabled = isEnabled;
    }

    protected void setInfo(String key, Object value) {
        this.storageInfo.put(key, value);
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public String getType() {
        return type;
    }

    @Override
    public Map<String, Object> getInfo() {
        return Collections.unmodifiableMap(storageInfo);
    }

    @Override
    public boolean isEnabled() {
        return isEnabled;
    }

    @Override
    public void enable() {
        setEnabled(true);
    }

    @Override
    public void disable() {
        setEnabled(false);
    }

}
