/*-
 * ~~~~~~licensing~~~~~~
 * application-impl
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.impl.build;

import org.apache.karaf.bundle.core.BundleInfo;
import org.apache.karaf.bundle.core.BundleService;
import org.osgi.framework.Bundle;
import org.osgi.framework.startlevel.BundleStartLevel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.integration.applications.impl.ServiceHelper;

public class BundleDescriptor extends AbstractDescriptor {

    private static final Logger LOG = LoggerFactory.getLogger(BundleDescriptor.class);

    public static BundleDescriptor forBundle(Bundle bundle) {
        BundleDescriptor result = new BundleDescriptor();
        result.readBundle(bundle);
        return result;
    }

    @Override
    protected TYPE getDescriptorType() {
        return TYPE.BUNDLE;
    }

    String symbolicName;
    long id;

    // origin install location
    String installLocation;

    // origin maven artifact location
    String mavenLocation;

    // location from where we can download the artifact
    String sourceLocation;

    public void readBundle(Bundle bundle) {
        symbolicName = bundle.getSymbolicName();
        id = bundle.getBundleId();

        priority = ((BundleStartLevel) bundle.adapt(BundleStartLevel.class)).getStartLevel();

        // installLocation = bundle.getLocation();

        try {
            BundleService bundleService = ServiceHelper.getInstance().getBundleService();
            BundleInfo bundleInfo = bundleService.getInfo(bundle);
            installLocation = bundleInfo.getUpdateLocation();
        } catch (Exception e) {
            LOG.error("Error reading bundle info", e);
            consistent = false;
            return;
        }

        int index = installLocation.indexOf("mvn:");
        if (index < 0) {
            consistent = false;
            return;
        }

        int index2 = installLocation.indexOf('?');
        if (index2 > index)
            mavenLocation = installLocation.substring(index + 4, index2);
        else
            mavenLocation = installLocation.substring(index + 4);

        sourceLocation = installLocation.substring(index);
    }

    @Override
    public String getDescriptorId() {
        return symbolicName + "~" + getDescriptorType();
    }
}
