/*-
 * ~~~~~~licensing~~~~~~
 * application-impl
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.impl.build;

import ru.entaxy.platform.core.artifact.ArtifactCoordinates;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;

public class ConfigDescriptor extends AbstractDescriptor {

    public static ConfigDescriptor forConfig(String configId) {
        ConfigDescriptor result = new ConfigDescriptor();
        result.pid = configId.split(":")[0];
        return result;
    }

    public static ConfigDescriptor forConfig(String configId, ArtifactCoordinates applicationCoordinates) {
        ConfigDescriptor result = new ConfigDescriptor();
        result.pid = configId.split(":")[0];
        result.init(applicationCoordinates);
        return result;
    }


    public static ConfigDescriptor forObjectConfig(EntaxyRuntimeObject configObject,
            EntaxyRuntimeObject configuredObject) {
        ConfigDescriptor result = new ConfigDescriptor();
        result.pid = configObject.getId();
        return result;
    }

    String pid;

    // origin install location
    String installLocation;

    // origin maven artifact location
    String mavenLocation;

    // location from where we can download the artifact
    String sourceLocation;

    @Override
    public String getDescriptorId() {
        return pid + "~" + getDescriptorType();
    }

    @Override
    protected TYPE getDescriptorType() {
        return TYPE.CONFIG;
    }

    public void init(ArtifactCoordinates applicationCoordinates) {
        ArtifactCoordinates localCoordinates = ArtifactCoordinates.fromUrl(applicationCoordinates.toMavenString());
        localCoordinates.type("cfg").classifier(pid);

        mavenLocation = localCoordinates.toMavenString();

        sourceLocation = "mvn:" + mavenLocation;

        installLocation = pid + ".cfg";

    }

}
