/*-
 * ~~~~~~licensing~~~~~~
 * application-impl
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.impl.project;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import ru.entaxy.platform.integration.applications.ApplicationProjectItem;
import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.impl.project.item.ColocationGroup;

public abstract class ApplicationProjectItemImpl implements ApplicationProjectItem {

    boolean isRequired = false;

    boolean isIgnored = false;

    boolean isPlatform = false;

    boolean isDeleted = false;

    EntaxyApplication.ITEM_SCOPE scope = EntaxyApplication.ITEM_SCOPE.DESIGN;

    EntaxyApplication.ITEM_TYPE type;

    Map<String, ApplicationProjectItemImpl> required = new HashMap<>();

    Map<String, ApplicationProjectItemImpl> requiredBy = new HashMap<>();

    ColocationGroup colocationGroup;

    public void setPlatform(Boolean platform) {
        isPlatform = platform;
    }

    public void setRequired(boolean isRequired) {
        this.isRequired = isRequired;
    }

    public void setIgnored(boolean isIgnored) {
        this.isIgnored = isIgnored;
    }

    public void addRequired(ApplicationProjectItemImpl item) {
        required.put(item.getId(), item);
        item.addRequiredBy(this);
    }

    public void removeRequired(ApplicationProjectItemImpl item) {
        item.removeRequiredBy(this);
        required.remove(item.getId());
    }

    protected void addRequiredBy(ApplicationProjectItemImpl item) {
        requiredBy.put(item.getId(), item);
    }

    protected void removeRequiredBy(ApplicationProjectItemImpl item) {
        requiredBy.remove(item.getId());
    }

    public ColocationGroup getColocationGroup() {
        return colocationGroup;
    }

    public void colocate(ColocationGroup colocationGroup) {
        if (colocationGroup == null) {
            if (this.colocationGroup != null)
                this.colocationGroup.remove(this);
        }
        this.colocationGroup = colocationGroup;
        if (this.colocationGroup != null)
            this.colocationGroup.add(this);
    }

    public void updateScope(EntaxyApplication.ITEM_SCOPE newScope) {

        // we'll leave most strict scope as a result

        // leave scope as is
        if (EntaxyApplication.ITEM_SCOPE.DESIGN.equals(newScope))
            return;
        scope = newScope;
    }

    public void setScope(EntaxyApplication.ITEM_SCOPE scope) {
        this.scope = scope;
    }

    protected void setType(EntaxyApplication.ITEM_TYPE type) {
        this.type = type;
    }

    @Override
    public EntaxyApplication.ITEM_SCOPE getScope() {
        return scope;
    }

    @Override
    public EntaxyApplication.ITEM_TYPE getType() {
        return type;
    }

    @Override
    public boolean isGhost() {
        // TODO Auto-generated method stub
        return false;
    }

    @Override
    public boolean isRequired() {
        return isRequired;
    }

    @Override
    public boolean isPlatform() {
        return isPlatform;
    }

    @Override
    public List<ApplicationProjectItem> getRequiredBy() {
        return new ArrayList<>(this.requiredBy.values());
    }

    @Override
    public List<ApplicationProjectItem> getRequirements() {
        return new ArrayList<>(this.required.values());
    }

    @Override
    public boolean isIgnored() {
        return isIgnored;
    }

    @Override
    public void ignore() {
        setIgnored(true);
    }

    @Override
    public boolean isDeleted() {
        return isDeleted;
    }
    
    public void setDeleted(boolean isDeleted) {
        this.isDeleted = isDeleted;
    }

    @Override
    public List<ApplicationProjectItem> getColocated() {
        if (this.colocationGroup == null)
            return Collections.emptyList();
        return this.colocationGroup.getMembers().stream().filter(item -> !item.getId().equalsIgnoreCase(getId()))
                .collect(Collectors.toList());
    }

}
