/*-
 * ~~~~~~licensing~~~~~~
 * application-impl
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.impl.project.item;

import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.item.project.EntaxyConfigItem;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectContainer;

public class ConfigItemImpl extends ObjectItemImpl implements EntaxyConfigItem {

    public static ConfigItemImpl create(EntaxyRuntimeObject runtimeObject) throws IllegalArgumentException {
        EntaxyRuntimeObjectContainer container = runtimeObject.getContainer();
        if (!"bundle".equals(container.getType()))
            throw new IllegalArgumentException("Object container is not a bundle: " + runtimeObject.getObjectFullId());
        String containerId = container.getId();
        long bundleId = -1;
        String bundleSymbolicName = container.getName();
        try {
            bundleId = Long.parseLong(containerId);
            if (bundleId <= 0)
                throw new Exception();
        } catch (Exception e) {
            throw new IllegalArgumentException("Object container id is not a bundle id: ["
                    + runtimeObject.getObjectFullId() + "] [" + containerId + "]");
        }
        return new ConfigItemImpl(bundleId, bundleSymbolicName, runtimeObject);
    }

    public ConfigItemImpl(long bundleId, String bundleSymbolicName, EntaxyRuntimeObject runtimeObject) {
        super(bundleId, bundleSymbolicName, runtimeObject);
        setType(EntaxyApplication.ITEM_TYPE.CONFIG);
    }

}
