/*-
 * ~~~~~~licensing~~~~~~
 * application-impl
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.impl.project.model;

import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.JsonObject;

import ru.entaxy.platform.base.support.JSONUtils.ObjectWrapper;
import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.EntaxyApplication.ITEM_SCOPE;
import ru.entaxy.platform.integration.applications.EntaxyApplication.ITEM_TYPE;
import ru.entaxy.platform.integration.applications.impl.project.model.item.BundleItem;
import ru.entaxy.platform.integration.applications.impl.project.model.item.ConfigItem;
import ru.entaxy.platform.integration.applications.impl.project.model.item.ContentModelItem;
import ru.entaxy.platform.integration.applications.impl.project.model.item.ObjectItem;
import ru.entaxy.platform.integration.applications.impl.project.model.item.ResourceItem;

public class ContentModelItemWrapper extends ObjectWrapper {

    private static final Logger LOG = LoggerFactory.getLogger(ContentModelItemWrapper.class);

    public static boolean isModelItem(JsonObject object) {
        return object.has("id") && object.has("scope") && object.has("type");
    }

    ContentModel contentModel;

    ContentModelItem contentModelItem;

    public ContentModelItemWrapper(ContentModel contentModel) {
        super();
        this.contentModel = contentModel;
    }

    @Override
    public void wrap(JsonObject object, Map<String, Object> context, String path) {
        try {
            String type = object.get("type").getAsString();
            EntaxyApplication.ITEM_TYPE itemType = EntaxyApplication.ITEM_TYPE.valueOf(type.toUpperCase());
            switch (itemType) {
                case BUNDLE:
                    contentModelItem = ContentModel.GSON.fromJson(object, BundleItem.class);
                    break;
                case CONFIG:
                    contentModelItem = ContentModel.GSON.fromJson(object, ConfigItem.class);
                    break;
                case OBJECT:
                    contentModelItem = ContentModel.GSON.fromJson(object, ObjectItem.class);
                    break;
                case RESOURCE:
                    contentModelItem = ContentModel.GSON.fromJson(object, ResourceItem.class);
                    break;
                default:
                    return;
            }
            contentModelItem.setItemType(itemType);

            try {
                contentModelItem.setItemScope(EntaxyApplication.ITEM_SCOPE.valueOf(object.get("scope").getAsString().toUpperCase()));
            } catch (Exception e) {
                contentModelItem.setItemScope(EntaxyApplication.ITEM_SCOPE.UNKNOWN);
            }

            if (contentModelItem.isRequired())
                contentModel.requiredItems.add(contentModelItem);
            else
                contentModel.includedItems.add(contentModelItem);

        } catch (Exception e) {
            LOG.error("Error reading object", e);
        }
    }

}
