/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.karaf.shell.ShellTableFixed;
import ru.entaxy.platform.integration.applications.ApplicationStorage;

@Service
@Command(name = "app-storage-list", scope = ApplicationShell.COMMAND_SCOPE)
public class StorageListCommand extends ApplicationManagerCommandSupport {

    @Option(name = "-d", aliases = {"--show-disabled"}, required = false, description = "Show disabled storage also")
    boolean showDisabled = false;

    @Override
    public Object execute() throws Exception {

        List<ApplicationStorage> storages =
                showDisabled ? applicationManager.getAllStorages() : applicationManager.getActiveStorages();

        Collections.sort(storages, new Comparator<ApplicationStorage>() {
            @Override
            public int compare(ApplicationStorage o1, ApplicationStorage o2) {
                return o1.getName().compareTo(o2.getName());
            }
        });

        ShellTableFixed table = new ShellTableFixed();
        table.column("Name");
        table.column("Type");
        table.column("Enabled");
        table.column("Info");

        for (ApplicationStorage storage : storages) {
            String storageInfo = storage.getInfo().entrySet().stream()
                    .map(entry -> entry.getKey().concat(": ")
                            .concat(entry.getValue() == null ? "" : entry.getValue().toString()))
                    .collect(Collectors.joining("\n"));
            table.addRow().addContent(storage.getName(), storage.getType(), storage.isEnabled() ? "*" : "",
                    storageInfo);
        }

        table.print(System.out);

        return null;
    }

}
