/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.application;

import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.EntaxyApplicationRevision;
import ru.entaxy.platform.integration.applications.EntaxyApplicationVersion;
import ru.entaxy.platform.integration.applications.shell.ApplicationShell;
import ru.entaxy.platform.config.runtime.ObjectConfigEditor;
import ru.entaxy.platform.config.runtime.ObjectConfigEditor.ConfigProperty;

@Service
@Command(name = "app-revision-config-set", scope = ApplicationShell.COMMAND_SCOPE)
public class ConfigSetPropertyCommand extends ManagedApplicationConfigCommand {

    @Argument(required = true, index = 0)
    String propertyName;

    @Argument(required = true, index = 1)
    String propertyValue;

    @Override
    protected void doExecute(EntaxyApplication application, EntaxyApplicationVersion version,
            EntaxyApplicationRevision revision, ObjectConfigEditor editor) {

        ConfigProperty property = editor.getAsConfigProperty(propertyName);
        if (property == null) {
            // output error
            System.err.println("ERROR: property [" + propertyName + "] not found");
            return;
        }

        if (property.hint.isInternal()) {
            // output error
            System.err.println("ERROR: property [" + propertyName + "] is system, it can't be updated");
            return;
        }

        try {
            editor.setValue(propertyName, propertyValue);
        } catch (Exception e) {
            // output error
            System.err.println("ERROR: setting property [" + propertyName + "], see exception below");
            e.printStackTrace();
            return;
        }
    }

}
