/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.application;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.EntaxyApplicationRevision;
import ru.entaxy.platform.integration.applications.EntaxyApplicationVersion;
import ru.entaxy.platform.integration.applications.exceptions.ApplicationVersionException;
import ru.entaxy.platform.integration.applications.shell.ApplicationShell;

@Service
@Command(name = "app-revision-create", scope = ApplicationShell.COMMAND_SCOPE)
public class RevisionCreateCommand extends ManagedApplicationVersionCommand {

    @Option(name = "-r", aliases = {"--from-revision"}, required = false)
    String fromRevision;

    @Override
    protected void doExecute(EntaxyApplication application, EntaxyApplicationVersion version) {

        int sourceRevision = -1;

        if (CommonUtils.isValid(fromRevision))
            try {
                sourceRevision = Integer.parseInt(fromRevision);
            } catch (Exception e) {
                // print error
                System.err.println("Invalid source revision number: " + fromRevision);
                return;
            }

        EntaxyApplicationRevision revision = null;

        try {
            if (sourceRevision < 0)
                revision = version.createRevision();
            else
                revision = version.createRevision(sourceRevision);
        } catch (ApplicationVersionException e) {
            // print error
            System.err.println("Error creating revision: " + e.getMessage());
            e.printStackTrace();
        }

        if (revision == null) {
            // print error
            System.err.println("Error creating revision: revision is null");
            return;
        }

        // print result
        System.out.println(String.format("Created revision %d with status %s", revision.getRevisionNumber(),
                revision.getStatus()));

    }

}
