/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.application;

import org.apache.karaf.shell.api.action.Option;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.EntaxyApplicationRevision;
import ru.entaxy.platform.integration.applications.EntaxyApplicationVersion;

public abstract class RevisionSupport extends ManagedApplicationVersionCommand {

    @Option(name = "-r", aliases = {"--revision"})
    public String revisionNumber;

    @Override
    protected void doExecute(EntaxyApplication application, EntaxyApplicationVersion version) {
        int revisionNum = version.getLastRevision().getRevisionNumber();
        if (CommonUtils.isValid(revisionNumber))
            try {
                revisionNum = Integer.parseInt(revisionNumber);
            } catch (Exception e) {

                // print error
                System.err.println("Invalid revision number: " + revisionNumber);
                return;
            }

        EntaxyApplicationRevision revision = version.getRevision(revisionNum);
        doExecute(application, version, revision);
    }

    protected void printRevision(EntaxyApplicationRevision revision) {
        // output
        System.out.println(
                String.format("Revision: %d; status: %s", revision.getRevisionNumber(), revision.getStatus().name()));
    }

    protected abstract void doExecute(EntaxyApplication application, EntaxyApplicationVersion version,
            EntaxyApplicationRevision revision);

}
