/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.project;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.karaf.shell.ShellTableFixed;
import ru.entaxy.platform.integration.applications.EntaxyApplicationProject;
import ru.entaxy.platform.integration.applications.EntaxyApplicationProjectVersion;
import ru.entaxy.platform.integration.applications.shell.ApplicationManagerCommandSupport;
import ru.entaxy.platform.integration.applications.shell.ApplicationShell;

@Service
@Command(name = "project-list", scope = ApplicationShell.COMMAND_SCOPE)
public class ProjectListCommand extends ApplicationManagerCommandSupport {

    @Override
    public Object execute() throws Exception {
        ShellTableFixed table = new ShellTableFixed();
        table.column("Name");
        table.column("Storage");
        table.column("Versions");

        List<EntaxyApplicationProject> apps = applicationManager.getProjects();
        Collections.sort(apps, new Comparator<EntaxyApplicationProject>() {

            @Override
            public int compare(EntaxyApplicationProject o1, EntaxyApplicationProject o2) {
                String storage1 = o1.getStorage().getName();
                String storage2 = o2.getStorage().getName();
                if (!storage1.equals(storage2))
                    return storage1.compareTo(storage2);

                return o1.getName().compareTo(o2.getName());
            }

        });

        for (EntaxyApplicationProject app : apps) {
            List<EntaxyApplicationProjectVersion> versions = app.getVersions();
            Collections.sort(versions, new Comparator<EntaxyApplicationProjectVersion>() {

                @Override
                public int compare(EntaxyApplicationProjectVersion o1, EntaxyApplicationProjectVersion o2) {
                    return o1.compareTo(o2);
                }

            });
            String versionsValue = versions.stream().map(v -> v.getVersionNumber()).collect(Collectors.joining("\n"));

            table.addRow().addContent(app.getName(), app.getStorage() == null ? "" : app.getStorage().getName(),
                    versionsValue);
        }

        // print output
        table.print(System.out);

        return null;
    }

}
