/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.project;

import java.util.List;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.karaf.shell.ShellTableFixed;
import ru.entaxy.platform.integration.applications.ApplicationProjectItem;
import ru.entaxy.platform.integration.applications.EntaxyApplicationProject;
import ru.entaxy.platform.integration.applications.EntaxyApplicationProjectVersion;
import ru.entaxy.platform.integration.applications.item.project.EntaxyBundleItem;
import ru.entaxy.platform.integration.applications.item.project.EntaxyConfigItem;
import ru.entaxy.platform.integration.applications.item.project.EntaxyObjectItem;
import ru.entaxy.platform.integration.applications.item.project.EntaxyResourceItem;
import ru.entaxy.platform.integration.applications.shell.ApplicationShell;

@Service
@Command(name = "project-version-items", scope = ApplicationShell.COMMAND_SCOPE)
public class VersionItemListCommand extends ManagedVersionCommand {

    @Override
    protected void doExecute(EntaxyApplicationProject application, EntaxyApplicationProjectVersion version) {
        List<ApplicationProjectItem> items = version.getItems();

        ShellTableFixed table = new ShellTableFixed();
        table.column("Status").colorProvider(ApplicationShell.itemStatusColorProvider).alignCenter();
        table.column("Id").colorProvider(ApplicationShell.normalColorProvider);
        table.column("Type").colorProvider(ApplicationShell.normalColorProvider);
        table.column("Scope").colorProvider(ApplicationShell.normalColorProvider);
        table.column("Required").colorProvider(ApplicationShell.normalColorProvider);
        table.column("Ignored").colorProvider(ApplicationShell.normalColorProvider);

        for (ApplicationProjectItem item : items) {
            String status = "+";
            if (item.isGhost())
                status = "?";
            if (item.isRequired())
                if (item.isIgnored())
                    status = "-";
                else
                    status = "!";

            String type = EntaxyConfigItem.class.isInstance(item) ? "config"
                    : EntaxyObjectItem.class.isInstance(item) ? "object"
                            : EntaxyBundleItem.class.isInstance(item) ? "bundle"
                                    : EntaxyResourceItem.class.isInstance(item) ? "resource" : "unknown";

            table.addRow().addContent(status, item.getId(), type,
                    item.getScope().name(),
                    item.isRequired() ? "*" : "",
                    item.isIgnored() ? "*" : "");


        }

        table.print(System.out);

    }

}
