/*-
 * ~~~~~~licensing~~~~~~
 * application-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.shell.project;

import org.apache.karaf.shell.api.action.Argument;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Completion;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.integration.applications.EntaxyApplicationProject;
import ru.entaxy.platform.integration.applications.EntaxyApplicationProjectVersion;
import ru.entaxy.platform.integration.applications.shell.ApplicationShell;
import ru.entaxy.platform.integration.applications.shell.completers.ProjectVersionCompleter;

@Service
@Command(name = "project-version-manage", scope = ApplicationShell.COMMAND_SCOPE)
public class VersionManageCommand extends ManagedProjectCommand {

    @Argument(required = true)
    @Completion(value = ProjectVersionCompleter.class)
    String versionNumber;

    @Override
    protected void doExecute(EntaxyApplicationProject application) {
        if (session.get(ApplicationShell.PROJECT_VERSION_KEY) != null) {
            // print error
            System.err.println("Another version is being managed, please cancel or commit");
            return;
        }

        EntaxyApplicationProjectVersion version = application.getVersion(versionNumber);
        if (version == null) {
            System.err.println("Version not found");
            return;
        }

        session.put(ApplicationShell.PROJECT_VERSION_KEY, version);

    }

}
