/*-
 * ~~~~~~licensing~~~~~~
 * resource-based-storage
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.storage.resource.impl;

import java.util.List;

public abstract class AbstractResourceComponentWithChildren<T, C extends AbstractResourceComponent<?>>
        extends AbstractResourceComponent<T> {


    final protected ComponentCache<C> childrenCache = new ComponentCache<>();

    final protected Object childrenCacheLock = new Object();

    /*
    protected AbstractResourceComponentWithChildren(EntaxyResourceService entaxyResourceService) {
        super(entaxyResourceService);
    }
    */

    protected AbstractResourceComponentWithChildren(ResourceContext resourceContext) {
        super(resourceContext);
    }

    @Override
    public void init() {
        super.init();
        initChildrenCache();
    }

    protected void initChildrenCache() {
        synchronized (childrenCacheLock) {
            for (String key : getChildrenKeyList()) {
                try {
                    C component = loadChild(key);
                    if (component == null)
                        continue;
                    childrenCache.put(component);
                } catch (Exception e) {
                    handleChildLoadException(key, e);
                }
            }
        }
    }

    protected C loadChild(String key) throws Exception {
        C component = createChildComponent(key);
        if (component == null)
            return null;
        component.init();
        return component;
    }

    protected C createNewChild(String key) throws Exception {
        C component = createChildComponent(key);
        component.init();
        synchronized (childrenCacheLock) {
            addChildKey(key);
            updateRootResource();
            childrenCache.put(component);
        }
        return component;
    }

    protected void removeChild(String key) throws Exception {
        synchronized (childrenCacheLock) {
            C component = childrenCache.get(key);
            if (component == null)
                return;
            component.delete();
            childrenCache.remove(key);
            removeChildKey(key);
            updateRootResource();
        }
    }

    protected abstract C createChildComponent(String key) throws Exception;

    protected abstract List<String> getChildrenKeyList();

    protected abstract void addChildKey(String key);

    protected abstract void removeChildKey(String key);

    protected abstract void handleChildLoadException(String key, Exception e);
}
