/*-
 * ~~~~~~licensing~~~~~~
 * resource-based-storage
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.storage.resource.impl.application;

import java.util.Calendar;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Optional;

public class ApplicationData {
    String name;

    Map<String, String> versions;

    public ApplicationData(String name) {
        this.name = name;
    }

    public boolean hasVersion(String versionNumber) {
        return versions != null && versions.containsKey(versionNumber);
    }

    public void addVersion(String version) {
        addVersion(version, Calendar.getInstance().getTimeInMillis() + "");
    }

    public void addVersion(String version, String timestamp) {
        if (versions == null)
            versions = new HashMap<>();
        if (!versions.containsKey(version))
            versions.put(version, timestamp);
    }

    public void removeVersion(String version) {
        if (versions == null)
            return;
        versions.remove(version);
    }

    public String getLatestVersion() {
        Optional<Entry<String, String>> latest =
                versions.entrySet().stream().sorted(new Comparator<Map.Entry<String, String>>() {

                    @Override
                    public int compare(Entry<String, String> o1, Entry<String, String> o2) {
                        // reverse order
                        return -1 * o1.getValue().compareTo(o2.getValue());
                    }
                }).findFirst();
        if (!latest.isPresent())
            return null;
        return latest.get().getKey();
    }
}
