/*-
 * ~~~~~~licensing~~~~~~
 * resource-based-storage
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.integration.applications.storage.resource.impl.application;

import java.io.InputStream;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.platform.base.support.JSONUtils;
import ru.entaxy.platform.integration.applications.ApplicationResource;
import ru.entaxy.platform.integration.applications.EntaxyApplication;
import ru.entaxy.platform.integration.applications.descriptor.ApplicationComponent;
import ru.entaxy.platform.integration.applications.descriptor.ApplicationDescriptor;
import ru.entaxy.platform.integration.applications.impl.StorageApplicationContentAdapter;
import ru.entaxy.platform.integration.applications.impl.StorageItemAdapter;
import ru.entaxy.platform.integration.applications.impl.application.ApplicationContentImpl;
import ru.entaxy.platform.integration.applications.impl.application.ApplicationItemImpl;
import ru.entaxy.platform.integration.applications.storage.resource.impl.ResourceContext;

public class ResourceApplicationContent extends ApplicationContentImpl implements StorageApplicationContentAdapter {

    private static final Logger LOG = LoggerFactory.getLogger(ResourceApplicationContent.class);

    private static Gson GSON = new GsonBuilder().setPrettyPrinting().create();

    protected boolean isDirty = false;

    public static ResourceApplicationContent fromContext(ResourceContext resourceContext) {

        // check descriptor
        ApplicationDescriptor applicationDescriptor = null;
        EntaxyResource descriptorResource = resourceContext.getResource(EntaxyApplication.DESCRIPTOR_NAME);
        if (descriptorResource.exists()) {
            // try (InputStream is = descriptorResource.getInputStream()) {
            try {
                applicationDescriptor = GSON.fromJson(JSONUtils.getJsonRootObject(descriptorResource.getAsString()),
                        ApplicationDescriptor.class);
            } catch (Exception ignore) {
                LOG.error("Error reading descriptor [" + descriptorResource.getURL() + "]", ignore);
            }
        }

        if (applicationDescriptor == null)
            return new ResourceApplicationContent(new ApplicationDescriptor(), resourceContext);

        ResourceApplicationContent result = new ResourceApplicationContent(applicationDescriptor, resourceContext);
        try {
            result.readDescriptor();
        } catch (Exception ignore) {
            LOG.error("Error processing descriptor [" + descriptorResource.getURL() + "]", ignore);
            return new ResourceApplicationContent(new ApplicationDescriptor(), resourceContext);
        }
        return result;


    }

    protected ResourceContext resourceContext;

    public ResourceApplicationContent(ApplicationDescriptor descriptor, ResourceContext resourceContext) {
        super(descriptor);
        this.resourceContext = resourceContext;
    }

    public void readDescriptor() throws Exception {
        createItem(descriptor.features);

        for (ApplicationComponent comp : descriptor.getComponents())
            createItem(comp);
    }

    public ResourceContext getResourceContext() {
        return resourceContext;
    }

    public boolean isDirty() {
        return isDirty;
    }

    protected void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    public void save() {
        setDirty(false);
    }

    @Override
    public StorageItemAdapter createItem(ApplicationComponent applicationComponent) throws Exception {
        ResourceApplicationItem item = new ResourceApplicationItem(applicationComponent, resourceContext);
        this.items.put(item.getLocation(), item);
        setDirty(true);
        return item;
    }

    @Override
    public ApplicationResource createResource(String location, InputStream inputStream) throws Exception {
        ResourceApplicationResource res = new ResourceApplicationResource(resourceContext, location);
        res.write(inputStream);
        this.resources.put(location, res);
        setDirty(true);
        return res;
    }

    @Override
    public StorageItemAdapter getItemAdapter(String location) throws Exception {
        ApplicationItemImpl item = this.items.get(location);
        if (item instanceof ResourceApplicationItem)
            return (ResourceApplicationItem) item;
        return null;
    }

}
