/*-
 * ~~~~~~licensing~~~~~~
 * base-support
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.base.support.osgi.feature;

import java.util.List;

import org.apache.karaf.features.Capability;
import org.apache.karaf.features.Feature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.osgi.bundle.CapabilityDescriptorImpl;
import ru.entaxy.platform.base.support.osgi.bundle.CapabilityHelper;

public class FeatureCapabilityHelper extends CapabilityHelper {

    private static final Logger LOG = LoggerFactory.getLogger(FeatureCapabilityHelper.class);

    protected Feature feature;

    public FeatureCapabilityHelper(Feature feature) {
        super();
        setMultipleNamespacesSupported(true);
        this.feature = feature;
        load();
    }

    @Override
    protected void load() {
        super.load();
        List<? extends Capability> featureCapabilities = feature.getCapabilities();
        for (Capability c : featureCapabilities) {
            String[] caps = c.getValue().split(",");
            for (String cap : caps)
                try {
                    CapabilityDescriptorImpl descriptor = parseCapability(cap);
                    if (descriptor != null)
                        addProvidedCapability(descriptor);
                } catch (Exception e) {
                    LOG.error(String.format("Error parsing capability [%s] for feature [%s/%s]", cap, feature.getName(),
                            feature.getVersion()), e);
                }
        }
    }

}
