/*-
 * ~~~~~~licensing~~~~~~
 * cellar-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.runtime.cellar.helper;

import java.util.Collections;
import java.util.Map;

import org.apache.karaf.cellar.core.ClusterManager;
import org.apache.karaf.cellar.core.GroupManager;
import org.apache.karaf.cellar.core.event.EventProducer;
import org.apache.karaf.cellar.features.management.CellarFeaturesMBean;
import org.apache.karaf.features.FeaturesService;
import org.osgi.service.cm.ConfigurationAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HelperFactory {

    private static final Logger LOG = LoggerFactory.getLogger(HelperFactory.class);

    public static <T> T createHelper(Class<T> targetClass) {

        return createHelper(targetClass, Collections.emptyMap());

    }

    public static <T> T createHelper(Class<T> targetClass, Map<Object, Object> parameters) {

        if (CellarFeaturesMBean.class.equals(targetClass))
            return (T) createFeaturesHelper(parameters);

        return null;

    }


    protected static CellarFeaturesHelper createFeaturesHelper(Map<Object, Object> parameters) {
        CellarFeaturesHelper result = new CellarFeaturesHelper();

        if (parameters.containsKey(ClusterManager.class.getName()))
            result.setClusterManager((ClusterManager) parameters.get(ClusterManager.class.getName()));
        else if (Services.INSTANCE != null)
            result.setClusterManager(Services.INSTANCE.clusterManager);

        if (parameters.containsKey(ConfigurationAdmin.class.getName()))
            result.setConfigurationAdmin((ConfigurationAdmin) parameters.get(ConfigurationAdmin.class.getName()));
        else if (Services.INSTANCE != null)
            result.setConfigurationAdmin(Services.INSTANCE.configurationAdmin);

        if (parameters.containsKey(EventProducer.class.getName()))
            result.setEventProducer((EventProducer) parameters.get(EventProducer.class.getName()));
        else if (Services.INSTANCE != null)
            result.setEventProducer(Services.INSTANCE.eventProducer);

        if (parameters.containsKey(FeaturesService.class.getName()))
            result.setFeaturesService((FeaturesService) parameters.get(FeaturesService.class.getName()));
        else if (Services.INSTANCE != null)
            result.setFeaturesService(Services.INSTANCE.featuresService);

        if (parameters.containsKey(GroupManager.class.getName()))
            result.setGroupManager((GroupManager) parameters.get(GroupManager.class.getName()));
        else if (Services.INSTANCE != null)
            result.setGroupManager(Services.INSTANCE.groupManager);

        return result;
    }

}
