/*-
 * ~~~~~~licensing~~~~~~
 * cellar-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.runtime.cellar.sequence.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.karaf.cellar.core.event.EventProducer;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;

import ru.entaxy.platform.runtime.cellar.helper.HelperFactory;
import ru.entaxy.platform.runtime.cellar.sequence.CellarSequenceManager;
import ru.entaxy.platform.runtime.cellar.sequence.SequenceBuilder;

@Component(service = CellarSequenceManager.class, immediate = true)
public class CellarSequenceManagerImpl implements CellarSequenceManager {

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC)
    protected volatile EventProducer eventProducer;

    protected final Map<String, SequenceBuilder> sequences = new HashMap<>();
    protected Object sequencesLock = new Object();

    @Override
    public <T> T getSequencedHelper(String sequenceId, Class<T> targetClass) {

        Map<Object, Object> properties = new HashMap<>();
        properties.put(EventProducer.class.getName(), getSequenceBuilder(sequenceId));

        return HelperFactory.createHelper(targetClass, properties);

    }

    @Override
    public void produceSequence(String sequenceId) {
        synchronized (sequencesLock) {
            if (sequences.containsKey(sequenceId))
                eventProducer.produce(sequences.get(sequenceId).getEvent());
        }
    }

    @Override
    public void releaseSequence(String sequenceId) {
        synchronized (sequencesLock) {
            sequences.remove(sequenceId);
        }
    }

    @Override
    public SequenceBuilder getSequence(String sequenceId) {
        return getSequenceBuilder(sequenceId);
    }

    protected SequenceBuilder getSequenceBuilder(String sequenceId) {
        synchronized (sequencesLock) {
            if (!sequences.containsKey(sequenceId))
                sequences.put(sequenceId, new SequenceBuilder(sequenceId));
            return sequences.get(sequenceId);
        }
    }

}
