/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-audit
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.audit.service.hawtio;

import org.osgi.service.component.annotations.Component;

import ru.entaxy.audit.data.AuditEvent;
import ru.entaxy.audit.data.Outcome;
import ru.entaxy.audit.data.Severity;
import ru.entaxy.audit.service.EventConverter;
import ru.entaxy.audit.service.EventConverterInfo;
import ru.entaxy.audit.service.InterpretedEvent;

@Component(service = EventConverter.class, immediate = true)
@EventConverterInfo(classes = {HawtioInterpretedEvent.class})
public class HawtioEventConverter implements EventConverter {

    @Override
    public <T extends InterpretedEvent> AuditEvent convert(T event) {
        if (!(event instanceof HawtioInterpretedEvent))
            return null;

        HawtioInterpretedEvent hawtioEvent = (HawtioInterpretedEvent) event;

        AuditEvent result = AuditEvent.AuditLoggingEventBuilder.getInstance()
                .target("auth")
                .outcome(hawtioEvent.isSuccessful ? Outcome.SUCCESS : Outcome.FAILURE)
                .message(String.format("%s", hawtioEvent.getMessage()))
                .severity(Severity.IMPORTANT)
                .category(hawtioEvent.getAction())
                .suser(hawtioEvent.getUser())
                .src(hawtioEvent.getSrc())
                .build();

        return result;

    }

}
