/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-audit
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.audit.service.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.audit.data.AuditEvent;
import ru.entaxy.audit.service.EventConverter;
import ru.entaxy.audit.service.InterpretedEvent;

@Component(service = ConverterService.class, immediate = true)
public class ConverterService {

    static public ConverterService INSTANCE = null;

    protected Map<Class<? extends InterpretedEvent>, Set<EventConverter>> converters = new HashMap<>();

    protected Object convertersLock = new Object();

    @Activate
    public void activate() {
        INSTANCE = this;
    }

    @Deactivate
    public void deactivate() {
        INSTANCE = null;
    }

    @Reference(cardinality = ReferenceCardinality.MULTIPLE, policyOption = ReferencePolicyOption.GREEDY,
            policy = ReferencePolicy.DYNAMIC, unbind = "removeConverter")
    public void addInterpreter(EventConverter converter) {
        synchronized (convertersLock) {
            for (Class<? extends InterpretedEvent> clazz : converter.getAllowedClasses()) {
                this.converters.putIfAbsent(clazz, new HashSet<>());
                this.converters.get(clazz).add(converter);
            }
        }
    }

    public void removeConverter(EventConverter converter) {
        synchronized (convertersLock) {
            for (Class<? extends InterpretedEvent> clazz : converter.getAllowedClasses()) {
                if (converters.containsKey(clazz))
                    converters.get(clazz).remove(converter);
            }
        }

    }

    public <T extends InterpretedEvent> AuditEvent convert(T event) {
        AuditEvent result = null;
        if (converters.containsKey(event.getClass())) {
            for (EventConverter converter : converters.get(event.getClass())) {
                try {
                    result = converter.convert(event);
                } catch (Exception ignore) {
                    // NOOP
                }
                if (result != null)
                    break;
            }

        }
        return result;
    }
}
