/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-audit
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.audit.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.osgi.service.event.Event;

import ru.entaxy.audit.service.EventInterpreter;
import ru.entaxy.audit.service.InterpretedEvent;

@Component(service = InterpreterService.class, immediate = true)
public class InterpreterService {

    static public InterpreterService INSTANCE = null;

    protected List<EventInterpreter> interpreters = new ArrayList<>();

    protected Object interpretersLock = new Object();

    @Activate
    public void activate() {
        INSTANCE = this;
    }

    @Deactivate
    public void deactivate() {
        INSTANCE = null;
    }

    @Reference(cardinality = ReferenceCardinality.MULTIPLE, policyOption = ReferencePolicyOption.GREEDY,
            policy = ReferencePolicy.DYNAMIC, unbind = "removeInterpreter")
    public void addInterpreter(EventInterpreter interpreter) {
        synchronized (interpretersLock) {
            if (!interpreters.contains(interpreter))
                interpreters.add(interpreter);
        }
    }

    public void removeInterpreter(EventInterpreter interpreter) {
        synchronized (interpretersLock) {
            interpreters.remove(interpreter);
        }

    }

    public InterpretedEvent interpret(Event event) {
        InterpretedEvent result = null;
        for (EventInterpreter interpreter : interpreters) {
            try {
                result = interpreter.interpret(event);
            } catch (Exception ignore) {
                // NOOP
            }
            if (result != null)
                break;
        }
        return result;
    }
}
