/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-audit
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.audit.service.jmx;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.management.ObjectName;

import org.osgi.service.event.Event;

public class JMXInvokeEvent extends JMXInterpretedEvent {

    public static final String PROP_PARAMS = "params";

    protected ObjectName objectName;

    protected String methodName;

    protected Object[] methodParams;

    protected Map<String, Object> objectNameData = new HashMap<>();

    public JMXInvokeEvent(Event origin) {
        super(origin);
        if (this.originalEvent.containsProperty(PROP_PARAMS)) {

            Object[] data = (Object[]) this.originalEvent.getProperty(PROP_PARAMS);

            setObjectName((ObjectName) data[0]);
            setMethodName((String) data[1]);
            setMethodParams((Object[]) data[2]);

        }
    }

    protected void parseObjectName() {
        this.objectNameData.clear();
        this.objectNameData.put("jmx.domain", this.objectName.getDomain());
        Hashtable<String, String> properties = this.objectName.getKeyPropertyList();
        for (Map.Entry<String, String> entry : properties.entrySet()) {
            this.objectNameData.put("jmx.property." + entry.getKey(), entry.getValue());
        }

    }

    public ObjectName getObjectName() {
        return objectName;
    }

    public void setObjectName(ObjectName objectName) {
        this.objectName = objectName;
        parseObjectName();
    }

    public String getMethodName() {
        return methodName;
    }

    public void setMethodName(String methodName) {
        this.methodName = methodName;
        this.eventEssense.put("jmx.methodName", methodName);
    }

    public Object[] getMethodParams() {
        return methodParams;
    }

    public void setMethodParams(Object[] methodParams) {
        this.methodParams = methodParams;
    }

    @Override
    public Map<String, Object> getEventEssense() {
        Map<String, Object> result = new HashMap<>(super.getEventEssense());
        result.putAll(this.objectNameData);
        return result;
    }

}
