/*-
 * ~~~~~~licensing~~~~~~
 * search-service
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.search;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.osgi.framework.Filter;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.InvalidSyntaxException;

public interface SearchService {

    List<SearchProvider> getProviders();

    SearchProvider getProviderForType(String targetType);

    default List<String> getProvidedTypes() {
        return getProviders().stream().map(p -> p.getTargetTypes()).flatMap(t -> t.stream()).distinct()
                .collect(Collectors.toList());
    }

    default Map<String, String> getStandardAttributes(String targetType) throws IllegalArgumentException {
        SearchProvider provider = getProviderForType(targetType);
        if (provider == null)
            throw new IllegalArgumentException(String.format("Search provider for type [%s] not found", targetType));
        return provider.getStandardAttributes(targetType);
    }

    default List<String> getAttributes(String targetType, String filter)
            throws InvalidSyntaxException, IllegalArgumentException {
        return getAttributes(targetType, FrameworkUtil.createFilter(filter));
    }

    default List<String> getAttributes(String targetType, Filter filter)
            throws InvalidSyntaxException, IllegalArgumentException {

        List<String> result = new ArrayList<>();

        List<Map<String, Object>> found = search(targetType, filter);

        if (found != null) {
            result.addAll(found.stream().map(m -> m.keySet()).flatMap(ks -> ks.stream()).distinct()
                    .collect(Collectors.toList()));
        }

        return result;
    }

    default List<Map<String, Object>> search(String targetType, String filter)
            throws InvalidSyntaxException, IllegalArgumentException {
        Filter osgiFilter = FrameworkUtil.createFilter(filter);
        return search(targetType, osgiFilter);
    }


    default List<Map<String, Object>> search(String targetType, Filter filter)
            throws InvalidSyntaxException, IllegalArgumentException {
        SearchProvider provider = getProviderForType(targetType);
        if (provider == null)
            throw new IllegalArgumentException(String.format("Search provider for type [%s] not found", targetType));
        return provider.search(targetType, filter);
    };
}
