/*-
 * ~~~~~~licensing~~~~~~
 * search-service
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.search.jmx.impl;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;

import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ServiceScope;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.platform.search.SearchService;
import ru.entaxy.platform.search.jmx.SearchMBean;

@Component(
        service = {SearchMBean.class, DynamicMBean.class, MBeanRegistration.class},
        property = {ManagementCore.JMX_OBJECTNAME + "=" + SearchMBean.JMX_OBJECT_NAME},
        scope = ServiceScope.SINGLETON,
        immediate = true)
public class SearchMBeanImpl extends AnnotatedMBean<SearchMBean> implements SearchMBean {

    @Reference
    SearchService searchService;

    public SearchMBeanImpl() throws NotCompliantMBeanException {
        super(SearchMBean.class);
    }

    @Override
    public List<String> listProviders() {
        return searchService.getProviders().stream().map(p -> p.getProviderId()).collect(Collectors.toList());
    }

    @Override
    public List<String> listProvidedTypes() {
        return searchService.getProvidedTypes();
    }

    @Override
    public Map<String, String> listStandardAttributes(String type)
            throws InvalidSyntaxException, IllegalArgumentException {
        return searchService.getStandardAttributes(type);
    }

    @Override
    public List<Map<String, Object>> search(String type, String filter)
            throws InvalidSyntaxException, IllegalArgumentException {
        return searchService.search(type, FrameworkUtil.createFilter(filter));
    }

}
