/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.ext;

import ru.entaxy.platform.core.artifact.Artifact;
import ru.entaxy.platform.core.artifact.ext.binary.UntypedBinaryInstaller;
import ru.entaxy.platform.core.artifact.ext.binary.impl.UntypedBinaryInstallerImpl;
import ru.entaxy.platform.core.artifact.ext.features.FeatureInstaller;
import ru.entaxy.platform.core.artifact.ext.features.FeaturesInstaller;
import ru.entaxy.platform.core.artifact.ext.features.impl.FeatureInstallerImpl;
import ru.entaxy.platform.core.artifact.ext.features.impl.FeaturesInstallerImpl;
import ru.entaxy.platform.core.artifact.installer.builder.typed.AbstractTypedInstallerFactory;
import ru.entaxy.platform.core.artifact.installer.builder.typed.TypedInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.typed.TypedInstallerFactoryDescriptor;

@TypedInstallerFactoryDescriptor(
        supportedClasses = {FeaturesInstaller.class, FeatureInstaller.class, UntypedBinaryInstaller.class},
        supportedTypes = {Artifact.ARTIFACT_CATEGORY_FEATURES, ArtifactExtended.ARTIFACT_CATEGORY_FEATURE,
            ArtifactExtended.ARTIFACT_CATEGORY_UNTYPED_BINARY})
public class ExtTypedInstallerFactory extends AbstractTypedInstallerFactory {

    @Override
    protected TypedInstaller doCreate(String type) {
        if (Artifact.ARTIFACT_CATEGORY_FEATURES.equalsIgnoreCase(type)) {
            return new FeaturesInstallerImpl();
        }
        if (ArtifactExtended.ARTIFACT_CATEGORY_UNTYPED_BINARY.equalsIgnoreCase(type)) {
            return new UntypedBinaryInstallerImpl();
        }
        return null;
    }

    @Override
    protected TypedInstaller doCreate(Class<? extends TypedInstaller> targetClass) {
        if (FeaturesInstaller.class.equals(targetClass))
            return new FeaturesInstallerImpl();
        if (FeatureInstaller.class.equals(targetClass))
            return new FeatureInstallerImpl();
        if (UntypedBinaryInstaller.class.equals(targetClass))
            return new UntypedBinaryInstallerImpl();
        return null;
    }

}
