/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.ext.features;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;

import ru.entaxy.platform.base.support.xml.CommonXMLUtils;
import ru.entaxy.platform.core.artifact.Artifact;
import ru.entaxy.platform.core.artifact.annotation.ArtifactSupport;
import ru.entaxy.platform.core.artifact.ext.binary.AbstractBinaryArtifact;

@ArtifactSupport(supportedCategory = Artifact.ARTIFACT_CATEGORY_FEATURES,
        supportedContentClasses = {File.class, InputStream.class, byte[].class, Document.class, String.class},
        defaultArtifactType = "xml")
public class FeaturesArtifact extends AbstractBinaryArtifact {

    private static final Logger LOG = LoggerFactory.getLogger(FeaturesArtifact.class);

    @Override
    public void toFile(File destination) {
        if (content == null)
            return;

        if (content instanceof Document) {
            String data = null;
            try {
                data = CommonXMLUtils.doc2string((Document) content);
            } catch (Exception e) {
                LOG.error("Error converting Document to String for artifact [" + getCoordinates().toMavenString() + "]",
                        e);
                return;
            }
            try (InputStream is = new ByteArrayInputStream(data.getBytes())) {
                Files.copy(is, destination.getAbsoluteFile().toPath(),
                        StandardCopyOption.REPLACE_EXISTING);
            } catch (Exception e) {
                LOG.error("Error saving to file for artifact [" + getCoordinates().toMavenString() + "]", e);
                return;
            }
            return;
        }

        super.toFile(destination);
    }

}
