/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.ext.features.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.karaf.cellar.features.management.CellarFeaturesMBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.base.support.osgi.OSGIUtils;
import ru.entaxy.platform.core.artifact.installer.builder.impl.InstallationResultImpl;
import ru.entaxy.platform.core.artifact.installer.builder.impl.cluster.ClusterTypedInstallerHelper;
import ru.entaxy.platform.runtime.cellar.sequence.CellarSequenceManager;

public class CellarFeaturesInstallerHelper extends ClusterTypedInstallerHelper
        implements FeaturesInstallerConfigurableHelper {

    private static final Logger LOG = LoggerFactory.getLogger(CellarFeaturesInstallerHelper.class);

    protected FeaturesInstallerHelperConfig config;

    public FeaturesInstallerHelperConfig getConfig() {
        return config;
    }

    @Override
    public void setConfig(FeaturesInstallerHelperConfig config) {
        this.config = config;
    }

    protected CellarFeaturesMBean getFeaturesService() throws Exception {
        if (CommonUtils.isValid(config.getSequenceId())) {
            CellarSequenceManager seqManager =
                    OSGIUtils.services().ofClass(CellarSequenceManager.class).waitService(2000).get();

            return seqManager.getSequencedHelper(config.getSequenceId(), CellarFeaturesMBean.class);

        }
        return OSGIUtils.services().ofClass(CellarFeaturesMBean.class).waitService(2000).get();
    }

    @Override
    public InstallationResultImpl install(InstallationResultImpl installationResult) {
        CellarFeaturesMBean featuresService;
        try {
            featuresService = getFeaturesService();
        } catch (Exception e) {
            LOG.error("Error getting CellarFeaturesMBean service", e);
            return installationResult.failed("Error getting CellarFeaturesMBean service").error(e);
        }

        List<String> groupsToInstall = new ArrayList<>();
        for (String groupName : getGroups())
            if (checkGroup(groupName))
                if (!groupsToInstall.contains(groupName))
                    groupsToInstall.add(groupName);

        if (groupsToInstall.size() == 0) {
            return installationResult.failed("No groups to install to");
        }

        // TODO decide if we really need several groups
        // now we use only the first one
        String groupToInstall = groupsToInstall.get(0);

        String repositoryUrl = config.getSourceLocation();
        if (repositoryUrl.indexOf(':') < 0)
            repositoryUrl = "mvn:" + repositoryUrl;

        try {
            if (featuresService.getRepositories(groupToInstall).contains(repositoryUrl)) {
                if (config.isRefresh())
                    featuresService.refreshRepository(groupToInstall, repositoryUrl);
            } else {
                featuresService.addRepository(groupToInstall, repositoryUrl, null, config.isInstallAllFetures());
            }

            if (!config.isInstallAllFetures())
                for (String feature : config.getFeaturesToInstall())
                    featuresService.installFeature(groupToInstall, feature);

            return installationResult.installed();
        } catch (Exception e) {
            LOG.error("Error installing repository [" + repositoryUrl + "]", e);
            return installationResult.failed("Error installing repository [" + repositoryUrl + "]").error(e);
        }
    }

    @Override
    public InstallationResultImpl uninstall(InstallationResultImpl installationResult) {
        // TODO Auto-generated method stub
        return null;
    }

}
