/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.ext.features.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import ru.entaxy.platform.core.artifact.ext.features.FeaturesInstaller;
import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;
import ru.entaxy.platform.core.artifact.installer.builder.impl.InstallationResultImpl;
import ru.entaxy.platform.core.artifact.installer.builder.impl.cluster.ClusterTypedInstallerHelper;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.HelperCreator;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.TypedInstallerHelperImpl;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.TypedInstallerImpl;

public class FeaturesInstallerImpl extends TypedInstallerImpl
        implements FeaturesInstaller, HelperCreator, FeaturesInstallerHelperConfig {

    protected boolean isInstallAllFeatures = false;

    protected boolean isRefresh = false;

    protected List<String> featuresToInstall = new ArrayList<>();

    protected FeaturesInstallerConfigurableHelper featuresHelper;

    protected String sequenceId = null;

    @Override
    public FeaturesInstaller inSequence(String sequenceId) {
        this.sequenceId = sequenceId;
        return this;
    }

    @Override
    public InstallationResult install() {
        InstallationResultImpl result = InstallationResultImpl.create();
        if (this.featuresHelper == null)
            result.failed("HELPER NOT FOUND");
        else
            this.featuresHelper.install(result);
        return result.property("artifact", this.artifact);
    }

    @Override
    public InstallationResult uninstall() {
        InstallationResultImpl result = InstallationResultImpl.create();
        if (this.featuresHelper == null)
            result.failed("HELPER NOT FOUND");
        else
            this.featuresHelper.uninstall(result);
        return result.property("artifact", this.artifact);
    }

    @Override
    public void setHelper(TypedInstallerHelperImpl helper) {
        if (helper instanceof FeaturesInstallerConfigurableHelper) {
            featuresHelper = (FeaturesInstallerConfigurableHelper) helper;
            featuresHelper.setConfig(this);
        }
        super.setHelper(helper);
    }

    @Override
    public FeaturesInstaller installAllFeatures() {
        isInstallAllFeatures = true;
        return this;
    }

    @Override
    public FeaturesInstaller installFeatures(String... features) {
        if (features != null)
            this.featuresToInstall = new ArrayList<>(Arrays.asList(features));
        return this;
    }

    @Override
    public <H extends TypedInstallerHelperImpl> H createHelper(Class<H> targetClass) {
        if (ClusterTypedInstallerHelper.class.isAssignableFrom(targetClass))
            return (H) new CellarFeaturesInstallerHelper();
        return (H) new LocalFeaturesInstallerHelper();
    }

    @Override
    public FeaturesInstaller refresh() {
        isRefresh = true;
        return this;
    }

    @Override
    public boolean isRefresh() {
        return isRefresh;
    }

    @Override
    public boolean isInstallAllFetures() {
        return isInstallAllFeatures;
    }

    @Override
    public List<String> getFeaturesToInstall() {
        return featuresToInstall;
    }

    @Override
    public String getSequenceId() {
        return sequenceId;
    }
}
