/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management-extensions
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.ext.impl;

import ru.entaxy.platform.core.artifact.installer.builder.InstallationResult;
import ru.entaxy.platform.core.artifact.installer.builder.impl.InstallationResultImpl;
import ru.entaxy.platform.core.artifact.installer.builder.impl.cluster.ClusterTypedInstallerHelper;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.HelperCreator;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.TypedInstallerHelperImpl;
import ru.entaxy.platform.core.artifact.installer.builder.typed.impl.TypedInstallerImpl;

public abstract class ExtendedTypedInstallerImpl<S extends InstallerConfigurableHelper<T>, T> extends TypedInstallerImpl
        implements HelperCreator {

    protected S localTypedHelper;

    protected abstract Class<S> getHelperClass();

    @Override
    public InstallationResult install() {
        InstallationResultImpl result = InstallationResultImpl.create();
        if (localTypedHelper == null)
            result.failed("HELPER NOT FOUND");
        else
            result = localTypedHelper.install(result);
        return result;
    }

    @Override
    public InstallationResult uninstall() {
        InstallationResultImpl result = InstallationResultImpl.create();
        if (localTypedHelper == null)
            result.failed("HELPER NOT FOUND");
        else
            result = localTypedHelper.uninstall(result);
        return result;
    }

    @Override
    public void setHelper(TypedInstallerHelperImpl helper) {
        if (getHelperClass().isInstance(helper)) {
            this.localTypedHelper = (S) helper;
            if (localTypedHelper.getConfigClass().isAssignableFrom(getClass()))
                localTypedHelper.setConfig((T) this);
        }
        super.setHelper(helper);
    }

    @Override
    public <H extends TypedInstallerHelperImpl> H createHelper(Class<H> targetClass) {
        if (ClusterTypedInstallerHelper.class.isAssignableFrom(targetClass))
            return (H) createClusterHelper(targetClass);
        return (H) createLocalHelper(targetClass);
    }

    protected abstract <H extends TypedInstallerHelperImpl> H createClusterHelper(Class<H> targetClass);

    protected abstract <H extends TypedInstallerHelperImpl> H createLocalHelper(Class<H> targetClass);
}
