/*-
 * ~~~~~~licensing~~~~~~
 * artifact-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.artifact.installer.builder.typed;

import java.util.Arrays;
import java.util.List;

public abstract class AbstractTypedInstallerFactory implements TypedInstallerFactory {

    protected List<Class<? extends TypedInstaller>> supportedClasses;

    protected List<String> supportedTypes;

    protected AbstractTypedInstallerFactory() {
        super();
        if (this.getClass().isAnnotationPresent(TypedInstallerFactoryDescriptor.class)) {
            TypedInstallerFactoryDescriptor descriptor =
                    this.getClass().getAnnotation(TypedInstallerFactoryDescriptor.class);
            this.supportedClasses = Arrays.asList(descriptor.supportedClasses());
            this.supportedTypes = Arrays.asList(descriptor.supportedTypes());
        }
    }

    @Override
    public List<Class<? extends TypedInstaller>> getSupportedClasses() {
        return supportedClasses;
    }

    @Override
    public List<String> getSupportedTypes() {
        return supportedTypes;
    }

    @Override
    public TypedInstaller create(String type) {
        if (!supportedTypes.contains(type))
            return null;
        return doCreate(type);
    }

    protected abstract TypedInstaller doCreate(String type);

    @Override
    public TypedInstaller create(Class<? extends TypedInstaller> targetClass) {
        if (!supportedClasses.contains(targetClass))
            return null;
        return doCreate(targetClass);
    }

    protected abstract TypedInstaller doCreate(Class<? extends TypedInstaller> targetClass);
}
