/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime.camel.aggregation;

import org.apache.camel.Exchange;
import org.apache.camel.component.jms.JmsMessage;
import org.apache.camel.processor.aggregate.AggregateExtension;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.jms.JMSException;
import javax.jms.Session;

public class JmsCamelAggregateExtension implements AggregateExtension {

    private static final Log LOG = LogFactory.getLog(JmsCamelAggregateExtension.class);

    @Override
    public boolean checkExtension(Exchange exchange) {
        return exchange instanceof JmsMessage;
    }

    @Override
    public Exchange doExtension(Exchange exchange) {
        JmsMessage jms = exchange.getIn(JmsMessage.class);
        Session session = jms.getJmsSession();
        try {
            if (session != null && Session.CLIENT_ACKNOWLEDGE == session.getAcknowledgeMode()) {
                jms.getJmsMessage().acknowledge();
            }
        } catch (JMSException e) {
            LOG.error("Error in CLIENT_ACKNOWLEDGE.", e);
        }
        return exchange;
    }
}
