/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime.cluster;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;

import org.apache.karaf.cellar.bundle.BundleState;
import org.apache.karaf.cellar.bundle.Constants;
import org.apache.karaf.cellar.core.ClusterManager;
import org.apache.karaf.cellar.core.Configurations;
import org.apache.karaf.cellar.core.GroupManager;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@Component(service = ClusterBundleDataProvider.class, immediate = true)
public class ClusterBundleDataProvider {

    protected static ClusterBundleDataProvider INSTANCE = null;

    private static final Logger LOG = LoggerFactory.getLogger(ClusterBundleDataProvider.class);

    protected BundleContext bundleContext;

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile protected ClusterManager clusterManager;

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile protected GroupManager groupManager;

    @Activate
    public void activate(ComponentContext componentContext) {
        this.bundleContext = componentContext.getBundleContext();
        INSTANCE = this;
    }

    public List<String> getGroups(Bundle bundle) {

        List<String> result = new ArrayList<>();

        String symbolicName = bundle.getSymbolicName();
//      Изменено, потому что bundle.getVersion() съедает ведущие нули
        String version = bundle.getHeaders().get(org.osgi.framework.Constants.BUNDLE_VERSION);

        Thread.currentThread().setContextClassLoader(getClass().getClassLoader());

        try {
            Set<String> groupNames = groupManager.listGroupNames(groupManager.getNode());
            for (String groupName : groupNames) {
                @SuppressWarnings("unchecked")
                Map<String, BundleState> clusterBundles =
                        clusterManager.getMap(Constants.BUNDLE_MAP + Configurations.SEPARATOR + groupName);
                Optional<BundleState> bundleState = clusterBundles.values().stream()
                        .filter(state -> state.getSymbolicName().equals(symbolicName))
                        .filter(state -> state.getVersion().equals(version)).findFirst();
                if (bundleState.isPresent()) {
                    result.add(groupName);
                } else {
                    Optional<BundleState> bundleStateWithoutVersionFiltering = clusterBundles.values().stream()
                            .filter(state -> state.getSymbolicName().equals(symbolicName)).findFirst();
                    if (bundleStateWithoutVersionFiltering.isPresent()) {
                        result.add("?" + groupName);
                    }
                }
            }
        } catch (Exception e) {
            LOG.error(String.format("Error getting groups for bundle [%s/%s]", symbolicName, version), e);
        }
        return result;
    }

}
