/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime.hazelcast;

import org.apache.camel.CamelContext;
import org.apache.camel.ProducerTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.hazelcast.core.EntryEvent;
import com.hazelcast.map.listener.EntryExpiredListener;

public class HazelcastCallbackEntryExpiredListener implements EntryExpiredListener<String,Object> {
    
    private static final Logger LOG = LoggerFactory.getLogger(HazelcastCallbackEntryExpiredListener.class);
    
    private CamelContext camelContext;
    private ProducerTemplate template;
    private String callbackUrl;
    
    public HazelcastCallbackEntryExpiredListener(CamelContext camelContext, String callbackUrl) {
        super();
        this.camelContext = camelContext;
        this.callbackUrl = callbackUrl;
        
        template = camelContext.createProducerTemplate();
        
        LOG.debug("HazelcastCallbackEntryExpiredListener initialized for callback " + callbackUrl);
    }

    @Override
    public void entryExpired(EntryEvent<String, Object> event) {
        LOG.trace("HazelcastCallbackEntryExpiredListener initialized for callback " + event);
        template.asyncSendBody(callbackUrl, event);
    }

}
