/*-
 * ~~~~~~licensing~~~~~~
 * core-support-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.support.runtime.jgroups;

import org.apache.camel.util.ObjectHelper;
import org.jgroups.JChannel;
import org.jgroups.blocks.locking.LockService;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.support.runtime.service.ServiceHolderHelper;

@Component(service = JGroupsLockServiceIgniter.class, immediate = true)
public class JGroupsLockServiceIgniter extends ServiceHolderHelper<LockService> {
    private static final Logger LOG = LoggerFactory.getLogger(JGroupsLockServiceIgniter.class);

    private static final String DEFAULT_JGROUPS_CONFIG = "etc/jgroups.locking.xml";
    private static final String DEFAULT_JGROUPS_CLUSTERNAME = "jgroups-master";

    private String jgroupsConfig;
    private String jgroupsClusterName;

    private JChannel channel;
    private LockService lockService;

    private BundleContext bundleContext;

    @Activate
    public JGroupsLockServiceIgniter(BundleContext bundleContext) {
        this(DEFAULT_JGROUPS_CONFIG, DEFAULT_JGROUPS_CLUSTERNAME, bundleContext);
    }

    public JGroupsLockServiceIgniter(String jgroupsConfig, String jgroupsClusterName, BundleContext bundleContext) {
        this.jgroupsConfig = jgroupsConfig;
        this.jgroupsClusterName = jgroupsClusterName;
        this.bundleContext = bundleContext;
    }

    @Activate
    public void start() throws Exception {
        ObjectHelper.notNull(jgroupsConfig, "jgroupsConfig");
        ObjectHelper.notNull(jgroupsClusterName, "jgroupsClusterName");
        ObjectHelper.notNull(bundleContext, "bundleContext");
        init();
        publish();
        LOG.info("JGroups ServiceHolder published");
    }

    protected void init() throws Exception {
        channel = new JChannel(jgroupsConfig);
        lockService = new LockService(channel);
        channel.connect(jgroupsClusterName);
        LOG.info("JGroups LockService instance initialized");
    }

    public void stop() throws Exception {
        if (channel != null) {
            channel.disconnect();
            channel.close();
            channel = null;
        }
    }

    @Override
    public LockService getService() throws Exception {
        return lockService;
    }

    @Override
    protected Class<LockService> getGenericType() {
        return LockService.class;
    }

    @Override
    public BundleContext getBundleContext() {
        return bundleContext;
    }

    public void setBundleContext(BundleContext bundleContext) {
        this.bundleContext = bundleContext;
    }
}
