/*-
 * ~~~~~~licensing~~~~~~
 * init-manager
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.runtime.core.initializer.api;

import java.util.Dictionary;
import java.util.Hashtable;

import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.ServiceRegistration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractStandaloneInitializer implements Initialized {

    private static final Logger log = LoggerFactory.getLogger(AbstractStandaloneInitializer.class);

    protected String initializerId;
    protected BundleContext bundleContext;
    protected ServiceRegistration<Initialized> serviceRegistration;

    protected AbstractStandaloneInitializer() {
        super();
        if (getClass().isAnnotationPresent(StandaloneInitializer.class)) {
            StandaloneInitializer anno = getClass().getAnnotation(StandaloneInitializer.class);
            this.initializerId = anno.id();
        }
    }

    public String getInitializerId() {
        return initializerId;
    }

    public void setInitializerId(String initializerId) {
        this.initializerId = initializerId;
    }

    public BundleContext getBundleContext() {
        return bundleContext;
    }

    public void setBundleContext(BundleContext bundleContext) {
        this.bundleContext = bundleContext;
    }

    protected BundleContext getCurrentBundleContext() {
        if (this.bundleContext != null)
            return bundleContext;
        return FrameworkUtil.getBundle(Initialized.class).getBundleContext();
    }

    protected void register() {
        try {
            Dictionary<String, Object> props = new Hashtable<>();
            props.put(PROP_INITIALIZER_ID, initializerId);
            serviceRegistration = getCurrentBundleContext().registerService(Initialized.class, this, props);
        } catch (Exception e) {
            log.error("Error registering service for initializer[" + initializerId + "]", e);
        }
    }

}
