/*-
 * ~~~~~~licensing~~~~~~
 * connection-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.connection.impl;

import javax.management.NotCompliantMBeanException;

import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Deactivate;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ServiceScope;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.Qualifier;
import ru.entaxy.platform.base.objects.EntaxyObjectService;
import ru.entaxy.platform.core.management.connection.ConnectionsMBean;
import ru.entaxy.platform.core.management.object.ContainerMBean;
import ru.entaxy.platform.core.management.object.factory.AbstractObjectContainerMBean;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectManager;

@Component(
        service = {ConnectionsMBeanImpl.class},
        scope = ServiceScope.SINGLETON,
        immediate = true)
@ContainerMBean(id = "connections", subQualifierName = "category", subQualifierValue = "connections")
public class ConnectionsMBeanImpl extends AbstractObjectContainerMBean<ConnectionsMBean>
        implements ConnectionsMBean {

    private static final Logger log = LoggerFactory.getLogger(ConnectionsMBeanImpl.class);

    public static final Qualifier Q_CONNECTIONS = ManagementCore.Q_RUNTIME
            .qualifier(ConnectionsMBean.CONNECTIONS_KEY, ConnectionsMBean.CONNECTIONS_KEY_VALUE);

    protected BundleContext bundleContext;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    EntaxyObjectService entaxyObjectService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    EntaxyRuntimeObjectManager entaxyRuntimeObjectManager;


    public ConnectionsMBeanImpl() throws NotCompliantMBeanException {
        super(ConnectionsMBean.class);
    }

    @Activate
    public void activate(ComponentContext componentContext) {
        this.bundleContext = componentContext.getBundleContext();
    }

    @Deactivate
    public void deactivate() {}


    @Override
    public String getConnectionConfig(String connectionName) throws Exception {
        return entaxyObjectService.findObject(connectionName, ConnectionsMBean.CONNECTION_TYPE).getConfiguration();
    }

    @Override
    public void startConnection(String idOrName) throws Exception {
        entaxyRuntimeObjectManager
                .start(entaxyObjectService.findObject(idOrName, ConnectionsMBean.CONNECTION_TYPE).getObjectFullId());
    }

    @Override
    public void stopConnection(String idOrName) throws Exception {
        entaxyRuntimeObjectManager
                .stopForced(entaxyObjectService.findObject(idOrName, ConnectionsMBean.CONNECTION_TYPE)
                        .getObjectFullId());
    }

    @Override
    public void uninstallConnection(String idOrName) throws Exception {
        entaxyRuntimeObjectManager
                .uninstallForced(entaxyObjectService.findObject(idOrName, ConnectionsMBean.CONNECTION_TYPE)
                        .getObjectFullId());
    }
}
