/*-
 * ~~~~~~licensing~~~~~~
 * object-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.object.factory.impl;

import org.apache.karaf.bundle.core.BundleInfo;
import org.apache.karaf.bundle.core.BundleService;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.FrameworkUtil;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.*;
import ru.entaxy.platform.base.objects.EntaxyObjectService;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectManager;

@Component(service = ServiceHelper.class, immediate = true)
public class ServiceHelper {

    private static ServiceHelper INSTANCE;

    public static ServiceHelper getInstance() {
        return INSTANCE;
    }

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile public EntaxyObjectService entaxyObjectService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile public EntaxyRuntimeObjectManager entaxyRuntimeObjectManager;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    BundleService bundleService;

    BundleContext bundleContext;

    @Activate
    public void activate(ComponentContext componentContext) {
        INSTANCE = this;
        this.bundleContext = componentContext.getBundleContext();
    }

    @Deactivate
    public void deactivate() {
        INSTANCE = null;
    }

    public BundleService getBundleService() {
        return bundleService;
    }

    public BundleInfo getBundleInfo(long id) {
        Bundle b = FrameworkUtil.getBundle(getClass()).getBundleContext().getBundle(id);
        if (b == null)
            return null;
        return bundleService.getInfo(b);
    }
}
