/*-
 * ~~~~~~licensing~~~~~~
 * object-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.object.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;

import ru.entaxy.platform.core.management.object.EntaxyRuntimeItemMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBeanFactory;

public class EntaxyRuntimeItemMBeanInfo<T extends EntaxyRuntimeItemMBean> {

    public enum ItemStatus {
        NEW,
        ACTIVE,
        REREGISTER,
        RECREATE,
        REMOVE,
        UNKNOWN
    }

    public MBeanHolder<T> mbeanHolder = null;
    boolean isVirtual = false;

    protected BundleContext bundleContext;

    public ItemStatus itemStatus = ItemStatus.UNKNOWN;

    public ItemStatus status() {
        return itemStatus;
    }

    public ItemStatus status(ItemStatus newStatus) {
        if (newStatus == null)
            return status(ItemStatus.UNKNOWN);
        this.itemStatus = newStatus;
        return status();
    }

    public boolean isInStatus(ItemStatus checkStatus) {
        return this.itemStatus.equals(checkStatus);
    }

    EntaxyRuntimeItemMBeanInfo<?> parentMBeanInfo;

    List<EntaxyRuntimeItemMBeanInfo<?>> children = new ArrayList<>();

    protected EntaxyRuntimeObjectMBeanFactory factory;

    public String getId() {
        return mbeanHolder == null ? "" : mbeanHolder.mbean.getId();
    }

    public boolean createHolder(T mbean) {
        mbeanHolder = MBeanHolder.create(mbean);
        return isCreated();
    }

    public boolean isCreated() {
        if (mbeanHolder == null)
            return false;
        return mbeanHolder.mbean != null;
    }

    public boolean isRegistered() {
        return isCreated() && (mbeanHolder.registration != null);
    }

    public void setBundleContext(BundleContext bundleContext) {
        this.bundleContext = bundleContext;
    }

    public void register(BundleContext bundleContext) {
        setBundleContext(bundleContext);
        register();
    }

    public void register() {
        try {
            if (this.mbeanHolder.bundleContext == null)
                this.mbeanHolder.bundleContext = this.bundleContext;
            this.mbeanHolder.register();
        } catch (Exception ignore) {
            // NOOP
        }
    }

    public void unregister() {
        try {
            this.mbeanHolder.unregister();
        } catch (Exception ignore) {
            // NOOP
        }
    }

    public Map<String, EntaxyRuntimeItemMBean> getAllChildMBeans() {
        if (this.mbeanHolder != null)
            return this.mbeanHolder.mbean.getChildMBeans();
        return Collections.emptyMap();
    };

    public Map<String, EntaxyRuntimeItemMBean> getPrimaryChildMBeans() {
        return getAllChildMBeans();
    };

}
