/*-
 * ~~~~~~licensing~~~~~~
 * object-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.object.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import ru.entaxy.platform.core.management.object.EntaxyRuntimeItemMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBean;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeRelation;

public class EntaxyRuntimeObjectMBeanInfo extends EntaxyRuntimeItemMBeanInfo<EntaxyRuntimeObjectMBean> {

    public enum LOAD_STATUS {
        AUTO_LOADED,
        LOADED,
        NONE
    }

    public EntaxyRuntimeObjectMBeanInfo(EntaxyRuntimeObject runtimeObject) {
        this.runtimeObject = runtimeObject;
    }

    String foundPattern;

    EntaxyRuntimeObject runtimeObject;

    EntaxyRuntimeObject parentObject;

    EntaxyRuntimeRelation relationFromParent;

    LOAD_STATUS loadStatus = LOAD_STATUS.NONE;

    Map<String, List<EntaxyRuntimeObjectMBeanInfo>> secondaryBeans = new HashMap<>();

    public boolean isGhost() {
        return this.runtimeObject.isGhost();
    }

    public void resetStatus() {
        this.loadStatus = LOAD_STATUS.NONE;
    }

    public void makeAutoLoaded() {
        this.loadStatus = LOAD_STATUS.AUTO_LOADED;
    }

    public boolean isAutoLoaded() {
        return LOAD_STATUS.AUTO_LOADED.equals(this.loadStatus);
    }

    public void makeLoaded() {
        this.loadStatus = LOAD_STATUS.LOADED;
    }

    public boolean isLoaded() {
        return LOAD_STATUS.LOADED.equals(this.loadStatus);
    }

    public boolean isUpdatedByLoad() {
        return !LOAD_STATUS.NONE.equals(this.loadStatus);
    }

    public void register() {
        super.register();
        for (String key : secondaryBeans.keySet()) {
            List<EntaxyRuntimeObjectMBeanInfo> list = secondaryBeans.get(key);
            for (EntaxyRuntimeObjectMBeanInfo sMBean : list)
                sMBean.register(this.bundleContext);
        }
    }

    public void unregister() {
        for (String key : secondaryBeans.keySet()) {
            List<EntaxyRuntimeObjectMBeanInfo> list = secondaryBeans.get(key);
            for (EntaxyRuntimeObjectMBeanInfo sMBean : list)
                sMBean.unregister();
        }
        super.unregister();
    }

    @Override
    public Map<String, EntaxyRuntimeItemMBean> getPrimaryChildMBeans() {
        final List<String> secondariesIds =
                secondaryBeans.values().stream().flatMap(list -> list.stream())
                        .map(bi -> bi.mbeanHolder.mbean.getId())
                        .collect(Collectors.toList());
        Map<String, EntaxyRuntimeItemMBean> result =
                getAllChildMBeans().entrySet().stream().filter(e -> !secondariesIds.contains(e.getValue().getId()))
                        .collect(Collectors.toMap(e -> e.getKey(), e -> e.getValue()));
        return result;
    }
}
