/*-
 * ~~~~~~licensing~~~~~~
 * object-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.object.shell;

import java.util.Map;
import java.util.Map.Entry;

import org.apache.karaf.shell.api.action.Action;
import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.karaf.shell.ShellTableFixed;
import ru.entaxy.platform.core.management.object.impl.EntaxyRuntimeItemMBeanInfo;
import ru.entaxy.platform.core.management.object.impl.EntaxyRuntimeObjectMBeanInfo;

@Service
@Command(scope = EntaxyRuntimeObjectMBeanManagerSupport.COMMAND_SCOPE, name = "object-mbean-list")
public class ListMBeans extends EntaxyRuntimeObjectMBeanManagerSupport implements Action {

    @Override
    public Object execute() throws Exception {
        ShellTableFixed shellTable = new ShellTableFixed();
        shellTable.column("Object Id");
        shellTable.column("Type");
        shellTable.column("Class name");
        shellTable.column("Registered");
        shellTable.column("Parent Id");
        shellTable.column("Ghost");

        Map<String, EntaxyRuntimeItemMBeanInfo<?>> map = mBeanManager.getInfoById();
        for (Entry<String, EntaxyRuntimeItemMBeanInfo<?>> entry : map.entrySet()) {

            EntaxyRuntimeItemMBeanInfo<?> info = entry.getValue();

            shellTable.addRow().addContent(
                    entry.getKey(),
                    info instanceof EntaxyRuntimeObjectMBeanInfo ? "OBJECT" : "CONTAINER",
                    info.mbeanHolder.mbean == null ? "" : info.mbeanHolder.mbean.getClass().getName(),
                    info.isRegistered() ? "*" : "",
                    info.mbeanHolder.mbean == null ? ""
                            : info.mbeanHolder.mbean.getParentMBean() == null ? ""
                                    : info.mbeanHolder.mbean.getParentMBean().getId(),
                    info instanceof EntaxyRuntimeObjectMBeanInfo
                            ? ((EntaxyRuntimeObjectMBeanInfo) info).isGhost() ? "*" : ""
                            : "");

        }

        shellTable.print(System.out);

        return null;
    }

}
