/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.esb.platform.core.management.profile.helper;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.management.profile.NotBackwardCompatibleException;
import ru.entaxy.platform.core.management.profile.support.legacy.AccountManager;

@Component(service = AccountHelper.class, immediate = true)
public class AccountHelperImpl implements AccountHelper {

    private static final Logger LOG = LoggerFactory.getLogger(AccountHelperImpl.class);

    protected AccountManager accountManager = null;

    protected AccountManager nullAccountManager = null;

    @Activate
    public void activate() {
        this.nullAccountManager = new AccountManager() {

            @Override
            public String getAccountForSystem(String systemUuid) throws Exception {
                throw new NotBackwardCompatibleException(Helpers.BACKWARD_COMPATIBILITY_DISABLED);
            }

            @Override
            public void createAccountForSystem(String login, String password, String systemUUID, String createdBy,
                    String editedBy) throws Exception {
                throw new NotBackwardCompatibleException(Helpers.BACKWARD_COMPATIBILITY_DISABLED);
            }

        };
    }

    @Reference(collectionType = CollectionType.SERVICE, cardinality = ReferenceCardinality.OPTIONAL,
            policy = ReferencePolicy.DYNAMIC, policyOption = ReferencePolicyOption.GREEDY,
            service = AccountManager.class, unbind = "unsetAccountManager")
    public void setAccountManager(AccountManager accountManager) {
        this.accountManager = accountManager;
    }

    public void unsetAccountManager(AccountManager accountManager) {
        this.accountManager = null;
    }

    protected AccountManager getEffectiveAccountManager() {
        return accountManager != null ? accountManager : nullAccountManager;
    }

    @Override
    public String getAccountForSystem(String systemUuid) throws Exception {
        return getEffectiveAccountManager().getAccountForSystem(systemUuid);
    }

    @Override
    public void createAccountForSystem(String login, String password, String systemUUID, String createdBy,
            String editedBy) throws Exception {
        getEffectiveAccountManager().createAccountForSystem(login, password, systemUUID, createdBy, editedBy);
    }


}
