/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.profile.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.management.NotCompliantMBeanException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.esb.platform.base.management.core.api.EntaxyRuntimeTyped;
import ru.entaxy.esb.platform.core.management.profile.helper.Helpers;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.core.management.object.ObjectMBean;
import ru.entaxy.platform.core.management.object.factory.AbstractObjectMBean;
import ru.entaxy.platform.core.management.profile.NotBackwardCompatibleException;
import ru.entaxy.platform.core.management.profile.ProfileMBean;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;

// @TODO move string to constant
@EntaxyRuntimeTyped(name = "entaxy.runtime.profile")
@ObjectMBean(subQualifierName = "profile")
public class ProfileMBeanImpl extends AbstractObjectMBean<ProfileMBean> implements ProfileMBean {

    private static final Logger log = LoggerFactory.getLogger(ProfileMBeanImpl.class);

    Helpers helpers;

    // BundleContext bundleContext;

    public ProfileMBeanImpl(EntaxyRuntimeObject entaxyRuntimeObject, /*
                                                                      * BundleContext bundleContext,
                                                                      */
            Helpers helpers)
            throws NotCompliantMBeanException {
        super(ProfileMBean.class, entaxyRuntimeObject);
        // this.bundleContext = bundleContext;
        this.helpers = helpers;
        setJMXSubQualifierValue(entaxyRuntimeObject.getId());
    }

    @Override
    public String getJMXSubQualifierName() {
        // TODO Auto-generated method stub
        return super.getJMXSubQualifierName();
    }

    @Override
    public String getJMXSubQualifierValue() {
        return getObjectId();
    }

    @Override
    public String getName() {
        return entaxyRuntimeObject.getAttributeOrDefault("name", "").toString();
    }

    @Override
    public String getDescription() {
        return entaxyRuntimeObject.getAttributeOrDefault("name", "").toString();
    }

    @Override
    @Deprecated
    public String getUuid() {
        try {
            return internalGetUuid();
        } catch (Exception ignore) {
        } ;
        return getObjectId();
    }

    @Deprecated
    protected String internalGetUuid() throws Exception {
        return helpers.systemHelper.getUUIDbyName(getName());
    }

    @Override
    public String getBasicAuthAccount() {
        try {
            return internalGetBasicAuthAccount();
        } catch (Exception ignore) {

        }
        return "";
    }

    protected String internalGetBasicAuthAccount() throws Exception {
        return helpers.accountHelper.getAccountForSystem(internalGetUuid());
    }

    @Override
    public void createAccount(String login, String password) throws Exception {
        if (CommonUtils.isValid(getBasicAuthAccount()))
            throw new IllegalArgumentException("Account already exists");

        // FIXME remove hardcoded
        this.helpers.accountHelper.createAccountForSystem(login, password, getUuid(), "admin", "admin");

    }

    @Override
    public void addConnector(String connectorFactoryId, Map<String, String> parameters) throws Exception {
        this.helpers.connectorHelper.addConnector(getName(), connectorFactoryId, parameters);
    }

    @Override
    public void removeConnector(String connectorName) throws Exception {
        this.helpers.connectorHelper.removeConnector(getName(), connectorName);
    }

    @Override
    public void startConnector(String connectorName) throws Exception {
        this.helpers.connectorHelper.startConnector(getName(), connectorName);
    }

    @Override
    public void stopConnector(String connectorName) throws Exception {
        this.helpers.connectorHelper.stopConnector(getName(), connectorName);
    }

    @Override
    public String getConnectorConfig(String connectorName) throws Exception {
        return helpers.entaxyObjectService.findObject(connectorName, "entaxy.runtime.connector").getConfiguration();
    }

    @Override
    public List<String> getAllowedTargets() {
        try {
            return helpers.permissionHelper.getObjectPermissions(internalGetUuid());
        } catch (NotBackwardCompatibleException ignore) {
            //
        } catch (Exception e) {
            log.warn("getAllowedTargets not succeded", e);
        }
        return new ArrayList<>();
    }

    @Override
    public List<String> getAllowedSources() {
        try {
            return helpers.permissionHelper.getSubjectPermissions(internalGetUuid());
        } catch (NotBackwardCompatibleException ignore) {
            //
        } catch (Exception e) {
            log.warn("getAllowedSources not succeded", e);
        }
        return new ArrayList<>();
    }

    @Override
    public void allowSource(String idOrName) throws Exception {
        helpers.permissionHelper.createPermission(idOrName, internalGetUuid());
    }

    @Override
    public void allowTarget(String idOrName) throws Exception {
        helpers.permissionHelper.createPermission(internalGetUuid(), idOrName);

    }

}
