/*-
 * ~~~~~~licensing~~~~~~
 * profile-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.profile.support.legacy;

import java.util.List;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.util.tracker.BundleTracker;

import ru.entaxy.esb.platform.core.management.profile.helper.Helpers;
import ru.entaxy.platform.base.support.osgi.tracker.BundleTrackerUtils;
import ru.entaxy.platform.base.support.osgi.tracker.filter.BundleCapabilityFilter;

@Component(service = LegacyConnectorManager.class, immediate = true)
public class LegacyConnectorManager {

    public static final String CAPABILITY_NAMESPACE = "entaxy.runtime.connector";

    protected BundleTracker<List<ManagedLegacyConnector>> connectorTracker;

    protected BundleContext bundleContext;

    @Reference(cardinality = ReferenceCardinality.MANDATORY, service = Helpers.class,
            collectionType = CollectionType.SERVICE)
    Helpers helpers;

    @Activate
    public void activate(ComponentContext componentContext) {
        this.bundleContext = componentContext.getBundleContext();
        connectorTracker = BundleTrackerUtils.<List<ManagedLegacyConnector>>createBuilder()
                .customizer(
                        (new DeployedLegacyConnectorCustomizer())
                                .listener(new ManagedLegacyConnectorsListener(bundleContext, this.helpers)))
                .addFilter(
                        (new BundleCapabilityFilter()).namespace(CAPABILITY_NAMESPACE))
                .bundleContext(bundleContext)
                .bundleState(Bundle.ACTIVE | Bundle.INSTALLED | Bundle.RESOLVED | Bundle.STARTING | Bundle.STOPPING)
                .get();
        connectorTracker.open();

    }
}
