/*-
 * ~~~~~~licensing~~~~~~
 * repository-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.repository.impl;

import java.util.Collections;
import java.util.List;

import javax.management.NotCompliantMBeanException;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.management.object.EntaxyRuntimeItemMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBeanFactory;
import ru.entaxy.platform.core.management.object.MBeanFactory;
import ru.entaxy.platform.core.management.object.factory.AbstractMBeanFactory;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;

@MBeanFactory(id = "repository-factory", supportedTypes = {"entaxy.runtime.repository"})
@Component(service = EntaxyRuntimeObjectMBeanFactory.class, immediate = true)
public class RepositoryMBeanFactory extends AbstractMBeanFactory {

    private static final Logger log = LoggerFactory.getLogger(RepositoryMBeanFactory.class);

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    RepositoriesMBeanImpl repositoriesMBean;

    @Override
    public <T extends EntaxyRuntimeObjectMBean> T createMBean(EntaxyRuntimeObject entaxyRuntimeObject) {
        RepositoryMBeanImpl beanImpl;
        try {
            beanImpl = new RepositoryMBeanImpl(entaxyRuntimeObject);
            beanImpl.setParentMBean(repositoriesMBean);
        } catch (NotCompliantMBeanException e) {
            log.error("Error creating ProfileMBeanImpl for [" + entaxyRuntimeObject.getObjectFullId() + "]", e);
            return null;
        }
        return (T) beanImpl;
    }

    @Override
    public List<? extends EntaxyRuntimeItemMBean> createMBeans() {
        return Collections.singletonList(repositoriesMBean);
    }

}
