/*-
 * ~~~~~~licensing~~~~~~
 * route-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.route;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.management.MBeanOperationInfo;

import ru.entaxy.esb.platform.base.management.core.api.Attribute;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.esb.platform.base.management.core.api.NotExported;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBean;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATED_ONLY)
public interface RoutesMBean extends EntaxyRuntimeObjectMBean {

    String ATTR_ROUTE_CONTAINER_TYPE = "routeContainerType";

    String JMX_SUBQUALIFIER_VALUE = "routes";

    String RUNTIME_TYPE = "entaxy.runtime.route-container";

    @Attribute
    default String getRouteContainerType() {
        return getRuntimeObject().getAttributeOrDefault(ATTR_ROUTE_CONTAINER_TYPE, "NONE").toString();
    }

    @Operation(desc = "List all included routes", impact = MBeanOperationInfo.INFO)
    default List<Map<String, Object>> listRoutes() {
        return Collections.emptyList();
    };

    @Override
    default String getJMXSubQualifierValue() {
        return JMX_SUBQUALIFIER_VALUE;
    }

    @Override
    @Attribute
    default String getRuntimeType() {
        return EntaxyRuntimeObjectMBean.super.getRuntimeType();
    }

    @Override
    @NotExported
    String readConfiguration();

    @Override
    @NotExported
    Map<String, Map<String, Object>> readExtendedData();

    @Override
    @NotExported
    void refreshBundleInfo();
}
