/*-
 * ~~~~~~licensing~~~~~~
 * route-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.route.impl;

import org.osgi.service.component.annotations.*;
import ru.entaxy.esb.platform.base.management.core.ManagementCore;
import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.esb.resources.EntaxyResourceService;
import ru.entaxy.platform.core.management.route.RouteComponentLibrariesMBean;
import ru.entaxy.platform.routes.design.library.ComponentLibrary;
import ru.entaxy.platform.routes.design.library.ComponentLibraryManager;

import javax.management.DynamicMBean;
import javax.management.MBeanRegistration;
import javax.management.NotCompliantMBeanException;
import java.util.List;
import java.util.stream.Collectors;

@Component(
        service = {RouteComponentLibrariesMBean.class, DynamicMBean.class, MBeanRegistration.class},
        property = {ManagementCore.JMX_OBJECTNAME + "=" + RouteComponentLibrariesMBean.JMX_OBJECT_NAME},
        scope = ServiceScope.SINGLETON,
        immediate = true)
public class RouteComponentLibrariesMBeanImpl extends AnnotatedMBean<RouteComponentLibrariesMBean>
        implements RouteComponentLibrariesMBean {

    @Reference
    ComponentLibraryManager componentLibraryManager;

    @Reference(cardinality = ReferenceCardinality.MANDATORY
            , service = EntaxyResourceService.class
            , collectionType = CollectionType.SERVICE)
    EntaxyResourceService entaxyResourceService;

    public RouteComponentLibrariesMBeanImpl() throws NotCompliantMBeanException {
        super(RouteComponentLibrariesMBean.class);
    }

    @Override
    public String readAllTagDescriptors() {
        List<ComponentLibrary.Schema> schemas = componentLibraryManager.getAllSchemasWithUiDescriptor();
        return schemas.stream().map(s -> {
            return entaxyResourceService
                    .getResource(s.getLibrary().getUrl() + "/" + s.getUiDescriptor())
                    .getAsString();
        }).collect(Collectors.toList()).toString();
    }
}
