/*-
 * ~~~~~~licensing~~~~~~
 * object-producing-resources-support
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.producer.resources.impl;

import java.util.HashMap;
import java.util.Map;

import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.CollectionType;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.resources.EntaxyResourceProducingProcessor;
import ru.entaxy.platform.core.producer.resources.EntaxyResourceProducingProcessorService;

@Component (service = EntaxyResourceProducingProcessorService.class, immediate = true)
public class EntaxyResourceProducingProcessorServiceImpl implements EntaxyResourceProducingProcessorService {

	private static final Logger log = LoggerFactory.getLogger(EntaxyResourceProducingProcessorServiceImpl.class);
	
	protected static EntaxyResourceProducingProcessorService _INSTANCE;
	
	public static EntaxyResourceProducingProcessorService getInstance() {
		return _INSTANCE;
	}
	
	protected Map<String, EntaxyResourceProducingProcessor> processorsMap = new HashMap<>(); 
	
	@Activate
	public void activate() {
		_INSTANCE = this;
	}
	
	@Reference (cardinality = ReferenceCardinality.MULTIPLE, collectionType = CollectionType.SERVICE, policy = ReferencePolicy.DYNAMIC
			, policyOption = ReferencePolicyOption.GREEDY, unbind = "removeProcessor")
	public void addProcessor(EntaxyResourceProducingProcessor processor) {
		synchronized (processorsMap) {
			this.processorsMap.put(processor.getName(), processor);
		}
	}

	public void removeProcessor(EntaxyResourceProducingProcessor processor) {
		synchronized (processorsMap) {
			this.processorsMap.remove(processor.getName());
		}
	}
	
	
	@Override
	public Object process(Object value, Map<String, Object> properties) throws Exception{
		Object val = properties.get(EntaxyResourceProducingProcessor.PROP_PROCESSOR);
		if (val == null)
			return value;
		String processorName = val.toString();
		if (!processorsMap.containsKey(processorName)) {
			log.warn("Unknown processor [{}] in [{}]", processorName, properties);
			return value;
		}
		return processorsMap.get(processorName).process(value, properties);
	}

}
