/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.MapUtils;

import ru.entaxy.platform.base.objects.EntaxyObject;

public interface EntaxyRuntimeObject extends EntaxyObject {

    public enum ENTAXY_RUNTIME_OBJECT_STATE {
        ACTIVE,
        INACTIVE,
        UNKNOWN
    }

    public interface CAPABILITIES_RUNTIME extends CAPABILITIES {
        String ATTR_RESOURCES = "resources";
    }

    public interface SEARCH {

        String ATTR_ID = "id";
        String ATTR_TYPE = "type";
        String ATTR_DISPLAY_NAME = "displayName";
        String ATTR_LABEL = "label";
        String ATTR_SCOPE = "scope";
        String ATTR_RESOURCES = "resources";
        String ATTR_FACTORY = "factory";
        String ATTR_RELATIONS_IN = "relationsIn";
        String ATTR_RELATIONS_OUT = "relationsOut";

        List<String> STANDARD_ATTRIBUTES_LIST = Arrays.asList(new String[] {
            ATTR_ID, ATTR_TYPE, ATTR_DISPLAY_NAME, ATTR_LABEL, ATTR_SCOPE, ATTR_RESOURCES, ATTR_FACTORY,
            ATTR_RELATIONS_IN, ATTR_RELATIONS_OUT
        });

        @SuppressWarnings("unchecked")
        Map<String, String> STANDARD_ATTRIBUTES = MapUtils.putAll(new LinkedHashMap<String, String>(), new String[][] {
            {ATTR_ID, "Object id"},
            {ATTR_TYPE, "Object type"},
            {ATTR_DISPLAY_NAME, "Display name"},
            {ATTR_LABEL, "Label"},
            {ATTR_SCOPE, "Scope"},
            {ATTR_RESOURCES, "Resources"},
            {ATTR_FACTORY, "Factory Id"},
            {ATTR_RELATIONS_IN, "Incoming relations"},
            {ATTR_RELATIONS_OUT, "Outgoing relations"}
        });
    }

    boolean isColocatedWith(EntaxyRuntimeObject object);

    List<EntaxyRuntimeRelation> getRelations(EntaxyObject.OBJECT_RELATION relationType);

    List<EntaxyRuntimeRelation> getSubordRelations();

    List<EntaxyRuntimeRelation> getIncomiingRelations(EntaxyObject.OBJECT_RELATION relationType);

    List<EntaxyRuntimeRelation> getIncomiingSubordRelations();

    List<EntaxyRuntimeRelation> getDependsOn();

    List<EntaxyRuntimeRelation> getDependents();

    EntaxyRuntimeObjectContainer getContainer();

    List<EntaxyRuntimeObject> getColocatedObjects();

    List<EntaxyRuntimeObjectResource> getResources();

    Map<String, Object> getAllAttributes();

    boolean hasAttribute(String name);

    Object getAttribute(String name);

    Object getAttributeOrDefault(String name, Object defaultValue);

}
