/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Filter;
import org.osgi.framework.FrameworkUtil;
import org.osgi.framework.InvalidSyntaxException;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject.ENTAXY_RUNTIME_OBJECT_STATE;

public interface EntaxyRuntimeObjectService {

    public interface Listener {

        void objectAdded(EntaxyRuntimeObject object);

        void objectModified(EntaxyRuntimeObject object);

        void objectRemoved(EntaxyRuntimeObject object);

    }

    void addListener(Listener listener);

    void removeListener(Listener listener);

    EntaxyRuntimeObject getRuntimeObject(EntaxyObject origin);

    EntaxyRuntimeObject getRuntimeObject(String runtimeObjectId);

    EntaxyRuntimeObjectContainer getContainer(String containerName);

    List<EntaxyRuntimeObject> getObjects();

    default List<EntaxyRuntimeObject> findObjects(String filter) throws InvalidSyntaxException {
        Filter osgiFilter = FrameworkUtil.createFilter(filter);
        return findObjects(osgiFilter);
    };

    List<EntaxyRuntimeObject> findObjects(Filter filter);

    Map<String, Object> getObjectEssense(EntaxyRuntimeObject object);

    List<EntaxyRuntimeObject> getObjectsForContainer(EntaxyRuntimeObjectContainer container);

    List<EntaxyRuntimeObjectContainer> getContainers();

    EntaxyRuntimeObjectContainer getContainerForObject(EntaxyRuntimeObject runtimeObject);

    EntaxyRuntimeObjectContainer getContainerForObject(String runtimeObjectId);

    ENTAXY_RUNTIME_OBJECT_STATE getObjectState(String runtimeObjectId);

    ENTAXY_RUNTIME_OBJECT_STATE getObjectState(EntaxyRuntimeObject runtimeObject);

    default Map<String, Map<String, Object>> getExtendedObjectInfo(String runtimeObjectId) {
        EntaxyRuntimeObject entaxyRuntimeObject = getRuntimeObject(runtimeObjectId);
        if (entaxyRuntimeObject == null)
            return new HashMap<>();
        return getExtendedObjectInfo(entaxyRuntimeObject);
    };

    Map<String, Map<String, Object>> getExtendedObjectInfo(EntaxyRuntimeObject runtimeObject);

}
