/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-core
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.Activate;
import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.objects.EntaxyObject.BundleInfo;
import ru.entaxy.platform.core.support.runtime.cluster.ClusterBundleDataProvider;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObjectExtendedDataProvider;

@Component(service = EntaxyRuntimeObjectExtendedDataProvider.class, immediate = true)
public class ClusterExtendedDataProvider implements EntaxyRuntimeObjectExtendedDataProvider {

    private static final Logger LOG = LoggerFactory.getLogger(ClusterExtendedDataProvider.class);

    private static final String CLUSTER_PROVIDER_ID = "cluster";

    @Reference(cardinality = ReferenceCardinality.OPTIONAL, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile protected ClusterBundleDataProvider clusterBundleDataProvider = null;

    protected BundleContext bundleContext;

    @Activate
    public void activate(ComponentContext componentContext) {
        this.bundleContext = componentContext.getBundleContext();
    }

    @Override
    public String getProviderId() {
        return CLUSTER_PROVIDER_ID;
    }

    @Override
    public Map<String, Object> getExtendedData(EntaxyRuntimeObject runtimeObject) {
        if (clusterBundleDataProvider == null)
            return null;
        Map<String, Object> result = new HashMap<>();

        List<String> groups = getClusterGroups(runtimeObject);
        if (groups != null) {
            result.put("groups", groups);
            result.put("objectClusterState", !groups.isEmpty() ?
                    groups.stream().allMatch(group -> group.startsWith("?")) ? "non-clustered" : "clustered" : "local");
        }

        return result;
    }

    protected List<String> getClusterGroups(EntaxyRuntimeObject runtimeObject) {
        try {
            BundleInfo bi = runtimeObject.getBundleInfo();
            Bundle bundle = bundleContext.getBundle(bi.getBundleId());
            return clusterBundleDataProvider.getGroups(bundle);
        } catch (Exception e) {
            LOG.error(String.format("Error getting cluster groups for [%s]", runtimeObject.getObjectFullId()), e);
            return null;
        }
    }

}
