/*-
 * ~~~~~~licensing~~~~~~
 * object-runtime-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.objects.runtime.shell;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.lifecycle.Reference;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.core.artifact.Artifact;
import ru.entaxy.platform.core.artifact.Artifacts;
import ru.entaxy.platform.core.artifact.repository.ArtifactRepository;
import ru.entaxy.platform.core.artifact.service.ArtifactService;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;

@Service
@Command(name = "object-versions", scope = EntaxyRuntimeObjectServiceSupport.OBJECTS_SCOPE,
        description = "List available object versions")
public class ObjectVersions extends RuntimeObjectAwareCommand {

    @Reference
    ArtifactService artifactService;

    @Override
    public Object doExecute(EntaxyRuntimeObject entaxyObject) throws Exception {
        Set<String> availaleVersions = new HashSet<>();

        Artifact artifact = Artifacts.create(Artifact.ARTIFACT_CATEGORY_BUNDLE);
        artifact.getCoordinates().groupId(entaxyObject.getType()).artifactId(entaxyObject.getId());

        String[] repos = new String[] {ArtifactRepository.REPO_NAME_LOCAL, ArtifactRepository.REPO_NAME_SHARED};
        for (String repoName : repos) {
            List<String> versions = artifactService.getRepository(repoName).getAvailableVersions(artifact);
            if (versions != null)
                availaleVersions.addAll(versions);
        }

        List<String> list = new ArrayList<>(availaleVersions);
        Collections.sort(list);

        System.out.println("Available versions:\n");
        System.out.println(list.stream().collect(Collectors.joining("\n")));

        return null;
    }

}
