/*-
 * ~~~~~~licensing~~~~~~
 * config-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.config.runtime;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.Constants;
import org.osgi.framework.ServiceReference;
import org.osgi.service.cm.Configuration;
import org.osgi.service.cm.ConfigurationAdmin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.base.objects.EntaxyObject;
import ru.entaxy.platform.base.support.JSONUtils;

public class ConfigEntaxyObject implements EntaxyObject, ConfigEntaxyObjectWrapped {

    private static final Logger LOG = LoggerFactory.getLogger(ConfigEntaxyObject.class);

    protected EntaxyObject origin;

    protected Map<String, Object> configurationMap = new HashMap<>();

    public ConfigEntaxyObject(EntaxyObject entaxyObject) {
        super();
        this.origin = entaxyObject;
        configurationMap.put(EntaxyObject.FIELDS.OBJECT_ID, origin.getId());
        configurationMap.put(EntaxyObject.FIELDS.OBJECT_TYPE, origin.getType());
        configurationMap.put(EntaxyObject.FIELDS.FACTORY_ID, origin.getFactoryId());
    }

    @Override
    public EntaxyObject getOrigin() {
        return this.origin;
    }

    @Override
    public String getId() {
        return origin.getId();
    }

    @Override
    public String getType() {
        return origin.getType();
    }

    @Override
    public String getDisplayName() {
        return origin.getDisplayName();
    }

    @Override
    public String getScope() {
        return origin.getScope();
    }

    @Override
    public String getOwner() {
        return origin.getOwner();
    }

    @Override
    public String getFactoryId() {
        return origin.getFactoryId();
    }

    @Override
    public BundleInfo getBundleInfo() {
        return origin.getBundleInfo();
    }

    @Override
    public String getConfiguration() {
        ServiceReference<ConfigurationAdmin> ref =
                ObjectConfigHelper.getInstance().getReference(ConfigurationAdmin.class);
        if (ref != null) {
            ConfigurationAdmin configurationAdmin = ObjectConfigHelper.getInstance().getService(ref);

            Map<String, Object> properties = new HashMap<>();

            try {
                Configuration config = configurationAdmin.getConfiguration(getId(), "?");

                Enumeration<String> keys = config.getProperties().keys();

                while (keys.hasMoreElements()) {
                    String key = keys.nextElement();
                    properties.put(key, config.getProperties().get(key));
                }

                properties.remove("felix.fileinstall.filename");
                properties.remove(Constants.SERVICE_PID);
                properties.remove(ConfigurationAdmin.SERVICE_FACTORYPID);

                configurationMap.put(EntaxyObject.FIELDS.PROPERTIES, properties);


                return JSONUtils.GSON.toJsonTree(configurationMap).toString();

            } catch (IOException e) {
                LOG.error("Error reading configuration [" + getId() + "]", e);
            } finally {
                ObjectConfigHelper.getInstance().ungetService(ref);
            }

        }
        return origin.getConfiguration();
    }

    @Override
    public boolean isProvided() {
        return origin.isProvided();
    }

    @Override
    public boolean isComplete() {
        return origin.isComplete();
    }

    @Override
    public boolean isGhost() {
        return origin.isGhost();
    }

    @Override
    public List<Relation> getRelations() {
        return origin.getRelations();
    }

    @Override
    public List<Relation> getIncomingRelations() {
        return origin.getIncomingRelations();
    }

}
