/*-
 * ~~~~~~licensing~~~~~~
 * route-producing
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.routes.design.library;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.producer.resources.EntaxyResourceProducingProcessor;
import ru.entaxy.platform.core.producer.resources.EntaxyResourceProducingProcessorService;

@Component(service = EntaxyResourceProducingProcessor.class, immediate = true)
public class ComponentLibraryResourceProcessor implements EntaxyResourceProducingProcessor {

    private static final Logger LOG = LoggerFactory.getLogger(ComponentLibraryResourceProcessor.class);

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    EntaxyResourceProducingProcessorService producingProcessorService;

    @Reference(cardinality = ReferenceCardinality.MANDATORY)
    ComponentLibraryManager componentLibraryManager;

    @Override
    public String getName() {
        return "component-library";
    }

    @SuppressWarnings("unchecked")
    @Override
    public Object process(Object value, Map<String, Object> properties) throws Exception {

        LOG.debug("\n\t ComponentLibraryResourceProcessor:");
        LOG.debug("\n\t" + properties.toString());

        Map<String, Object> parameters;
        if (properties.containsKey("resourceContext") && (properties.get("resourceContext") instanceof Map))
            parameters = new HashMap<>((Map<String, Object>) properties.get("resourceContext"));
        else
            parameters = new HashMap<>(properties);

        if (properties.containsKey("parameters") && (properties.get("parameters") instanceof Map))
            parameters.putAll((Map<String, Object>) properties.get("parameters"));

        // List<ComponentLibrary> libraries =
        // componentLibraryManager.getLibrariesForContext(parameters);
        // List<ComponentLibrary.Schema> schemas =
        // componentLibraryManager.getSchemasForContext(parameters);
        List<ComponentLibrary.Transformation> transformations =
                componentLibraryManager.getTransformationsForContext(parameters);

        LOG.info("\n\t TO APPLY:"
                + "\n\tTRANSFORMATIONS:\n\t"
                + transformations.stream().map(t -> "[" + t.getProcessor() + "] " + t.getSource())
                        .collect(Collectors.joining("\n\t")));


        Object result = value;

        for (ComponentLibrary.Transformation transformation : transformations) {

            Map<String, Object> processorProperties = new HashMap<>();
            processorProperties.put(PROP_PROCESSOR, transformation.getProcessor());
            processorProperties.put("source", transformation.getSourceUrl());
            processorProperties.put("parameters", parameters);

            LOG.info("\n EXEC TRANSFORM: [{}]/[{}]/[{}]/[{}]\nWITH PROPERTIES:\n{}", transformation.getSourceUrl(),
                    transformation.getProcessor(), transformation.getSchema().getRoot(),
                    transformation.getSchema().getLibrary().getName(),
                    processorProperties);

            result = producingProcessorService.process(result, processorProperties);

        }

        return result;
    }

}
