/*-
 * ~~~~~~licensing~~~~~~
 * route-producing
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.routes.design.library.storage;

import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import org.osgi.framework.Bundle;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.routes.design.library.ComponentLibrary;

public class ProvidedLibraryDescriptor {

    private static final Logger LOG = LoggerFactory.getLogger(ProvidedLibraryDescriptor.class);

    public static final String LIBRARY_PATH = "/ru/entaxy/" + ComponentLibrary.COMPONENT_LIBRARY_RESOURCE_PROTOCOL;

    public Bundle bundle;
    public String libraryPath;
    public String libraryName;

    protected List<ProvidedLibraryResource> libraryResources = null;

    public ProvidedLibraryDescriptor(Bundle bundle, String path, String name) {
        this.bundle = bundle;
        this.libraryPath = path;
        this.libraryName = name;
    }

    public List<ProvidedLibraryResource> getLibraryResources() {
        if (libraryResources == null) {
            libraryResources = new ArrayList<>();

            Enumeration<URL> entries = bundle.findEntries(libraryPath, "*.*", true);
            if (entries != null) {

                while (entries.hasMoreElements()) {
                    URL entry = entries.nextElement();
                    String entryUrl = entry.toString();

                    if (entryUrl.endsWith("/"))
                        continue;

                    String resourcePath = entryUrl.substring(entryUrl.indexOf(libraryPath) + libraryPath.length());

                    /*
                    if (resourcePath.equals(ComponentLibraryStorage.LIBRARY_DESCRIPTOR_NAME))
                        continue;
                    */

                    String resourceName = null;
                    int index = resourcePath.lastIndexOf('/');
                    if (index < 0)
                        resourceName = resourcePath;
                    else
                        resourceName = resourcePath.substring(index + 1);

                    ProvidedLibraryResource resource = new ProvidedLibraryResource();
                    resource.url = entry;
                    resource.path = resourcePath;
                    resource.name = resourceName;

                    libraryResources.add(resource);
                }

            }

        }
        return libraryResources;
    }

    public static class ProvidedLibraryResource {

        URL url;
        String name;
        String path;

    }

}
