<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2025 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions"
                xmlns:entaxy="http://www.entaxy.ru/schemas/1.0"
                xmlns:m="http://www.entaxy.ru/schemas/entaxy-mediators/1.0"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:blueprint="http://www.osgi.org/xmlns/blueprint/v1.0.0"
                xmlns:local="http://www.entaxy.ru/schemas/entaxy-mediators/connector-selector/1.0">
    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>

    <xsl:param name="context.type" select="'entaxy.runtime.profile'"/>
    <xsl:param name="context.fieldName" select="'in-flow-pre-route'"/>
    <xsl:param name="context.objectId" select="'sys-13'"/>

    <xsl:template match="/">
        <xsl:variable name="processed">
            <xsl:sequence select="local:add-ids(./element()[1])"/>
        </xsl:variable>
        <fragment>
            <xsl:for-each select="$processed//m:connector-selector">
                <xsl:call-template name="create-selector-bean"/>
            </xsl:for-each>
            <xsl:apply-templates select="$processed/element()[1]"/>
        </fragment>
    </xsl:template>

    <xsl:function name="local:add-ids" as="element()*">
        <xsl:param name="root"/>
        <xsl:apply-templates select="$root"/>
    </xsl:function>

    <xsl:template match="//m:connector-selector[not(@*)]" exclude-result-prefixes="xsl fn xs local">
        <m:connector-selector>
            <xsl:attribute name="id" select="replace(generate-id(), ':', '') "/>
            <xsl:apply-templates select="child::node()"/>
        </m:connector-selector>
    </xsl:template>

    <xsl:template match="//m:connector-selector[@id]" exclude-result-prefixes="xsl fn xs local">
        <xsl:comment>generated from 'connector-selector'</xsl:comment>
        <to xmlns="http://camel.apache.org/schema/blueprint">
            <xsl:attribute name="uri">
                <xsl:text>bean:selector-</xsl:text><xsl:value-of select="./@id"/>
            </xsl:attribute> <!-- select="concat('bean',':','selector-', ./@id)" /-->
        </to>
    </xsl:template>

    <xsl:template name="create-selector-bean">
        <blueprint:bean class="ru.entaxy.platform.profiles.runtime.support.ConnectorSelector" activation="eager">
            <xsl:attribute name="id">
                <xsl:text>selector-</xsl:text><xsl:value-of select="./@id"/>
            </xsl:attribute>
            <!-- select="concat('selector-', ./@id)" / -->
            <blueprint:property name="config">
                <xsl:attribute name="value" select="local:generate-config(.)"/>
            </blueprint:property>
            <blueprint:property name="profile">
                <xsl:attribute name="ref" select="concat('profile-', $context.objectId)"/>
            </blueprint:property>
        </blueprint:bean>
    </xsl:template>

    <xsl:function name="local:generate-config">
        <xsl:param name="selector-node"/>
        <xsl:text>{</xsl:text>
        <xsl:apply-templates select="$selector-node/node()"/>
        <xsl:text>"_end":true}</xsl:text>
    </xsl:function>

    <xsl:template match="//m:connector-selector[@id]/m:properties-to-use">
        <xsl:text>"properties":[</xsl:text>
        <xsl:for-each select="./m:property">
            <xsl:text>"</xsl:text><xsl:value-of select="./text()"/><xsl:text>"</xsl:text>
            <xsl:if test="./following-sibling::m:property">
                <xsl:text>,</xsl:text>
            </xsl:if>
        </xsl:for-each>
        <xsl:text>],</xsl:text>
    </xsl:template>

    <xsl:template match="//m:connector-selector[@id]/m:patterns">
        <xsl:text>"patterns":[</xsl:text>
        <xsl:for-each select="./m:pattern">
            <xsl:text>"</xsl:text><xsl:value-of select="./text()"/><xsl:text>"</xsl:text>
            <xsl:if test="./following-sibling::m:pattern">
                <xsl:text>,</xsl:text>
            </xsl:if>
        </xsl:for-each>
        <xsl:text>],</xsl:text>
    </xsl:template>

    <xsl:template match="//m:connector-selector[@id]/m:options">
        <xsl:text>"options":{</xsl:text>
        <xsl:for-each select="./element()">
            <xsl:text>"</xsl:text><xsl:value-of select="./local-name()"/><xsl:text>":</xsl:text>
            <xsl:choose>
                <xsl:when test="string-length(./text()) > 0">
                    <xsl:variable name="value" select="./text()"/>
                    <xsl:choose>
                        <xsl:when test="($value = 'true') or ($value = 'false')">
                            <xsl:value-of select="./text()"/><xsl:text>,</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>"</xsl:text><xsl:value-of select="./text()"/><xsl:text>",</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true,</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        <xsl:text>"_end":true},</xsl:text>
    </xsl:template>

    <xsl:template name="default-copy" match="node()|@*">
        <xsl:copy>
            <xsl:apply-templates select="node()|@*"/>
        </xsl:copy>
    </xsl:template>

</xsl:stylesheet>
