<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2025 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="3.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions"
                xmlns:entaxy="http://www.entaxy.ru/schemas/1.0"
                xmlns:m="http://www.entaxy.ru/schemas/entaxy-mediators/1.0"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                xmlns:blueprint="http://www.osgi.org/xmlns/blueprint/v1.0.0"
                xmlns:camel="http://camel.apache.org/schema/blueprint"
                xmlns="http://camel.apache.org/schema/blueprint">

    <xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes" cdata-section-elements="m:set-content"/>


    <xsl:param name="context.type" select="'entaxy.runtime.profile'"/>
    <xsl:param name="context.fieldName" select="'in-flow-pre-route'"/>
    <xsl:param name="context.objectId" select="'sys-13'"/>

    <!-- only for inner routes -->
    <xsl:param name="context.properties.__routeOwner" select="'unknown:'"/>
    <!-- aliases -->
    <xsl:param name="routeOwnerId" select="substring-before($context.properties.__routeOwner,':')"/>

    <!--
        BEGIN components
    -->

    <!-- standard-router -->
    <xsl:template match="//m:standard-router">
        <xsl:text>&#10;</xsl:text>
        <xsl:comment>BEGIN standard-router</xsl:comment>
        <choice>
            <when>
                <simple>${headers.ENTAXY_Destination} == null</simple>
                <log>
                    <xsl:attribute name="message">${header.NTX_loggingKey} Destination isn't set</xsl:attribute>
                    <xsl:attribute name="loggingLevel">ERROR</xsl:attribute>
                </log>
                <throwException exceptionType="java.lang.IllegalArgumentException"
                                message="Destination isn't set"/>
            </when>
            <when>
                <simple>${headers.ENTAXY_DestinationType} == 'system.name'</simple>
                <toD>
                    <xsl:attribute name="uri">system:${headers.ENTAXY_Destination}</xsl:attribute>
                </toD>
            </when>
            <when>
                <simple>${headers.ENTAXY_DestinationType} == 'system-group.name'</simple>
                <toD>
                    <xsl:attribute name="uri">system-group:${headers.ENTAXY_Destination}</xsl:attribute>
                </toD>
            </when>
            <when>
                <simple>${headers.ENTAXY_DestinationType} == 'queue.name'</simple>
                <toD>
                    <xsl:attribute name="uri">entaxy-broker:queue:${headers.ENTAXY_Destination}</xsl:attribute>
                </toD>
            </when>
            <when>
                <simple>${headers.ENTAXY_DestinationType} == 'topic.name'</simple>
                <toD>
                    <xsl:attribute name="uri">entaxy-broker:topic:${headers.ENTAXY_Destination}</xsl:attribute>
                </toD>
            </when>
            <otherwise>
                <toD>
                    <xsl:attribute name="uri">system:${headers.ENTAXY_Destination}</xsl:attribute>
                </toD>
            </otherwise>
        </choice>
        <xsl:text>&#10;</xsl:text>
        <xsl:comment>END standard-router</xsl:comment>

    </xsl:template>

    <!-- set-destination -->
    <xsl:template match="//m:set-destination" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="full" select="./text()"/>
        <xsl:variable name="name">
            <xsl:choose>
                <xsl:when test="contains($full, ':')">
                    <xsl:value-of select="substring-before($full, ':')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$full"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="type" select="substring-after($full, ':')"/>
        <xsl:comment>generated from 'set-destination'</xsl:comment>
        <setHeader name="ENTAXY_Destination">
            <constant>
                <xsl:value-of select="$name"/>
            </constant>
        </setHeader>
        <xsl:choose>
            <xsl:when test="string-length($type) > 0">
                <setHeader name="ENTAXY_DestinationType">
                    <constant>
                        <xsl:value-of select="$type"/>
                    </constant>
                </setHeader>
            </xsl:when>
            <xsl:otherwise>
                <removeHeader headerName="ENTAXY_DestinationType"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- default-destination -->
    <xsl:template match="//m:default-destination" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="full" select="./text()"/>
        <xsl:variable name="name">
            <xsl:choose>
                <xsl:when test="contains($full, ':')">
                    <xsl:value-of select="substring-before($full, ':')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$full"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="type" select="substring-after($full, ':')"/>
        <xsl:comment>generated from 'default-destination'</xsl:comment>
        <choice>
            <when>
                <simple>${headers.ENTAXY_Destination} == null || ${headers.ENTAXY_Destination} == ''</simple>
                <removeHeader headerName="ENTAXY_DestinationType"/>
                <setHeader name="ENTAXY_Destination">
                    <constant>
                        <xsl:value-of select="$name"/>
                    </constant>
                </setHeader>
                <xsl:if test="string-length($type) > 0">
                    <setHeader name="ENTAXY_DestinationType">
                        <constant>
                            <xsl:value-of select="$type"/>
                        </constant>
                    </setHeader>
                </xsl:if>
            </when>
        </choice>
    </xsl:template>

    <!-- prefer-connector -->
    <!--	
		USAGE:
			<prefer-connector>
				<protocol>file</protocol>
				<classifier></classifier>
				<type></type>
			</prefer-connector>
	-->
    <xsl:template match="//m:prefer-connector" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="nodes" select="child::element()"/>
        <setHeader name="NTX_preferred_connector">
            <constant>
                <xsl:text>{</xsl:text>
                <xsl:for-each select="$nodes">
                    <xsl:text>"</xsl:text><xsl:value-of select="local-name(.)"/><xsl:text>": "</xsl:text><xsl:value-of
                        select="."/><xsl:text>"</xsl:text>
                    <xsl:variable name="i" select="position()"/>
                    <xsl:choose>
                        <xsl:when test="$nodes[$i + 1]!=''">
                            <xsl:text>,</xsl:text>
                        </xsl:when>
                    </xsl:choose>
                </xsl:for-each>
                <xsl:text>}</xsl:text>
            </constant>
        </setHeader>
    </xsl:template>


    <!-- log -->
    <xsl:template match="//m:log" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="nodes" select="child::element()"/>
        <log>
            <xsl:variable name="message">
                <xsl:value-of select="@message"/>
            </xsl:variable>
            <xsl:if test="string-length($message) > 0">
                <xsl:attribute name="message"
                               select="concat('${headers.NTX_loggingKey} ', $message)"/>
            </xsl:if>
            <xsl:variable name="loggingLevel">
                <xsl:value-of select="@loggingLevel"/>
            </xsl:variable>
            <xsl:if test="string-length($loggingLevel) > 0">
                <xsl:attribute name="loggingLevel" select="$loggingLevel"/>
            </xsl:if>
            <xsl:variable name="logName">
                <xsl:value-of select="@logName"/>
            </xsl:variable>
            <xsl:if test="string-length($logName) > 0">
                <xsl:attribute name="logName" select="$logName"/>
            </xsl:if>
            <xsl:variable name="marker">
                <xsl:value-of select="@marker"/>
            </xsl:variable>
            <xsl:if test="string-length($marker) > 0">
                <xsl:attribute name="marker" select="$marker"/>
            </xsl:if>
            <xsl:variable name="loggerRef">
                <xsl:value-of select="@loggerRef"/>
            </xsl:variable>
            <xsl:if test="string-length($loggerRef) > 0">
                <xsl:attribute name="loggerRef" select="$loggerRef"/>
            </xsl:if>
        </log>
    </xsl:template>

    <xsl:template match="//m:set-log-key" exclude-result-prefixes="xsl fn xs camel">
        <xsl:for-each select="./*">
            <xsl:choose>
                <xsl:when test="local-name() = 'custom'">
                    <setProperty>
                        <xsl:attribute name="name" select="@name"/>
                        <xsl:copy-of select="node()"/>
                    </setProperty>
                </xsl:when>
                <xsl:otherwise>
                    <setProperty>
                        <xsl:attribute name="name" select="local-name()"/>
                        <xsl:copy-of select="node()"/>
                    </setProperty>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
        <to uri="direct-vm:fill-logging-key"/>
    </xsl:template>

    <!-- next-route -->
    <xsl:template match="//m:next-route" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="local">
            <xsl:choose>
                <xsl:when test="./@local">
                    <xsl:value-of select="./@local"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>false</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="async">
            <xsl:choose>
                <xsl:when test="./@async">
                    <xsl:value-of select="./@async"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>false</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="component">
            <xsl:choose>
                <xsl:when test="$async = 'false'">
                    <xsl:text>direct</xsl:text>
                    <xsl:if test="$local = 'false'">
                        <xsl:text>-vm</xsl:text>
                    </xsl:if>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="$local = 'true'">
                            <xsl:text>seda</xsl:text>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>vm</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="name">
            <xsl:value-of select="./text()"/>
        </xsl:variable>
        <xsl:comment>generated from 'next-route'</xsl:comment>
        <setHeader name="NTX_NextPoint">
            <constant>
                <xsl:value-of select="$component"/><xsl:text>:</xsl:text><xsl:value-of select="$name"/>
            </constant>
        </setHeader>
    </xsl:template>

    <!-- xslt-xalan with resources -->
    <xsl:template match="//m:xslt-xalan" exclude-result-prefixes="xsl fn xs camel">
        <blueprint:reference id="xslUrlResolver" interface="javax.xml.transform.URIResolver" filter="(type=xslt)"/>
        <xsl:comment>generated from 'xslt-xalan'</xsl:comment>
        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'resource' and name() != 'target' and name() != 'targetType'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work correctly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($params, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="resource">
            <xsl:value-of select="@resource"/>
        </xsl:variable>
        <xsl:variable name="resultResource">
            <xsl:choose>
                <xsl:when test="contains($resource, ':')">
                    <xsl:value-of select="translate($resource,':','/')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$resource"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('atlasmap-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <step>
            <xsl:attribute name="id" select="$stepId" />

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri"
                                       select="concat('xslt:entaxy-resource://', $resultResource, '?uriResolver=#xslUrlResolver', $params)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri"
                                       select="concat('xslt:entaxy-resource://', $resultResource, '?uriResolver=#xslUrlResolver', $params)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="string-length(@target) > 0 and not(@targetType = 'body')">
                    <xsl:choose>
                        <xsl:when test="@targetType = 'header'">
                            <setHeader name="{@target}">
                                <simple>${body}</simple>
                            </setHeader>
                        </xsl:when>
                        <xsl:otherwise>
                            <setProperty name="{@target}">
                                <simple>${body}</simple>
                            </setProperty>
                        </xsl:otherwise>
                    </xsl:choose>
                    <setBody><simple>${headers.NTX_Body}</simple></setBody>
                </xsl:when>
            </xsl:choose>
            <removeHeader headerName="NTX_Body"/>
        </step>
    </xsl:template>

    <!-- xslt with resources -->
    <xsl:template match="//m:xslt" exclude-result-prefixes="xsl fn xs camel">
        <blueprint:reference id="xslUrlResolver" interface="javax.xml.transform.URIResolver" filter="(type=xslt)"/>
        <xsl:comment>generated from 'xslt'</xsl:comment>

        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'resource' and name() != 'target' and name() != 'targetType'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work correctly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($params, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="resource">
            <xsl:value-of select="@resource"/>
        </xsl:variable>
        <xsl:variable name="resultResource">
            <xsl:choose>
                <xsl:when test="contains($resource, ':')">
                    <xsl:value-of select="translate($resource,':','/')"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$resource"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('atlasmap-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <step>
            <xsl:attribute name="id" select="$stepId" />

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <when>
                <simple>${body.class.name} == 'org.apache.camel.component.file.GenericFile'</simple>
                <xsl:comment>Convert body from 'org.apache.camel.component.file.GenericFile' before 'xslt-saxon' invocation because it locks on Windows the file referenced by the message body and does not give to rename/remove processed file.</xsl:comment>
                <convertBodyTo type="String"/>
            </when>
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri"
                                       select="concat('xslt-saxon:entaxy-resource://', $resultResource, '?uriResolver=#xslUrlResolver', $params)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri"
                                       select="concat('xslt-saxon:entaxy-resource://', $resultResource, '?uriResolver=#xslUrlResolver', $params)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="string-length(@target) > 0 and not(@targetType = 'body')">
                    <xsl:choose>
                        <xsl:when test="@targetType = 'header'">
                            <setHeader name="{@target}">
                                <simple>${body}</simple>
                            </setHeader>
                        </xsl:when>
                        <xsl:otherwise>
                            <setProperty name="{@target}">
                                <simple>${body}</simple>
                            </setProperty>
                        </xsl:otherwise>
                    </xsl:choose>
                    <setBody><simple>${headers.NTX_Body}</simple></setBody>
                </xsl:when>
            </xsl:choose>
            <removeHeader headerName="NTX_Body"/>
        </step>
    </xsl:template>

    <xsl:template match="//*:xpath[not(@saxon)]" exclude-result-prefixes="xsl fn xs camel">
        <xpath saxon="true">
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}" select="." />
            </xsl:for-each>
            <xsl:value-of select="./text()" />
        </xpath>
    </xsl:template>

    <!-- xpath target="NTX_Example" targetType="header" -->
    <xsl:template name="xpath" exclude-result-prefixes="xsl fn xs camel">
        <xpath saxon="true">
            <xsl:for-each select="@*">
                <xsl:if test="name() != 'target' and name() != 'targetType'">
                    <xsl:attribute name="{name()}" select="." />
                </xsl:if>
            </xsl:for-each>
            <xsl:value-of select="./text()" />
        </xpath>
    </xsl:template>
    <xsl:template match="//m:xpath" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'xpath'</xsl:comment>
        <xsl:choose>
            <xsl:when test="(@targetType and not(@targetType = 'body')) and string-length(@target) > 0">
                <xsl:choose>
                    <xsl:when test="@targetType = 'header'">
                        <setHeader name="{@target}">
                            <xsl:call-template name="xpath" />
                        </setHeader>
                    </xsl:when>
                    <xsl:otherwise>
                        <setProperty name="{@target}">
                            <xsl:call-template name="xpath" />
                        </setProperty>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:when test="@targetType = 'body'">
                <setBody>
                    <xsl:call-template name="xpath" />
                </setBody>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="xpath" />
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- ignite-cache key="lama1" action="GET" cacheName="runtime" -->
    <xsl:template match="//m:ignite-cache" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'ignite-cache'</xsl:comment>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('ignite-cache-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'key' and name() != 'action' and name() != 'cacheName' and name() != 'target' and name() != 'targetType'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work corectly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($params, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <step>
            <xsl:attribute name="id" select="$stepId" />
            <xsl:if test="string-length(@key) > 0">
                <setHeader name="CamelIgniteCacheKey">
                    <simple><xsl:value-of select="@key"/></simple>
                </setHeader>
            </xsl:if>

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <xsl:if test="node() and node()[normalize-space()]">
                <setBody><xsl:apply-templates/></setBody>
            </xsl:if>

            <xsl:variable name="action" select="@action"/>
            <xsl:variable name="cacheName" select="@cacheName"/>
            <xsl:variable name="action">
                <xsl:choose>
                    <xsl:when test="string-length($action) > 0">
                        <xsl:value-of select="@action"/>
                    </xsl:when>
                    <xsl:otherwise>GET</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="cacheName">
                <xsl:choose>
                    <xsl:when test="string-length($cacheName) > 0">
                        <xsl:value-of select="@cacheName"/>
                    </xsl:when>
                    <xsl:otherwise>runtime</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="resultParams">
                <xsl:value-of select="concat( '?operation=', $action)"/>
                <xsl:if test="string-length($params) > 0">
                    <xsl:value-of select="$params"/>
                </xsl:if>
            </xsl:variable>
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="string-length(@target) > 0 and not(@targetType = 'body')">
                    <xsl:choose>
                        <xsl:when test="@targetType = 'header'">
                            <setHeader name="{@target}">
                                <simple>${body}</simple>
                            </setHeader>
                        </xsl:when>
                        <xsl:otherwise>
                            <setProperty name="{@target}">
                                <simple>${body}</simple>
                            </setProperty>
                        </xsl:otherwise>
                    </xsl:choose>
                    <setBody><simple>${headers.NTX_Body}</simple></setBody>
                </xsl:when>
            </xsl:choose>
            <removeHeader headerName="NTX_Body"/>
        </step>
    </xsl:template>

    <!-- cache-read key="lama1" cacheName="runtime" -->
    <xsl:template match="//m:cache-read" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'cache-read'</xsl:comment>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('cache-read-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'key'and name() != 'cacheName' and name() != 'id' and name() != 'target' and name() != 'targetType' ">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work corectly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains(Sparams, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <step>
            <xsl:attribute name="id" select="$stepId" />
            <xsl:if test="string-length(@key) > 0">
                <setHeader name="CamelIgniteCacheKey">
                    <simple><xsl:value-of select="@key"/></simple>
                </setHeader>
            </xsl:if>

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <xsl:variable name="cacheName" select="@cacheName"/>
            <xsl:variable name="cacheName">
                <xsl:choose>
                    <xsl:when test="string-length($cacheName) > 0">
                        <xsl:value-of select="@cacheName"/>
                    </xsl:when>
                    <xsl:otherwise>runtime</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="resultParams">
                <xsl:text>?operation=GET</xsl:text>
                <xsl:value-of select="$params"/>
            </xsl:variable>
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="string-length(@target) > 0 and not(@targetType = 'body')">
                    <xsl:choose>
                        <xsl:when test="@targetType = 'header'">
                            <setHeader name="{@target}">
                                <simple>${body}</simple>
                            </setHeader>
                        </xsl:when>
                        <xsl:otherwise>
                            <setProperty name="{@target}">
                                <simple>${body}</simple>
                            </setProperty>
                        </xsl:otherwise>
                    </xsl:choose>
                    <setBody><simple>${headers.NTX_Body}</simple></setBody>
                </xsl:when>
            </xsl:choose>
            <removeHeader headerName="NTX_Body"/>
        </step>
    </xsl:template>

    <!-- cache-write key="lama1" cacheName="runtime" -->
    <xsl:template match="//m:cache-write" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'cache-write'</xsl:comment>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('cache-write-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'key' and name() != 'id' and name() != 'cacheName'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>
        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work corectly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($params, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <step>
            <xsl:attribute name="id" select="$stepId" />
            <xsl:if test="string-length(@key) > 0">
                <setHeader name="CamelIgniteCacheKey">
                    <simple><xsl:value-of select="@key"/></simple>
                </setHeader>
            </xsl:if>

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <xsl:if test="node() and node()[normalize-space()]">
                <setBody><xsl:apply-templates/></setBody>
            </xsl:if>

            <xsl:variable name="cacheName" select="@cacheName"/>
            <xsl:variable name="cacheName">
                <xsl:choose>
                    <xsl:when test="string-length($cacheName) > 0">
                        <xsl:value-of select="@cacheName"/>
                    </xsl:when>
                    <xsl:otherwise>runtime</xsl:otherwise>
                </xsl:choose>
            </xsl:variable>
            <xsl:variable name="resultParams">
                <xsl:text>?operation=PUT</xsl:text>
                <xsl:value-of select="$params"/>
            </xsl:variable>

            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri" select="concat('ignite-cache:', $cacheName, $resultParams)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <setBody><simple>${headers.NTX_Body}</simple></setBody>
            <removeHeader headerName="NTX_Body"/>
        </step>
    </xsl:template>

    <xsl:template match="//m:filter" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'filter'</xsl:comment>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('filter-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <step>
            <xsl:attribute name="id" select="$stepId" />
            <choice>
                <when>
                    <xsl:for-each select="./*">
                        <xsl:apply-templates select="."/>
                    </xsl:for-each>
                </when>
                <otherwise>
                    <stop/>
                </otherwise>
            </choice>
        </step>
    </xsl:template>


    <xsl:template match="//m:threshold-filter" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'threshold-filter'</xsl:comment>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('threshold-filter-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="cacheName" select="@cacheName"/>
        <xsl:variable name="cacheName">
            <xsl:choose>
                <xsl:when test="string-length($cacheName) > 0">
                    <xsl:value-of select="@cacheName"/>
                </xsl:when>
                <xsl:otherwise>runtime</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="accept">
            <xsl:choose>
                <xsl:when test="@accept = 'greaterOrEqual'">&gt;=</xsl:when>
                <xsl:when test="@accept = 'greater'">&gt;</xsl:when>
                <xsl:when test="@accept = 'less'">&lt;</xsl:when>
                <xsl:when test="@accept = 'lessOrEqual'">&lt;=</xsl:when>
                <xsl:when test="@accept = 'equal'">=</xsl:when>
                <xsl:otherwise>&gt;</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="acceptNull">
            <xsl:if test="@acceptNull = 'true'"> || ${headers.NTX_ThresholdFilterTestValue} == null</xsl:if>
        </xsl:variable>

        <step>
            <xsl:attribute name="id" select="$stepId" />

            <setHeader name="NTX_ThresholdFilterBody">
                <simple>${body}</simple>
            </setHeader>

            <setHeader name="NTX_ThresholdFilterTestValue">
                <xsl:apply-templates select="./m:test-value/node()"/>
            </setHeader>

            <setHeader name="CamelIgniteCacheKey">
                <xsl:apply-templates select="./m:filter-key/node()"/>
            </setHeader>

            <to uri="ignite-cache:{$cacheName}?operation=GET" />

            <choice>
                <when>
                    <simple>${body} == null <xsl:value-of select="$acceptNull"/></simple>

                    <xsl:if test="@update = 'true'">
                        <setBody>
                            <simple>{"${headers.CamelIgniteCacheKey}": "${headers.NTX_ThresholdFilterTestValue}"}</simple>
                        </setBody>
                        <to uri="ignite-cache:{$cacheName}?operation=PUT" />
                    </xsl:if>
                </when>
                <when>
                    <simple>${body} != null &amp;&amp; ${headers.NTX_ThresholdFilterTestValue} <xsl:value-of select="$accept"/> ${body}</simple>

                    <xsl:if test="@update = 'true'">
                        <setBody>
                            <simple>{"${headers.CamelIgniteCacheKey}": "${headers.NTX_ThresholdFilterTestValue}"}</simple>
                        </setBody>
                        <to uri="ignite-cache:{$cacheName}?operation=PUT" />
                    </xsl:if>
                </when>
                <otherwise>
                    <stop/>
                </otherwise>
            </choice>

            <setBody><simple>${headers.NTX_ThresholdFilterBody}</simple></setBody>
            <removeHeaders pattern="NTX_ThresholdFilter.+|CamelIgniteCacheKey"/>
        </step>
    </xsl:template>

    <xsl:template match="//*:pollEnrich[not(@strategyRef)]" exclude-result-prefixes="xsl fn xs camel">
        <blueprint:bean id="entaxyHeaderMergeAggregator" class="ru.entaxy.esb.system.common.aggregation.HeaderMergeAggregatorImpl"/>
        <xsl:comment>generated from 'pollEnrich'</xsl:comment>
        <pollEnrich strategyRef="entaxyHeaderMergeAggregator">
            <xsl:for-each select="@*">
                <xsl:attribute name="{name()}" select="." />
            </xsl:for-each>
            <xsl:apply-templates/>
        </pollEnrich>
    </xsl:template>


    <!-- atlasmap with resources -->
    <xsl:template match="//m:atlasmap" exclude-result-prefixes="xsl fn xs camel">
        <xsl:comment>generated from 'atlasmap'</xsl:comment>
        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="./@*">
                <xsl:if test="name() != 'resource' and name() != 'target' and name() != 'targetType'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work corectly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($params, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="resource">
            <xsl:value-of select="@resource"/>
        </xsl:variable>
        <xsl:variable name="resultParams">
            <xsl:choose>
                <xsl:when test="string-length($params) > 0">
                    <xsl:value-of select="concat( '?', replace($params, '&amp;', ''))"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="stepId">
            <xsl:choose>
                <xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
                <xsl:otherwise><xsl:value-of select="concat('atlasmap-', generate-id(.))" /></xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <step>
            <xsl:attribute name="id" select="$stepId" />

            <setHeader name="NTX_Body">
                <simple>${body}</simple>
            </setHeader>
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri" select="concat('atlas:', $resource, $resultParams)"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri" select="concat('atlas:', $resource, $resultParams)"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:choose>
                <xsl:when test="string-length(@target) > 0 and not(@targetType = 'body')">
                    <xsl:choose>
                        <xsl:when test="@targetType = 'header'">
                            <setHeader name="{@target}">
                                <simple>${body}</simple>
                            </setHeader>
                        </xsl:when>
                        <xsl:otherwise>
                            <setProperty name="{@target}">
                                <simple>${body}</simple>
                            </setProperty>
                        </xsl:otherwise>
                    </xsl:choose>
                    <setBody><simple>${headers.NTX_Body}</simple></setBody>
                </xsl:when>
            </xsl:choose>
            <removeHeader headerName="NTX_Body"/>
        </step>

    </xsl:template>

    <!-- read-secret -->
    <xsl:template match="//m:read-secret" exclude-result-prefixes="xsl fn xs camel">
        <xsl:choose>
            <xsl:when test="@vault">
                <xsl:variable name="vault">
                    <xsl:value-of select="@vault"/>
                </xsl:variable>
                <xsl:variable name="property">
                    <xsl:choose>
                        <xsl:when test="@property">
                            <xsl:value-of select="@property"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>__SECRET__</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="secretName">
                    <xsl:choose>
                        <xsl:when test="@alias">
                            <xsl:value-of select="@alias"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:text>secret</xsl:text>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:variable>
                <xsl:variable name="dollar">
                    <xsl:text>$</xsl:text>
                </xsl:variable>
                <reference interface="ru.entaxy.security.vault.runtime.EntaxyVault"
                           xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
                    <xsl:attribute name="id" select="concat('vault-ref-',$vault)"/>
                    <xsl:attribute name="filter" select="concat('(name=', $vault, ')')"/>
                </reference>
                <bean class="ru.entaxy.security.vault.runtime.camel.EntaxyVaultHelperBean" activation="eager"
                      xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
                    <xsl:attribute name="id" select="concat('vault-wrapper-',$vault)"/>
                    <property name="entaxyVault">
                        <xsl:attribute name="ref" select="concat('vault-ref-',$vault)"/>
                    </property>
                </bean>
                <to>
                    <xsl:attribute name="uri"
                                   select="concat('bean:vault-wrapper-',$vault,'?method=getValue(', $dollar, '{exchange}, ''', $secretName, ''',''', $property, ''')' )"/>
                </to>
            </xsl:when>
        </xsl:choose>
    </xsl:template>

    <!-- respond -->
    <xsl:template match="//m:respond" exclude-result-prefixes="xsl fn xs camel">
        <xsl:variable name="respondNow">
            <xsl:choose>
                <xsl:when test="@now">
                    <xsl:value-of select="@now"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="respondContinue">
            <xsl:choose>
                <xsl:when test="@continue">
                    <xsl:value-of select="@continue"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="respondContinueAsync">
            <xsl:choose>
                <xsl:when test="@continueAsync">
                    <xsl:value-of select="@continueAsync"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text>true</xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <setHeader name="NTX_RESPOND_exec">
            <constant>true</constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_now">
            <constant>
                <xsl:value-of select="$respondNow"/>
            </constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_continue">
            <constant>
                <xsl:value-of select="$respondContinue"/>
            </constant>
        </setHeader>
        <setHeader name="NTX_RESPOND_continueAsync">
            <constant>
                <xsl:value-of select="$respondContinueAsync"/>
            </constant>
        </setHeader>
    </xsl:template>

    <!-- to-connection -->
    <xsl:template match="//m:to-connection" exclude-result-prefixes="xsl fn xs camel">
        <reference interface="org.apache.camel.Component" xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
            <xsl:attribute name="id" select="@name"/>
            <xsl:attribute name="filter" select="concat('(connection.name=', @name, ')')"/>
        </reference>
        <xsl:variable name="dir">
            <xsl:choose>
                <xsl:when test="@path">
                    <xsl:value-of select="@path"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:text></xsl:text>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:choose>
            <xsl:when test="@dynamic = 'true'">
                <toD>
                    <xsl:variable name="queryString">
                        <xsl:for-each select="./element()">
                            <xsl:call-template name="convert-to-query"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="resultQS">
                        <xsl:choose>
                            <xsl:when test="string-length($queryString) > 0">
                                <xsl:value-of select="concat('?', $queryString)"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:text/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:attribute name="uri" select="concat(@name,':' ,$dir , $resultQS)"/>
                </toD>
            </xsl:when>
            <xsl:otherwise>
                <to>
                    <xsl:variable name="queryString">
                        <xsl:for-each select="./element()">
                            <xsl:call-template name="convert-to-query"/>
                        </xsl:for-each>
                    </xsl:variable>
                    <xsl:variable name="resultQS">
                        <xsl:choose>
                            <xsl:when test="string-length($queryString) > 0">
                                <xsl:value-of select="concat('?', $queryString)"/>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:text/>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:attribute name="uri" select="concat(@name,':' ,$dir , $resultQS)"/>
                </to>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- use-connection -->
    <xsl:template match="//m:use-connection" exclude-result-prefixes="xsl fn xs camel">
        <reference interface="org.apache.camel.Component" xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
            <xsl:attribute name="id" select="@name"/>
            <xsl:attribute name="filter" select="concat('(connection.name=', @name, ')')"/>
        </reference>
    </xsl:template>

    <!-- call-route -->
    <xsl:template match="//m:call-route" exclude-result-prefixes="xsl fn xs camel">

        <xsl:variable name="finalRouteId" select="@name"/>
        <xsl:variable name="finalOwner">
            <xsl:choose>
                <xsl:when test="$routeOwnerId != 'unknown'">
                    <xsl:value-of select="$routeOwnerId"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$context.objectId"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="finalLocal">
            <xsl:choose>
                <xsl:when test="@library and not(@library = '')">
                    <xsl:value-of select="'false'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'true'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="finalLibrary">
            <xsl:choose>
                <xsl:when test="@library and not(@library = '')">
                    <xsl:value-of select="@library"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$finalOwner"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="finalAsync">
            <xsl:choose>
                <xsl:when test="@async">
                    <xsl:value-of select="@async"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="finalSchema">
            <xsl:choose>
                <xsl:when test="$finalLocal = 'true'">
                    <xsl:choose>
                        <xsl:when test="$finalAsync = 'true'">
                            <xsl:value-of select="'seda'"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="'direct'"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:choose>
                        <xsl:when test="$finalAsync = 'true'">
                            <xsl:value-of select="'vm'"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="'direct-vm'"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:variable name="finalUri">
            <xsl:value-of select="concat($finalSchema, ':', $finalLibrary, '__', $finalRouteId)"/>
        </xsl:variable>

        <xsl:variable name="finalDynamic">
            <xsl:choose>
                <!--
                @TODO investigate why 'matches' doesn't work corectly in Saxon
                <xsl:when test="matches($finalUri, '.*(\$\{.+\}).*')">
                -->
                <!--
                @TODO investigate why we have to get '${' via concatenation rather than setting it directly
                seems to be Saxon-related issue
                -->
                <xsl:when test="contains($finalUri, concat('$','{'))">
                    <xsl:value-of select="'true'"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="'false'"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="finalTo">
            <xsl:choose>
                <xsl:when test="$finalDynamic = 'true'">
                    <toD>
                        <xsl:attribute name="uri" select="$finalUri"/>
                    </toD>
                </xsl:when>
                <xsl:otherwise>
                    <to>
                        <xsl:attribute name="uri" select="$finalUri"/>
                    </to>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>

        <xsl:text>&#10;</xsl:text>
        <xsl:comment>
            <xsl:text> Generated from 'call-route': name=</xsl:text><xsl:value-of select="@name"/><xsl:text>; library=</xsl:text><xsl:value-of
                select="@library"/><xsl:text>; async=</xsl:text><xsl:value-of select="@async"/><xsl:text> </xsl:text>
        </xsl:comment>
        <xsl:text>&#10;</xsl:text>
        <xsl:choose>
            <xsl:when test="$finalAsync = 'true'">
                <multicast shareUnitOfWork="true">
                    <xsl:copy-of select="$finalTo"/>
                </multicast>
            </xsl:when>
            <xsl:otherwise>
                <xsl:copy-of select="$finalTo"/>
            </xsl:otherwise>
        </xsl:choose>

    </xsl:template>

	<!-- map-data -->
	<xsl:template match="//m:map-data" exclude-result-prefixes="xsl fn xs camel">
		<reference interface="ru.entaxy.platform.modules.datamapper.DataMapperService"
				   xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0" id="">
			<xsl:attribute name="id" select="'data-mapper-service'"/>
		</reference>
		<bean class="ru.entaxy.platform.modules.datamapper.camel.DataMapperCamelProcessor" activation="eager"
			  xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0">
			<xsl:attribute name="id" select="'data-mapper-camel-processor'"/>
			<property name="dataMapperService">
				<xsl:attribute name="ref" select="'data-mapper-service'"/>
			</property>
		</bean>		
        <xsl:comment>
            <xsl:text> Generated from 'map-data': mapper=</xsl:text><xsl:value-of select="@mapper"/><xsl:text>; map=</xsl:text><xsl:value-of
                select="@map"/><xsl:text> </xsl:text>
        </xsl:comment>		
		<step>
			<xsl:attribute name="id" select="concat('map-data_', generate-id())" />
			<xsl:if test="@mapper">
				<setProperty name="NTX_data_mapper_id">
					<simple><xsl:value-of select="@mapper" /></simple>
				</setProperty>
			</xsl:if>	
			<xsl:if test="@map">
				<setProperty name="NTX_data_mapper_map_name">
					<simple><xsl:value-of select="@map" /></simple>
				</setProperty>
			</xsl:if>	
			<xsl:if test="@noDataAction">
				<setProperty name="NTX_data_mapper_no_data_action">
					<simple><xsl:value-of select="@noDataAction" /></simple>
				</setProperty>
			</xsl:if>	
			<xsl:if test="@noDataValue">
				<setProperty name="NTX_data_mapper_no_data_value">
					<simple><xsl:value-of select="@noDataValue" /></simple>
				</setProperty>
			</xsl:if>	
			<xsl:if test="@trimForString">
				<setProperty name="NTX_data_mapper_trim_for_string">
					<simple><xsl:value-of select="@trimForString" /></simple>
				</setProperty>
			</xsl:if>
			<xsl:if test="@returnTo">
				<setProperty name="NTX_data_mapper_return_property">
					<simple><xsl:value-of select="@returnTo" /></simple>
				</setProperty>
			</xsl:if>
			<setProperty name="NTX_data_mapper_input">
			<xsl:choose>
				<xsl:when test="./element()">
					<xsl:copy-of select="child::element()[1]" />
				</xsl:when>
				<xsl:otherwise>
					<simple>${body}</simple>
				</xsl:otherwise>
			</xsl:choose>
			</setProperty>
			<to uri="bean:data-mapper-camel-processor" />
			<removeProperties pattern="NTX_data_mapper_*" />
		</step>
		
	</xsl:template>
	
	<!-- 
		BEGIN: set-content
	-->
	<xsl:template match="//m:set-content" exclude-result-prefixes="xsi xsl fn xs camel">

		<xsl:variable name="targetIsBody" as="xs:boolean">
			<xsl:choose>
				<xsl:when test="@targetType='header' or @targetType='property'"><xsl:value-of select="fn:false()" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="fn:true()" /></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="stepId">
			<xsl:choose>
				<xsl:when test="@id"><xsl:value-of select="@id" /></xsl:when>
				<xsl:otherwise><xsl:value-of select="concat('set-content-', generate-id(.))" /></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="elementName">
			<xsl:choose>
				<xsl:when test="@targetType = 'header'"><xsl:text>setHeader</xsl:text></xsl:when>
				<xsl:when test="@targetType = 'property'"><xsl:text>setProperty</xsl:text></xsl:when>
				<xsl:otherwise>setBody</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="targetName">
			<xsl:choose>
				<xsl:when test="@target"><xsl:value-of select="@target" /></xsl:when>
				<xsl:otherwise><xsl:text>__CONTENT__</xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="argIndexes" as="xs:string*">
			<xsl:for-each select="./m:args/m:arg[@index and child::element()[1]]">
				<xsl:sequence select="@index" />
			</xsl:for-each>
		</xsl:variable>	
		
		<xsl:variable name="argNames" as="xs:string*">
			<xsl:for-each select="./m:args/m:arg[@name and fn:string-length(fn:normalize-space(@name))>0 and child::element()[1]]">
				<xsl:sequence select="fn:normalize-space(@name)" />
			</xsl:for-each>
		</xsl:variable>	
		
		
        <xsl:comment>
            <xsl:text> Generated from 'set-content' </xsl:text>
        </xsl:comment>	
        <xsl:text>&#10;</xsl:text>
		<step>
			<xsl:attribute name="id" select="$stepId" />
			<xsl:if test="./m:description">
				<description>
					<xsl:copy-of select="./m:description/node()" />
				</description>
			</xsl:if>
			<removeProperties pattern="NTX_CONTENT_ARG_*" />
			<xsl:for-each select="./m:args/m:arg[@index and child::element()[1]]">
				<setProperty>
					<xsl:attribute name="name" select="concat('NTX_CONTENT_ARG_', ./@index)" />
                    <xsl:apply-templates select="./child::element()[1]"/>
				</setProperty>
			</xsl:for-each>
			<xsl:for-each select="./m:args/m:arg[@name and child::element()[1]]">
				<setHeader>
					<xsl:attribute name="name" select="fn:normalize-space(@name)" />
                    <xsl:apply-templates select="./child::element()[1]"/>
				</setHeader>
			</xsl:for-each>

			<!-- main element -->
			<xsl:choose>
				<xsl:when test="./m:content">
					<xsl:call-template name="process-content">
						<xsl:with-param name="contentNode" as="node()">
							<xsl:copy-of select="./m:content" exclude-result-prefixes="fn entaxy camel"/>
						</xsl:with-param>
						<xsl:with-param name="argIndexes" select="$argIndexes" as="xs:string*" />
						<xsl:with-param name="elementName" select="$elementName" as="xs:string" />
						<xsl:with-param name="targetName" select="$targetName" as="xs:string" />
						<xsl:with-param name="targetIsBody" select="$targetIsBody" as="xs:boolean" />
					</xsl:call-template>
				</xsl:when>
				<xsl:when test="./m:freemarker">
					<xsl:call-template name="process-freemarker">
						<xsl:with-param name="freemarkerNode" select="./m:freemarker" as="node()" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:comment select="concat('WARNING: unknown element in set-content: ', ./element()[1]/fn:local-name())"></xsl:comment>
				</xsl:otherwise>
			</xsl:choose>

			<removeProperties pattern="NTX_CONTENT_ARG_*" />
			<xsl:for-each select="$argNames">
				<removeHeader>
					<xsl:attribute name="headerName" select="." />
				</removeHeader>
			</xsl:for-each>

            <xsl:choose>
                <xsl:when test="@type='xml'">
                    <convertBodyTo type="org.w3c.dom.Document"/>
                </xsl:when>
                <xsl:when test="@type='json'">
                    <marshal>
                        <json library="Gson"/>
                    </marshal>
                </xsl:when>
            </xsl:choose>

		</step>	
	</xsl:template>
	
	<xsl:template name="process-content" exclude-result-prefixes="xsl fn xs camel">
		<xsl:param name="contentNode" as="node()" />
		<xsl:param name="argIndexes" as="xs:string*" />
		<xsl:param name="elementName" as="xs:string" />
		<xsl:param name="targetName" as="xs:string" />
		<xsl:param name="targetIsBody" as="xs:boolean" />
		
		<xsl:variable name="contentHasElements" select="fn:count($contentNode/element())>0" as="xs:boolean" />

		<xsl:variable name="clean" as="node()*">
			<xsl:for-each select="$contentNode/child::node()">
				<!--xsl:copy-of select="." exclude-result-prefixes="fn camel entaxy xsl" copy-namespaces="yes" / -->
				<xsl:apply-templates select="." mode="clean" />
			</xsl:for-each>
		</xsl:variable>
		<xsl:variable name="content" as="xs:string">
			<xsl:choose>
				<xsl:when test="$contentHasElements">
					<xsl:value-of select="fn:serialize($clean, map{'version': '1.1', 'method':'xml','omit-xml-declaration': fn:true(), 'undeclare-prefixes': fn:true()})" exclude-result-prefixes="fn camel entaxy" disable-output-escaping="yes"/>
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$clean" disable-output-escaping="yes" /></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		
		<xsl:variable name="substitutedSeq1" as="xs:string*">
			<xsl:analyze-string regex="[^\$]\$([0-9]+)" select="$content" flags="m">
				<xsl:matching-substring>
					<xsl:choose>
						<xsl:when test="fn:exists(fn:index-of($argIndexes, fn:regex-group(1)))">
							<xsl:value-of select="concat(fn:substring(current(), 1, 2) ,'{exchangeProperty.NTX_CONTENT_ARG_', fn:regex-group(1), '}')" />
						</xsl:when>
						<xsl:otherwise><xsl:value-of select="current()"/></xsl:otherwise>
					</xsl:choose>
				</xsl:matching-substring>
				<xsl:non-matching-substring>
					<xsl:value-of select="current()"/>
				</xsl:non-matching-substring>
			</xsl:analyze-string>
		</xsl:variable>
		<xsl:variable name="substitutedSeq" as="xs:string*">
			<xsl:analyze-string regex="(\$\$)" select="fn:string-join($substitutedSeq1)" flags="m">
				<xsl:matching-substring>
					<xsl:text>$</xsl:text>
				</xsl:matching-substring>
				<xsl:non-matching-substring>
					<xsl:value-of select="current()"/>
				</xsl:non-matching-substring>
			</xsl:analyze-string>
		</xsl:variable>
		<xsl:variable name="substituted" select="fn:string-join($substitutedSeq, '')" as="xs:string" />
		<xsl:variable name="nodes" as="node()*">
			<m:set-content>
				<xsl:attribute name="type" select="$elementName" />
				<xsl:value-of select="$substituted" />
			</m:set-content>
		</xsl:variable>
			<xsl:for-each select="$nodes">
				<xsl:element name="{$elementName}">
					<xsl:if test="not($targetIsBody)">
						<xsl:attribute name="name"><xsl:value-of select="$targetName" /></xsl:attribute>
					</xsl:if>
					<simple>
						<xsl:value-of select="concat('&lt;![CDATA[',node(), ']]&gt;')" disable-output-escaping="yes"/>
					</simple>	
				</xsl:element>
			</xsl:for-each>
	
	</xsl:template>


	<xsl:template name="process-freemarker" exclude-result-prefixes="xsl fn xs camel">
		<xsl:param name="freemarkerNode" as="node()" />

		<xsl:variable name="templateUriSet" as="xs:boolean">
			<xsl:value-of select="fn:string-length($freemarkerNode/@resource)>0" />
		</xsl:variable>
		<xsl:variable name="templateUri" as="xs:string">
			<xsl:value-of select="$freemarkerNode/@resource" />
		</xsl:variable>
		<xsl:variable name="templateFromBody" as="xs:boolean">
			<xsl:value-of select="$freemarkerNode/@fromBody='true'" />
		</xsl:variable>
		<xsl:variable name="templateFromHeader" as="xs:boolean">
			<xsl:value-of select="fn:string-length($freemarkerNode/@fromHeader)>0" />
		</xsl:variable>
		<xsl:variable name="templateFromTag" as="xs:boolean">
			<xsl:value-of select="fn:string-length(fn:normalize-space($freemarkerNode/text()))>0" />
		</xsl:variable>
        <xsl:variable name="params"/>
        <xsl:variable name="params">
            <xsl:for-each select="$freemarkerNode/@*">
                <xsl:if test="name() != 'fromHeader' and name() != 'fromBody' and name() != 'resource'">
                    <xsl:value-of select="concat($params, '&amp;', name(), '=', .)"/>
                </xsl:if>
            </xsl:for-each>
        </xsl:variable>

		<xsl:choose>
			<xsl:when test="$templateFromTag">
				<setHeader name="CamelFreemarkerTemplate">
					<constant><xsl:value-of select="$freemarkerNode/text()"/></constant>
				</setHeader>
			</xsl:when>
			<xsl:otherwise>
				<xsl:choose>
					<xsl:when test="$templateFromBody">
						<setHeader name="CamelFreemarkerTemplate">
							<simple>${body}</simple>
						</setHeader>
						<removeHeader headerName="CamelFreemarkerResourceUri" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="$templateFromHeader">
								<setHeader name="CamelFreemarkerTemplate">
									<simple><xsl:value-of select="concat('$', '{headers.', $freemarkerNode/@fromHeader, '}')" /></simple>
								</setHeader>
								<removeHeader headerName="CamelFreemarkerResourceUri" />
							</xsl:when>
							<xsl:otherwise>
								<xsl:comment>WARNING! Template uri or body not explicitly set!</xsl:comment>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>

		<xsl:variable name="contentCache">
			<xsl:choose>
				<xsl:when test="$freemarkerNode/@contentCache"><xsl:value-of select="$freemarkerNode/@contentCache" /></xsl:when>
				<xsl:otherwise><xsl:text>true</xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>	
		<xsl:variable name="allowTemplateFromHeader">
			<xsl:choose>
                <xsl:when test="$templateFromBody or $templateFromHeader or $templateFromTag"><xsl:text>true</xsl:text></xsl:when>
                <!-- for security reasons -->
				<xsl:otherwise><xsl:text>false</xsl:text></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>	

		<to>
            <xsl:choose>
                <xsl:when test="$templateUriSet">
                    <xsl:attribute name="uri" select="concat('freemarker:', $templateUri, '?contentCache=', $contentCache, $params)" />
                </xsl:when>
                <xsl:otherwise>
                    <xsl:attribute name="uri" select="concat('freemarker:none?contentCache=', $contentCache, '&amp;allowTemplateFromHeader=', $allowTemplateFromHeader, $params)" />
                </xsl:otherwise>
            </xsl:choose>
		</to>

		<removeHeader headerName="CamelFreemarkerResourceUri" />
		<removeHeader headerName="CamelFreemarkerTemplate" />

	</xsl:template>

	<!-- 
		END: set-content
	-->

	
    <!-- entaxy-trace -->
    <!--
        USAGE:
        <m:entaxy-trace reset="true">
            <simple>${headers.HEADER_TO_TRACE}</simple>
        </m:entaxy-trace>
    -->
    <xsl:template match="//m:entaxy-trace" exclude-result-prefixes="xsl fn xs camel">
        <xsl:choose>
            <xsl:when test="@reset = true()">
                <setHeader name="ENTAXY_TRACE">
                    <constant></constant>
                </setHeader>
            </xsl:when>
        </xsl:choose>
        <xsl:choose>
            <xsl:when test="child::element()[1]">
                <choice>
                    <when>
                        <simple>${headers.ENTAXY_TRACE} != null</simple>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>${headers.ENTAXY_TRACE} -&gt;</simple>
                        </setHeader>
                    </when>
                </choice>
                <setHeader name="NTX_TRACE_RECORD">
                    <xsl:apply-templates select="child::element()[1]"/>
                </setHeader>
                <choice>
                    <when>
                        <simple>${headers.ENTAXY_TRACE} == null</simple>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>[${headers.NTX_TRACE_RECORD}]</simple>
                        </setHeader>
                    </when>
                    <otherwise>
                        <setHeader name="ENTAXY_TRACE">
                            <simple>${headers.ENTAXY_TRACE}[${headers.NTX_TRACE_RECORD}]</simple>
                        </setHeader>
                    </otherwise>
                </choice>
                <removeHeader headerName="NTX_TRACE_RECORD"/>
            </xsl:when>
        </xsl:choose>
    </xsl:template>


    <!--
        END components
    -->
	<xsl:template match="*" mode="clean">
		<xsl:element name="{name()}" namespace="{namespace-uri()}">
		<xsl:variable name="vtheElem" select="."/>
		
		<xsl:for-each select="namespace::*">
			<xsl:variable name="vPrefix" select="name()"/>
			
			<xsl:if test=
			"$vtheElem/descendant::*
				  [(namespace-uri()=current()
				 and 
				  substring-before(name(),':') = $vPrefix)
				 or
				  @*[substring-before(name(),':') = $vPrefix]
				  ]
			">
			<xsl:copy-of select="."/>
			</xsl:if>
		</xsl:for-each>
		<xsl:apply-templates select="node()|@*" mode="clean" />
		</xsl:element>
	 </xsl:template>

	 <xsl:template match="node()|@*" priority="-2" mode="clean">
		 <xsl:copy>
		   <xsl:apply-templates select="node()|@*" mode="clean" />
		 </xsl:copy>
	 </xsl:template>	 

    <xsl:template match="/">
        <xsl:apply-templates select="child::node()"/>
    </xsl:template>

    <xsl:template name="default-copy" match="node()|@*" exclude-result-prefixes="fn xs xsi">
        <xsl:copy>
            <xsl:apply-templates select="node()|@*"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template name="convert-to-query" exclude-result-prefixes="xsl fn xs camel">
        <xsl:value-of select="concat(node-name(.), '=', text())"/>
        <xsl:if test="./following-sibling::element()">
            <xsl:text>&amp;</xsl:text>
        </xsl:if>
    </xsl:template>

</xsl:stylesheet>
