<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2025 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
				xmlns:fn="http://www.w3.org/2005/xpath-functions"  xmlns:entaxy="http://www.entaxy.ru/schemas/1.0"
				xmlns:m="http://www.entaxy.ru/schemas/entaxy-mediators/1.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
				xmlns:blueprint="http://www.osgi.org/xmlns/blueprint/v1.0.0">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	
	<xsl:param name="context.type" select="'entaxy.runtime.route'" />
	<xsl:param name="context.fieldName" select="'route'" />
	<xsl:param name="context.objectId" select="'route-00'" />
	
	<xsl:param name="context.factory.typeinfo.routeType" select="'COMMON'" />
	<xsl:param name="context.factory.typeinfo.routePattern" select="'ABSTRACT'" />
	<!-- alias -->	
	<xsl:param name="routeType" select="$context.factory.typeinfo.routeType" />
	<xsl:param name="routePattern" select="$context.factory.typeinfo.routePattern" />
	
	<!-- only for callable routes -->
	<xsl:param name="context.properties.globalCallMode" select="'NONE'" />
	<xsl:param name="context.properties.localCallMode" select="'SYNC'" />
	<!-- aliases -->
	<xsl:param name="globalCallMode" select="$context.properties.globalCallMode" />
	<xsl:param name="localCallMode" select="$context.properties.localCallMode" />
	
	<!-- only for eggregate routes -->	
	<xsl:param name="context.properties.aggregatorId" select="'none'" />
	
	<xsl:template match="//entaxy:common-route" exclude-result-prefixes="fn xs">
		<xsl:choose>
			<xsl:when test="$routeType = 'CALLABLE'">
				<!-- profile route fields processing -->
				
				<xsl:variable name="localSync">
					<xsl:choose>
						<xsl:when test="$localCallMode = 'SYNC' or $localCallMode='BOTH'">
							<xsl:value-of select="true()" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="false()" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>	
				<xsl:variable name="localAsync">
					<xsl:choose>
						<xsl:when test="$localCallMode = 'ASYNC' or $localCallMode='BOTH'">
							<xsl:value-of select="true()" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="false()" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>	
				<xsl:variable name="globalSync">
					<xsl:choose>
						<xsl:when test="$globalCallMode = 'SYNC' or $globalCallMode='BOTH'">
							<xsl:value-of select="true()" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="false()" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>	
				<xsl:variable name="globalAsync">
					<xsl:choose>
						<xsl:when test="$globalCallMode = 'ASYNC' or $globalCallMode='BOTH'">
							<xsl:value-of select="true()" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="false()" />
						</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>	

				<xsl:variable name="routeInternalId">
					<xsl:value-of select="concat($context.objectId, '-route-internal')" />
				</xsl:variable>	
				
				<!-- output entry points -->	
				<xsl:comment>
					<xsl:text>&#10;</xsl:text><xsl:text>globalAsync = </xsl:text><xsl:value-of select="$globalAsync" />
					<xsl:text>&#10;</xsl:text><xsl:text>globalSync = </xsl:text><xsl:value-of select="$globalSync" />
					<xsl:text>&#10;</xsl:text><xsl:text>localAsync = </xsl:text><xsl:value-of select="$localAsync" />
					<xsl:text>&#10;</xsl:text><xsl:text>localSync = </xsl:text><xsl:value-of select="$localSync" />
				</xsl:comment>
				<!-- seda -->
				<xsl:if test="$localAsync = true()">
					<xsl:call-template name="callable-route-entry-point">
						<xsl:with-param name="mode">async</xsl:with-param>
						<xsl:with-param name="visibility">local</xsl:with-param>
						<xsl:with-param name="protocol">seda</xsl:with-param>
						<xsl:with-param name="objectId" select="$context.objectId" />
						<xsl:with-param name="routeInternalId" select="$routeInternalId" />
					</xsl:call-template>
					<!--
					<route xmlns="http://camel.apache.org/schema/blueprint">
						<xsl:text>&#10;</xsl:text>
						<xsl:comment>local async (seda) entry point for <xsl:value-of select="$context.objectId"/></xsl:comment>	
						<from>
							<xsl:attribute name="uri">
								<xsl:text>seda:</xsl:text><xsl:value-of select="$context.objectId" />
							</xsl:attribute>
						</from>
						<to>
							<xsl:attribute name="uri">
								<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
							</xsl:attribute>
						</to>
					</route>
					-->
				</xsl:if>
				<!-- direct -->
				<xsl:if test="$localSync = true()">
					<xsl:call-template name="callable-route-entry-point">
						<xsl:with-param name="mode">sync</xsl:with-param>
						<xsl:with-param name="visibility">local</xsl:with-param>
						<xsl:with-param name="protocol">direct</xsl:with-param>
						<xsl:with-param name="objectId" select="$context.objectId" />
						<xsl:with-param name="routeInternalId" select="$routeInternalId" />
					</xsl:call-template>
					<!--
					<route xmlns="http://camel.apache.org/schema/blueprint">
						<xsl:text>&#10;</xsl:text>
						<xsl:comment>local sync (direct) entry point for <xsl:value-of select="$context.objectId"/></xsl:comment>	
						<from>
							<xsl:attribute name="uri">
								<xsl:text>direct:</xsl:text><xsl:value-of select="$context.objectId" />
							</xsl:attribute>
						</from>
						<to>
							<xsl:attribute name="uri">
								<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
							</xsl:attribute>
						</to>
					</route>
					-->
				</xsl:if>
				<!-- vm -->
				<xsl:if test="$globalAsync = true()">
					<xsl:call-template name="callable-route-entry-point">
						<xsl:with-param name="mode">async</xsl:with-param>
						<xsl:with-param name="visibility">global</xsl:with-param>
						<xsl:with-param name="protocol">vm</xsl:with-param>
						<xsl:with-param name="objectId" select="$context.objectId" />
						<xsl:with-param name="routeInternalId" select="$routeInternalId" />
					</xsl:call-template>
					<!--
					<route xmlns="http://camel.apache.org/schema/blueprint">
						<xsl:text>&#10;</xsl:text>
						<xsl:comment>global async (vm) entry point for <xsl:value-of select="$context.objectId"/></xsl:comment>	
						<from>
							<xsl:attribute name="uri">
								<xsl:text>vm:</xsl:text><xsl:value-of select="$context.objectId" />
							</xsl:attribute>
						</from>
						<to>
							<xsl:attribute name="uri">
								<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
							</xsl:attribute>
						</to>
					</route>
					-->
				</xsl:if>
				<!-- direct-vm -->
				<xsl:if test="$globalSync = true()">
					<xsl:call-template name="callable-route-entry-point">
						<xsl:with-param name="mode">sync</xsl:with-param>
						<xsl:with-param name="visibility">global</xsl:with-param>
						<xsl:with-param name="protocol">direct-vm</xsl:with-param>
						<xsl:with-param name="objectId" select="$context.objectId" />
						<xsl:with-param name="routeInternalId" select="$routeInternalId" />
					</xsl:call-template>
					<!--
					<route xmlns="http://camel.apache.org/schema/blueprint">
						<xsl:text>&#10;</xsl:text>
						<xsl:comment>global sync (direct-vm) entry point for <xsl:value-of select="$context.objectId"/></xsl:comment>	
						<from>
							<xsl:attribute name="uri">
								<xsl:text>direct-vm:</xsl:text><xsl:value-of select="$context.objectId" />
							</xsl:attribute>
						</from>
						<to>
							<xsl:attribute name="uri">
								<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
							</xsl:attribute>
						</to>
					</route>
					-->
				</xsl:if>
				
				<!-- route itself -->
				<route xmlns="http://camel.apache.org/schema/blueprint" delayer="0">
					<xsl:attribute name="id"><xsl:value-of select="$context.objectId" /></xsl:attribute>
					<xsl:variable name="uri">
						<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
					</xsl:variable>
					<xsl:text>&#10;</xsl:text>
					<xsl:comment>generated from 'entaxy:common-route' for <xsl:value-of select="$context.objectId"/>:<xsl:value-of select="$context.type"/></xsl:comment>
					<from>
						<xsl:attribute name="uri" select="$uri" />
					</from>
					<xsl:choose>
						<xsl:when test="$routePattern = 'AGGREGATOR'">
							<aggregate>
								<xsl:attribute name="id"><xsl:value-of select="$context.properties.aggregatorId" /></xsl:attribute>
								<xsl:apply-templates select="node()" />								
							</aggregate>
						</xsl:when>
						<xsl:otherwise>
							<xsl:apply-templates select="node()" />								
						</xsl:otherwise>
					</xsl:choose>
				</route>
				
			</xsl:when>
			<xsl:when test="$routeType = 'SCHEDULED'">
				<route xmlns="http://camel.apache.org/schema/blueprint" delayer="0">
					<xsl:attribute name="id"><xsl:value-of select="$context.objectId" /></xsl:attribute>
					<xsl:comment>generated from 'entaxy:common-route' for <xsl:value-of select="$context.objectId"/>:<xsl:value-of select="$context.type"/></xsl:comment>
					<xsl:apply-templates select="node()" />								
				</route>
			</xsl:when>
			<xsl:when test="$routeType = 'LISTENER'">
				<route xmlns="http://camel.apache.org/schema/blueprint" delayer="0">
					<xsl:attribute name="id"><xsl:value-of select="$context.objectId" /></xsl:attribute>
					<xsl:comment>generated from 'entaxy:common-route' for <xsl:value-of select="$context.objectId"/>:<xsl:value-of select="$context.type"/></xsl:comment>
					<xsl:apply-templates select="node()" />								
				</route>
			</xsl:when>
			<xsl:otherwise>
				<!-- UNSUPPORTED ROUTE TYPE -->
				<xsl:comment>
					<xsl:text>WARNING: UNSUPPORTED ROUTE TYPE: </xsl:text><xsl:value-of select="$routeType" />
				</xsl:comment>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

	<xsl:template name="callable-route-entry-point" exclude-result-prefixes="fn xs">
		<xsl:param name="mode" select="'sync'" />
		<xsl:param name="visibility" select="'global'" />
		<xsl:param name="protocol" select="'direct-vm'" />
		<xsl:param name="objectId" select="'route-0'" />
		<xsl:param name="routeInternalId" select="'route-0-route-internal'" />
		<route xmlns="http://camel.apache.org/schema/blueprint">
			<xsl:attribute name="id">
				<xsl:value-of select="$objectId" /><xsl:text>-</xsl:text><xsl:value-of select="$visibility" /><xsl:text>-</xsl:text><xsl:value-of select="$mode" /><xsl:text>-entry-point</xsl:text>
			</xsl:attribute>
			<xsl:text>&#10;</xsl:text>
			<xsl:comment><xsl:value-of select="$visibility"/><xsl:text> </xsl:text><xsl:value-of select="$mode"/> (<xsl:value-of select="$protocol"/>) entry point for <xsl:value-of select="$objectId"/></xsl:comment>	
			<from>
				<xsl:attribute name="uri">
					<xsl:value-of select="$protocol"/><xsl:text>:</xsl:text><xsl:value-of select="$objectId" />
				</xsl:attribute>
			</from>
			<to>
				<xsl:attribute name="uri">
					<xsl:text>direct:</xsl:text><xsl:value-of select="$routeInternalId" />
				</xsl:attribute>
			</to>
		</route>
	</xsl:template>

	<xsl:template match="/">
		<xsl:apply-templates select="child::node()" />
	</xsl:template>
		
	<xsl:template name="default-copy" match="node()|@*" exclude-result-prefixes="fn xs xsi">
		<xsl:copy>
			<xsl:apply-templates select="node()|@*" />
		</xsl:copy>
	</xsl:template>		
	
</xsl:stylesheet>
