<?xml version="1.0" encoding="UTF-8"?>
<!--
  ~~~~~~licensing~~~~~~
  route-producing
  ==========
  Copyright (C) 2020 - 2025 EmDev LLC
  ==========
  You may not use this file except in accordance with the License Terms of the Copyright
  Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
  rights to the Software and any copies are the property of the Copyright Holder. Unless
  it is explicitly allowed the Copyright Holder, the User is prohibited from using the
  Software for commercial purposes to provide services to third parties.
  
  The Copyright Holder hereby declares that the Software is provided on an "AS IS".
  Under no circumstances does the Copyright Holder guarantee or promise that the
  Software provided by him will be suitable or not suitable for the specific purposes
  of the User, that the Software will meet all commercial and personal subjective
  expectations of the User, that the Software will work properly, without technical
  errors, quickly and uninterruptedly.
  
  Under no circumstances shall the Copyright Holder or its Affiliates is not liable
  to the User for any direct or indirect losses of the User, his expenses or actual
  damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
  or damage to data, property, etc.
  ~~~~~~/licensing~~~~~~
  -->

<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:fn="http://www.w3.org/2005/xpath-functions"  xmlns:entaxy="http://www.entaxy.ru/schemas/1.0"  xmlns:m="http://www.entaxy.ru/schemas/entaxy-mediators/1.0" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:blueprint="http://www.osgi.org/xmlns/blueprint/v1.0.0" xmlns:local="http://www.entaxy.ru/schemas/entaxy-mediators/connector-selector/1.0">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" indent="yes"/>
	
	<xsl:param name="context.type" select="'entaxy.runtime.profile'" />
	<xsl:param name="context.fieldName" select="'in-flow-pre-route'" />
	<xsl:param name="context.objectId" select="'sys-13'" />
	
	<xsl:param name="context.factory.typeinfo.serviceType" select="'UNKNOWN'" />
	<xsl:param name="context.factory.typeinfo.serviceScope" select="'UNKNOWN'" />
	
	<xsl:template match="//m:operation-router" exclude-result-prefixes="xsl fn xs local">
		<choice xmlns="http://camel.apache.org/schema/blueprint">
			<xsl:apply-templates select="child::node()" />
		</choice>
	</xsl:template>

	<xsl:template match="//m:operation-router/m:operation" exclude-result-prefixes="xsl fn xs local">
		<when xmlns="http://camel.apache.org/schema/blueprint">
			<simple>${headers.operationName} == '<xsl:value-of select="@name" />'</simple>
			<xsl:apply-templates select="child::node()" />
		</when>
	</xsl:template>
	
	<xsl:template match="//m:operation-router/m:unknown-operation" exclude-result-prefixes="xsl fn xs local">
		<otherwise xmlns="http://camel.apache.org/schema/blueprint">
			<xsl:apply-templates select="child::node()" />
		</otherwise>
	</xsl:template>

	<xsl:template match="//m:service-connector" exclude-result-prefixes="xsl fn xs local">
		<xsl:choose>
			<xsl:when test="$context.factory.typeinfo.serviceScope = 'protected'">
				<log loggingLevel="INFO" xmlns="http://camel.apache.org/schema/blueprint">
					<xsl:attribute name="message" select="concat('${headers.NTX_loggingKey} Sending to connector: ', @direction)"></xsl:attribute>
				</log>
				<to xmlns="http://camel.apache.org/schema/blueprint">
					<xsl:attribute name="uri" select="concat('bean:', $context.objectId, '-connector-registry?method=', @direction, 'ConnectorFind')"></xsl:attribute>
				</to>
				<choice xmlns="http://camel.apache.org/schema/blueprint">
					<when>
						<simple>${headers.NTX_targetConnector} != null</simple>
						<setProperty name="NTX_TMP_targetConnector">
							<simple>${headers.NTX_targetConnector}</simple>
						</setProperty>
						<removeHeader headerName="NTX_targetConnector" />
						<toD>
							<xsl:attribute name="uri" select="concat('${exchangeProperty.NTX_TMP_targetConnector}', '')" />
						</toD>
					</when>
					<otherwise>
						<throwException exceptionType="ru.entaxy.platform.services.runtime.exceptions.ConnectorNotFound">
							<xsl:attribute name="message" 
								select="concat('Connector not found: service=', $context.objectId, '; profile=${headers[X-SystemName]}; direction=', @direction)">
							</xsl:attribute>
						</throwException>
					</otherwise>
				</choice>	
			</xsl:when>
			<xsl:otherwise>
				<xsl:comment>m:service-connector element is only applicable for protected services</xsl:comment>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template match="//m:set-operation" exclude-result-prefixes="xsl fn xs local">
		<setHeader name="operationName" xmlns="http://camel.apache.org/schema/blueprint">
			<constant><xsl:value-of select="./text()" /></constant>
		</setHeader>
	</xsl:template>
	
	<xsl:template name="default-copy" match="node()|@*">
		<xsl:copy>
			<xsl:apply-templates select="node()|@*" />
		</xsl:copy>
	</xsl:template>
		
</xsl:stylesheet>
