[#ftl attributes={"generated.type":"blueprint"}]
<blueprint xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
           xmlns:cm="http://aries.apache.org/blueprint/xmlns/blueprint-cm/v1.1.0"
           xmlns:camelcxf="http://camel.apache.org/schema/blueprint/cxf"
           xmlns:cxf="http://cxf.apache.org/blueprint/core"
           xsi:schemaLocation="
             http://www.osgi.org/xmlns/blueprint/v1.0.0 https://www.osgi.org/xmlns/blueprint/v1.0.0/blueprint.xsd
             http://cxf.apache.org/blueprint/core http://cxf.apache.org/schemas/blueprint/core.xsd
             ">
[#import "templates:object-commons/common-utils.ftl" as utils]
[#include "customObjectReferences.ftl" ]
<!--
	
	INFO: 
	   
	factoryId: openapi-based-rest-service-public
	factoryType: entaxy.runtime.service
	outputType: init 
	properties:
	[#list properties?keys as key]
		[=key]
	[/#list]
 -->
 
	[#assign finalAddress=properties.address]
	[#if !finalAddress?starts_with('http://') && !finalAddress?starts_with('https://')]
		[#if !finalAddress?starts_with('/')]
			[#assign finalAddress='/'+properties.address]
		[/#if]
	[/#if]
	
	[#if properties._serviceMetadata.classes.apis??]	
	[#--  --list 0..properties._serviceMetadata.classes.apis?size-1 as i [/#list]--]
		[#assign apis = properties._serviceMetadata.classes.apis?join(",")]
	[/#if]
	<!--
	
	apis:  [=apis]
	path:  service-resources:[=properties._serviceMetadata.serviceResourcesRoot]/[=properties._serviceMetadata.classes.folder]/
 	-->
 	
 	<bean id="class-resolver-[=objectId]" activation="eager" class="ru.entaxy.platform.core.support.runtime.cxf.EntaxyClassResolver">
		<argument ref="blueprintBundleContext"/>
		<argument ref="[=camelContextId!]"/>
		<argument>
			<list value-type="java.lang.String">
				<value>service-resources:[=properties._serviceMetadata.serviceResourcesRoot]/[=properties._serviceMetadata.classes.folder]/</value>
			</list>
		</argument>
	</bean>
	
	<camelcxf:rsClient id="client-[=objectId]" 
			serviceClass="org.apache.cxf.jaxrs.client.WebClient"
			address="[=finalAddress]"
			[#if properties.authUsername??]
            username="[=authUsername]"
            [/#if]
            [#if properties.authPassword??]
            password="[=authPassword]"
            [/#if]
			>
		<camelcxf:properties>
			<entry key="http.receive.timeout" value="60"/>
		</camelcxf:properties>
<!--
		<camelcxf:providers>
			<ref component-id="jsonProvider"/>
		</camelcxf:providers>
-->
		<camelcxf:features>
			<ref component-id="openApiFeature"/>
		</camelcxf:features>
	</camelcxf:rsClient>

    <!-- JAXRS providers 
	<bean id="jsonProviderFactory" class="ru.entaxy.platform.core.support.runtime.jackson.JacksonJsonProviderFactory">
		<property name="addJodaModule" value="true"/>
		<property name="acceptSingleValueAsArray" value="true"/>
		<property name="failOnUnknownProperties" value="false"/>
	</bean>
	<bean id="jsonProvider" factory-ref="jsonProviderFactory" factory-method="create"/>
 	-->
 	<!-- CXF OpenApiFeature -->
    <bean id="openApiFeature" class="org.apache.cxf.jaxrs.openapi.OpenApiFeature">
        <property name="runAsFilter" value="true" />
        <property name="useContextBasedConfig" value="true" />
        <property name="scan" value="true" />
    </bean>
 
	<camelContext id="[=properties.camelContextId!]" xmlns="http://camel.apache.org/schema/blueprint">
		<route id="error_from_cxf">
            <from uri="direct:error_from_cxf"/>
            <log message="cxf message: ${body}" loggingLevel="DEBUG"/>
            <log message="headers: ${headers}\n" loggingLevel="DEBUG"/>
            <to uri="direct-vm:cxfErrorEndpoint"/>
        </route>

        <route id="[=objectId]-cxf-endpoint" streamCache="true">
            <from uri="direct-vm:call-[=objectId]-service" />
            
            [#if properties.oauthEnable?? && properties.oauthEnable?boolean == true]
            <to uri="direct:oauth-prepare" />
            [/#if]
            
            <to uri="direct:[=objectId]-operation-router" />
            <removeProperties pattern="NTX_CLIENT_*"/>
            [#if properties.oauthRefreshPolicy?? && properties.oauthRefreshPolicy == "error" 
				&& properties.oauthErrorCode?? && properties.oauthErrorCode != ""]
				[#global properties = properties + {"camel_throwExceptionOnFailure": "false"} ]
			<setProperty name="NTX_bodyHolder">
				<simple>${body}</simple>
			</setProperty>
			[/#if]
			
			[#global properties = properties + {"camel_resourceClasses": "[=apis]"} ]
            <to uri="cxfrs:bean:client-[=objectId][=utils.createQueryString(properties, "camel_", true, ["fieldToExclude1","fieldToExclude2"])]"/>
            
            <log loggingLevel="TRACE" message="headers\n ${headers}"/>
			<log loggingLevel="TRACE" message="properties\n ${exchange.getProperties()}"/>
            
            [#if properties.oauthRefreshPolicy?? && properties.oauthRefreshPolicy == "error" 
				&& properties.oauthErrorCode?? && properties.oauthErrorCode != ""]
			<to uri="direct:oauth-refresh-error" />
			<choice>
				<when>
					<simple>${exchangeProperty.oauthRefreshed}</simple>
					<setBody>
						<simple>${exchangeProperty.NTX_bodyHolder}</simple>
					</setBody>
			[#global properties = properties + {"camel_throwExceptionOnFailure": "true"} ]
					<to uri="cxfrs:bean:client-[=objectId][=utils.createQueryString(properties, "camel_", true, ["fieldToExclude1","fieldToExclude2"])]"/>
				</when>
				<when>
					<simple>${header.CamelHttpResponseCode} &gt; 207</simple>
					<throwException message="Call operation failed invoking ${exchangeProperty.CamelToEndpoint} with statusCode: ${header.CamelHttpResponseCode}" exceptionType="java.io.IOException"/>
				</when>
			</choice>
			[/#if]
        </route>
    </camelContext>

</blueprint> 
