/*-
 * ~~~~~~licensing~~~~~~
 * service-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.services.runtime.interceptor.jwt;

import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Message;
import org.apache.cxf.rs.security.jose.jws.JwsJwtCompactConsumer;
import org.apache.cxf.rs.security.jose.jwt.JwtToken;

import javax.ws.rs.ForbiddenException;
import java.util.List;
import java.util.Map;

public interface GetterToken {

    public static final String CXF_MESSAGE_PROTOCOL_HEADERS = "org.apache.cxf.message.Message.PROTOCOL_HEADERS";
    public static final String AUTHORIZATION = "Authorization";
    public static final String REALM_ACCESS = "realm_access";
    public static final String ROLES = "roles";

    default JwtToken getJwtToken(Message message) {
        Map headersMap = (Map) message.get(CXF_MESSAGE_PROTOCOL_HEADERS);
        List authorization = (List)headersMap.get(AUTHORIZATION);

        if (authorization == null || authorization.isEmpty())
            throw new Fault(new ForbiddenException("Header Authorization not found. Associated profile not found"));

        String authorizationString = (String)authorization.get(0);
        String[] fullToken = authorizationString.split(" ");
        if (fullToken.length != 2)
            throw new Fault(new ForbiddenException("Header Authorization not correctly. Associated profile not found"));
        String token = fullToken[1];

        JwsJwtCompactConsumer jwtConsumer = new JwsJwtCompactConsumer(token);
        JwtToken jwt = jwtConsumer.getJwtToken();
        return jwt;
    }

}
