/*-
 * ~~~~~~licensing~~~~~~
 * service-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
/**
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.entaxy.platform.services.runtime.interceptor.jwt;

import org.apache.cxf.interceptor.Fault;
import org.apache.cxf.message.Message;
import org.apache.cxf.rs.security.jose.jwt.JwtClaims;
import org.apache.cxf.rs.security.jose.jwt.JwtToken;
import org.apache.cxf.transport.http.Headers;
import ru.entaxy.platform.core.support.runtime.cxf.security.AnonymousSecurityContext;
import ru.entaxy.platform.services.runtime.interceptor.ResolveProfileNameInterceptor;

import javax.ws.rs.ForbiddenException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class ResolveProfileNameFromTokenInterceptor extends ResolveProfileNameInterceptor implements GetterToken {

    public ResolveProfileNameFromTokenInterceptor() {}

    public void handleMessage(Message message) throws Fault {
        Object securityContext = message.get("org.apache.cxf.security.SecurityContext");
        if (securityContext instanceof AnonymousSecurityContext) {
            return;
        }
        JwtToken jwt = getJwtToken(message);
        JwtClaims claims = jwt.getClaims();

        Map realmAccess = (Map)claims.getClaim(REALM_ACCESS);
        if (realmAccess == null || realmAccess.isEmpty()) {
            throw new Fault(new ForbiddenException("Real access not found. Associated profile not found"));
        }
        List<String> roles = (List)realmAccess.get(ROLES);

        if (roles == null || roles.isEmpty()) {
            throw new Fault(new ForbiddenException("Poles not found. Associated profile not found"));
        }
        boolean found = false;
        for (String role : roles) {
            if (role.startsWith(SYSTEM_ROLE_PREFIX)) {
                Map<String, List<String>> headers = Headers.getSetProtocolHeaders(message);
                String systemName = role.substring(SYSTEM_ROLE_PREFIX.length());
                headers.put(HEADER_SYSTEM_ID, Collections.singletonList(systemName));
                headers.put(HEADER_SYSTEM_UUID, Collections.singletonList(systemName));
                headers.put(HEADER_SYSTEM_NAME, Collections.singletonList(systemName));
                found = true;
                break;
            }
        }
        if (!found) {
            throw new Fault(new ForbiddenException("Associated profile not found"));
        }


        return;
    }
}
