/*-
 * ~~~~~~licensing~~~~~~
 * atlasmap-entaxy-services
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
/*
 * Copyright (C) 2017 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.atlasmap.api;

import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.util.Map;
import java.util.Properties;

import io.atlasmap.spi.AtlasCombineStrategy;
import io.atlasmap.spi.AtlasConversionService;
import io.atlasmap.spi.AtlasFieldActionService;
import io.atlasmap.spi.AtlasPropertyStrategy;
import io.atlasmap.spi.AtlasSeparateStrategy;

public interface AtlasContextFactory {

    static final String PROPERTY_ATLASMAP_CORE_VERSION = "atlasmap.core.version";

    enum Format { ADM, JSON };

    void init();

    void destroy();

    AtlasContext createContext(File atlasMappingFile) throws AtlasException;

    AtlasContext createContext(URI atlasMappingUri) throws AtlasException;

    AtlasContext createContext(Format format, InputStream atlasMappingStream) throws AtlasException;

    AtlasPreviewContext createPreviewContext() throws AtlasException;

    @Deprecated
    AtlasCombineStrategy getCombineStrategy() throws AtlasException;

    AtlasConversionService getConversionService() throws AtlasException;

    AtlasFieldActionService getFieldActionService() throws AtlasException;

    AtlasPropertyStrategy getPropertyStrategy() throws AtlasException;

    void setPropertyStrategy(AtlasPropertyStrategy strategy) throws AtlasException;

    @Deprecated
    AtlasSeparateStrategy getSeparateStrategy() throws AtlasException;

    AtlasValidationService getValidationService() throws AtlasException;

    void setProperties(Map<String, String> properties);

    void setProperties(Properties properties);

    Map<String, String> getProperties();

    void addClassLoader(ClassLoader cl);

}
