/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 *
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 *
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyAtlasmapConsoleModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.modalTitle}}</h4>
            </div>
              <div class="modal-body-without-header entaxy-atlasmap-console-modal">
                <div class="entaxy-atlasmap-console-container">
                  <entaxy-atlasmap-console></entaxy-atlasmap-console>
                </div>
              </div>
              <div class="modal-footer">
                <div class="btn-group" uib-dropdown>
                  <button id="split-button" type="button" class="btn btn-primary" ng-click="$ctrl.save()">
                    Save
                  </button>
                  <button type="button" class="btn btn-primary" uib-dropdown-toggle>
                    <span class="caret"></span>
                  </button>
                  <ul class="dropdown-menu" uib-dropdown-menu role="menu" aria-labelledby="split-button">
                    <li role="menuitem"><a ng-click="$ctrl.saveAs()">Save as...</a></li>
                  </ul>
                </div>
              </div>
          </div>
        `,
        controller: entaxyAtlasmapConsoleModalController
    })
        .name;

    function entaxyAtlasmapConsoleModalController($uibModal, $scope, entaxyResourcesService, entaxyHotkeysService) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function () {
            entaxyHotkeysService.setHotkeys($scope, getHotkeysConfigs());

            ctrl.modalTitle = 'Edit resource: ' + ctrl.resolve.resourceName;
        }

        ctrl.save = function () {
            let iframe = document.getElementById('atlasmapConsoleFrame');
            iframe.contentWindow.saveIntoRegistryFn();
            ctrl.modalInstance.close();
        }

        ctrl.saveAs = function () {
            entaxyResourcesService.openResourceNameInputModal(
                { name: ctrl.resolve.resourceName, isFolder: false },
                ctrl.resolve.names,
                true,
                false)
            .then((name) => {

                let location = ctrl.resolve.location.replace(ctrl.resolve.resourceName, name);
                let iframe = document.getElementById('atlasmapConsoleFrame');
                iframe.contentWindow.saveIntoRegistryAsFn(location);

                window.onmessage = function (event) {
                    if (event.origin !== iframe.contentWindow.origin) {
                        return;
                    }
                    if (event.data && event.data.type === 'savingComplete') {
                        if (event.data.result) {
                            ctrl.modalInstance.close(name);
                        } else {
                            ctrl.modalInstance.dismiss('Error occurred while saving resource ' + name);
                        }
                    }
                }
            });
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        function getHotkeysConfigs() {
            return [
                {
                    ...Entaxy.getSaveHotkeyDescription(),
                    callback: function (event) {
                        event.preventDefault();
                        ctrl.save();
                    }
                },
                {
                    ...Entaxy.getSaveAsHotkeyDescription(),
                    callback: function (event) {
                        event.preventDefault();
                        ctrl.saveAs();
                    }
                }
            ];
        }
    }
    entaxyAtlasmapConsoleModalController.$inject = ['$uibModal', '$scope', 'entaxyResourcesService',
        'entaxyHotkeysService'];

})(Entaxy || (Entaxy = {}));