/*-
 * ~~~~~~licensing~~~~~~
 * data-mapper-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.modules.data.datamapper.management.impl;

import java.util.List;

import javax.management.NotCompliantMBeanException;

import org.apache.commons.lang3.math.NumberUtils;

import ru.entaxy.esb.platform.base.management.core.api.AnnotatedMBean;
import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.modules.data.datamapper.management.EmbeddedDataMapperMBean;
import ru.entaxy.platform.modules.datamapper.DataMapper;
import ru.entaxy.platform.modules.datamapper.exceptions.DataMapperException;

public class EmbeddedDataMapperMBeanImpl extends AnnotatedMBean<EmbeddedDataMapperMBean>
        implements EmbeddedDataMapperMBean {

    protected DataMapper dataMapper;

    protected EmbeddedDataMapperMBeanImpl(DataMapper dataMapper) throws NotCompliantMBeanException {
        super(EmbeddedDataMapperMBean.class);
        this.dataMapper = dataMapper;
    }

    @Override
    public String getId() {
        return dataMapper.getId();
    }

    @Override
    public List<String> listMaps() {
        return dataMapper.getMaps();
    }

    @Override
    public Object resolve(String mapName, String sourceValue, boolean isTrimForString) throws DataMapperException {
        return dataMapper.getMapValue(mapName, sourceValue, isTrimForString);
    }

    @Override
    public void put(String mapName, String sourceValue, String targetValue, String targetType) throws Exception {

        if (!CommonUtils.isValid(sourceValue))
            throw new IllegalArgumentException("Source value is null");

        VALUE_TYPE finalType = VALUE_TYPE.STRING;

        try {
            finalType = VALUE_TYPE.valueOf(targetType);
        } catch (Exception e) {

        }

        Object finalValue = null;

        switch (finalType) {
            case BOOLEAN:
                try {
                    finalValue = Boolean.parseBoolean(targetValue);
                } catch (Exception e) {
                    finalValue = false;
                }
                break;
            case NUMBER:
                finalValue = NumberUtils.createNumber(targetValue);
                break;
            case STRING:
            default:
                finalValue = targetValue;
                break;
        }

        dataMapper.putMapValue(mapName, sourceValue, finalValue);

    }

    @Override
    public void remove(String mapName, String sourceValue) throws DataMapperException {
        dataMapper.removeMapValue(mapName, sourceValue);
    }

}
