/*-
 * ~~~~~~licensing~~~~~~
 * data-mapper-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.modules.datamapper.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.service.component.annotations.Component;
import org.osgi.service.component.annotations.Reference;
import org.osgi.service.component.annotations.ReferenceCardinality;
import org.osgi.service.component.annotations.ReferencePolicy;
import org.osgi.service.component.annotations.ReferencePolicyOption;

import ru.entaxy.platform.modules.datamapper.DataMapper;
import ru.entaxy.platform.modules.datamapper.DataMapperService;

@Component(service = DataMapperService.class, immediate = true)
public class DataMapperServiceImpl implements DataMapperService {

    protected Map<String, DataMapper> dataMappers = Collections.synchronizedMap(new HashMap<>());

    @Reference(cardinality = ReferenceCardinality.MULTIPLE, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY, unbind = "removeDataMapper")
    public void addDataMapper(DataMapper dataMapper) {
        this.dataMappers.put(dataMapper.getId(), dataMapper);
    }

    public void removeDataMapper(DataMapper dataMapper) {
        this.dataMappers.remove(dataMapper.getId());
    }

    /*
     * DataMapperService implementation
     */

    @Override
    public List<DataMapper> getDataMappers() {
        return new ArrayList<>(dataMappers.values());
    }

    @Override
    public DataMapper getDataMapper(String id) {
        return dataMappers.get(id);
    }

}
