/*-
 * ~~~~~~licensing~~~~~~
 * data-mapper-shell
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.modules.datamapper.shell;

import org.apache.karaf.shell.api.action.Command;
import org.apache.karaf.shell.api.action.Completion;
import org.apache.karaf.shell.api.action.Option;
import org.apache.karaf.shell.api.action.lifecycle.Service;

import ru.entaxy.platform.base.support.CommonUtils;
import ru.entaxy.platform.modules.datamapper.shell.completers.ValueTypeCompleter;

@Service
@Command(name = "data-mapper-put", scope = AbstractDataMapperCommand.COMMAND_SCOPE)
public class AddValueCommand extends AbstractDataMapperAwareCommand {

    @Option(name = "-d", aliases = {"--data-map"}, required = true)
    String mapName;

    @Option(name = "-k", aliases = {"--mapped-key"}, required = true)
    String mappedKey;

    @Option(name = "-v", aliases = {"--mapped-value"}, required = true)
    String mappedValue;

    @Completion(value = ValueTypeCompleter.class)
    @Option(name = "-t", aliases = {"--value-type"}, required = false)
    String valueType;

    @Override
    protected void doExecute() throws Exception {
        ValueTypes type = CommonUtils.isValid(valueType) ? ValueTypes.valueOf(valueType) : ValueTypes.STRING;
        if (type == null)
            type = ValueTypes.STRING;
        Object value = mappedValue;
        switch (type) {
            case BOOLEAN:
                value = "true".equalsIgnoreCase(mappedValue) ? true : false;
                break;

            case INTEGER:
                try {
                    value = Integer.parseInt(mappedValue);
                    break;
                } catch (Exception ignore) {
                    // ignore and try long
                }
            case LONG:
                try {
                    value = Long.parseLong(mappedValue);
                    break;
                } catch (Exception ignore) {
                    // ignore and try double
                }
            case DOUBLE:
                try {
                    value = Double.parseDouble(mappedValue);
                    break;
                } catch (Exception e) {
                    throw e;
                }

            case STRING:
            default:
                break;
        }

        dataMapper.putMapValue(mapName, mappedKey, value);

    }

}
