[#ftl attributes={"generated.type":"blueprint"}]
<?xml version="1.0" encoding="UTF-8"?>
<blueprint xmlns="http://www.osgi.org/xmlns/blueprint/v1.0.0"
           xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		   xmlns:soap-type="http://www.entaxy.ru/ExchangeTypes/1.0"
		   xsi:schemaLocation="http://www.osgi.org/xmlns/blueprint/v1.0.0 https://www.osgi.org/xmlns/blueprint/v1.0.0/blueprint.xsd">

[#import "templates:object-commons/common-utils.ftl" as utils]

	<reference id="entaxy-broker" interface="org.apache.camel.Component"
			   filter="(connection.name=entaxy-broker)"/>

	<bean id="forced" class="java.lang.IllegalArgumentException">
		<argument index="0" value="Content is invalid"/>
	</bean>

	<bean id="headersConverter" class="ru.entaxy.esb.system.common.util.HeadersConverter">
		<property name="namespace" value="http://www.entaxy.ru/ExchangeTypes/1.0"/>
		<property name="customHeaders" value="NTX_CustomHeaders"/>
		<property name="customHeaderPrefix" value="NTX_UNIFORM_EXCHANGE_CustomHeader"/>
	</bean>

	<camelContext id="[=objectId]" xmlns="http://camel.apache.org/schema/blueprint">

		<route id="[=objectId].uniform-service-active-in-connector">
			<from uri="timer://get-package?synchronous=true&amp;period=[=utils.convertConfigValue(properties.period)]"/>
			<setHeader name="NTX_SystemId">
				<constant>[=properties.profile]</constant>
			</setHeader>
			<setHeader name="ENTAXY_SystemEndpoint">
				<constant>[=utils.convertConfigValue(properties.endpointUrl)]</constant>
			</setHeader>

			<setProperty name="sender">
				<simple>[=properties.profile]</simple>
			</setProperty>
			<setProperty name="service">
				<constant>uniform-service-active</constant>
			</setProperty>
			<setProperty name="operation">
				<simple>GET</simple>
			</setProperty>
			[#if properties.generateLoggingKey?? && properties.generateLoggingKey]
				<to uri="direct-vm:fill-logging-key"/>
			[/#if]
			<log message="${headers.NTX_loggingKey} Timer for uniform exchange active-in started" loggingLevel="DEBUG"/>

			<to uri="direct:uniform-service-connector-get-package-active?block=true&amp;timeout=60000"/>
		</route>

		<!--In-->
		<route id="[=objectId].get-package-active" streamCache="true">
			<from uri="direct:uniform-service-connector-get-package-active"/>
			<when>
				<simple>${headers.ENTAXY_SystemEndpoint} == null || ${headers.NTX_SystemId} == null</simple>
				<log message="${headers.NTX_loggingKey} ENTAXY_SystemEndpoint is not set" loggingLevel="ERROR"/>
				<stop/>
			</when>
			<setHeader name="SystemHasMessages">
				<constant>true</constant>
			</setHeader>

			<setHeader name="operationNamespace">
				<constant>http://www.entaxy.ru/connector/universal/system/1.0</constant>
			</setHeader>
			<setHeader name="CamelDestinationOverrideUrl">
				<simple>${headers.ENTAXY_SystemEndpoint}</simple>
			</setHeader>

			<loop doWhile="true" copy="true">
				<simple resultType="Boolean">${headers.SystemHasMessages} == 'true'</simple>
				<to uri="direct:get-package"/>
			</loop>
		</route>

		<route id="[=objectId].direct:get-package">
			<from uri="direct:get-package"/>
			<setHeader name="NTX_BusUUID">
				<constant>FFFFFFFF-FFFF-FFFF-FFFF-FFFFFFFFFFFF</constant>
			</setHeader>
			<setHeader name="ENTAXY_Destination">
				<constant></constant>
			</setHeader>
			<to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/GetPacketRequest.xsl?failOnNullBody=false"/>
			<removeHeader headerName="NTX_BusUUID"/>
			<removeHeader headerName="ENTAXY_Destination"/>

			<setHeader name="operationName">
				<constant>GetPackets</constant>
			</setHeader>
			<to uri="direct-vm:uniform-service-send-package-active"/>
			<removeHeader headerName="SOAPAction"/>

			<setBody>
				<xpath>//soap-type:response/soap-type:packets</xpath>
			</setBody>

			<choice>
				<when>
					<xpath>//soap-type:packets/soap-type:packet[1]/soap-type:message/soap-type:content/node()
					</xpath>
					<setBody>
						<xpath>//soap-type:packets/soap-type:packet[1]</xpath>
					</setBody>

					<to uri="entaxy-broker:queue:entaxy.uniform.service.exchange.send.to.profile?exchangePattern=InOnly"/>

					<setHeader name="NTX_UUID">
						<simple>${headers.ENTAXY_MessageUUID}</simple>
					</setHeader>
					<to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/ConfirmGettingPacketRequest.xsl?failOnNullBody=false"/>
					<setHeader name="operationName">
						<constant>ConfirmGettingPackets</constant>
					</setHeader>

					<to uri="direct-vm:uniform-service-send-package-active"/>
					<removeHeader headerName="SOAPAction"/>

					<when>
						<xpath>not(//soap-type:status/text() = '200')</xpath>
						<log message="${headers.NTX_loggingKey} ConfirmGettingPacket fail" loggingLevel="ERROR"/>
						<!--                            TODO что делать с сообщением в очереди ? -->
					</when>
					<setHeader name="SystemHasMessages">
						<constant>true</constant>
					</setHeader>
				</when>
				<otherwise>
					<setHeader name="SystemHasMessages">
						<constant>false</constant>
					</setHeader>
					<log message="${headers.NTX_loggingKey} System ${headers.NTX_SystemId} does not have messages" loggingLevel="INFO"/>
				</otherwise>
			</choice>
		</route>

		<route id="[=objectId].soap-send-to-profile">
			<from uri="entaxy-broker:queue:entaxy.uniform.service.exchange.send.to.profile"/>
			<setHeader name="ENTAXY_MessageUUID">
				<xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:messageUUID/text()</xpath>
			</setHeader>
			<setHeader name="ENTAXY_Destination">
				<xpath resultType="String">
					//soap-type:packet/soap-type:header/soap-type:destination/soap-type:id/text()
				</xpath>
			</setHeader>
			<setHeader name="ENTAXY_DestinationType">
				<xpath resultType="String">
					//soap-type:packet/soap-type:header/soap-type:destination/soap-type:type/text()
				</xpath>
			</setHeader>
			<setHeader name="ENTAXY_Source">
				<xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:source/soap-type:id/text()
				</xpath>
			</setHeader>
			<setHeader name="ENTAXY_SourceType">
				<xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:source/soap-type:type/text()
				</xpath>
			</setHeader>

			<choice>
				<when>
					<simple>${headers.ENTAXY_Source} == ''</simple>
					<setHeader name="ENTAXY_Source">
						<simple>${headers.X-SystemName}</simple>
					</setHeader>
				</when>
				<when>
					<simple>${headers.NTX_SystemId} != ${headers.ENTAXY_Source}</simple>
					<log message="${headers.NTX_loggingKey} Source name header and SystemId is not equals!" loggingLevel="WARN"/>
				</when>
			</choice>
			<setHeader name="ENTAXY_MessageType">
				<xpath resultType="String">//soap-type:packet/soap-type:message/soap-type:messageType/text()</xpath>
			</setHeader>
			<setHeader name="ENTAXY_Priority">
				<xpath resultType="Integer">//soap-type:packet/soap-type:message/soap-type:priority/text()</xpath>
			</setHeader>
			<setHeader name="ENTAXY_ExchangePattern">
				<xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:exchangePattern/text()
				</xpath>
			</setHeader>
			<choice>
				<when>
					<simple>${headers.ENTAXY_ExchangePattern} == 'InOut'</simple>
					<setExchangePattern pattern="InOut"/>
				</when>
				<otherwise>
					<setExchangePattern pattern="InOnly"/>
				</otherwise>
			</choice>
			<when>
				<simple>${headers.ENTAXY_Priority} == null || ${headers.ENTAXY_Priority} == ""</simple>
				<setHeader name="ENTAXY_Priority">
					<simple resultType="Integer">4</simple>
				</setHeader>
			</when>

			<setHeader name="NTX_CustomHeaders">
				<xpath>//soap-type:packet/soap-type:message/soap-type:customHeader</xpath>
			</setHeader>

			<to uri="direct:get-content-and-content-type"/>
			<when>
				<simple>${body} == null</simple>
				<log message="${headers.NTX_loggingKey} Content is invalid" loggingLevel="ERROR"/>
				<throwException ref="forced"/>
			</when>
			<removeHeaders pattern="*" excludePattern="NTX_.+|CamelJms.+|ENTAXY_.+"/>
			<to uri="direct:exit-cascade-start" />
		</route>

		<route id="[=objectId].send-packets">
			<from uri="direct:send-packets"/>

			<setHeader name="NTX_UNIFORM_EXCHANGE_PacketCount">
				<xpath resultType="Integer">count(/soap-type:packets/soap-type:packet)</xpath>
			</setHeader>
			<when>
				<simple>${headers.NTX_UNIFORM_EXCHANGE_PacketCount} &gt; 1</simple>
				<throwException exceptionType="java.lang.UnsupportedOperationException"
								message="Not supported more then 1 packet"/>
			</when>

			<setBody>
				<xpath resultType="org.w3c.dom.Document">/soap-type:packets/soap-type:packet[1]</xpath>
			</setBody>

			<setHeader name="ENTAXY_MessageUUID">
				<xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:messageUUID/text()</xpath>
			</setHeader>
			<choice>
				<when>
					<simple>${headers.ENTAXY_MessageUUID} == "00000000-0000-0000-0000-000000000000"</simple>
					[#--                    health check --]
					<removeHeaders pattern="ENTAXY_.+|NTX_.+|X-.+"/>
					<setBody>
						<simple><![CDATA[
							<test/>
							]]>
						</simple>
					</setBody>
					<to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/PutPacketResponse.xsl?failOnNullBody=false"/>
				</when>
				<otherwise>
					<setHeader name="ENTAXY_Destination">
						<xpath resultType="String">
							/soap-type:packet/soap-type:header/soap-type:destination/soap-type:id/text()
						</xpath>
					</setHeader>
					<setHeader name="ENTAXY_DestinationType">
						<xpath resultType="String">
							/soap-type:packet/soap-type:header/soap-type:destination/soap-type:type/text()
						</xpath>
					</setHeader>

					<log message="${headers.NTX_loggingKey} Receive message ${headers.ENTAXY_MessageUUID} for ${headers.ENTAXY_Destination}"
						 loggingLevel="INFO"/>
					<setHeader name="ENTAXY_Source">
						<xpath resultType="String">
							/soap-type:packet/soap-type:message/soap-type:source/soap-type:id/text()
						</xpath>
					</setHeader>
					<setHeader name="ENTAXY_SourceType">
						<xpath resultType="String">
							/soap-type:packet/soap-type:message/soap-type:source/soap-type:type/text()
						</xpath>
					</setHeader>
					<when>
						<simple>${headers.X-SystemName} != ${headers.ENTAXY_Source}</simple>
						<log message="${headers.NTX_loggingKey} Source name header and auth username is not equals!" loggingLevel="WARN"/>
						<when>
							<simple>${headers.ENTAXY_Source} == ''</simple>
							<setHeader name="ENTAXY_Source">
								<simple>${headers.X-SystemName}</simple>
							</setHeader>
						</when>
					</when>
					<setHeader name="ENTAXY_MessageType">
						<xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:messageType/text()
						</xpath>
					</setHeader>
					<setHeader name="ENTAXY_Priority">
						<xpath resultType="Integer">/soap-type:packet/soap-type:message/soap-type:priority/text()
						</xpath>
					</setHeader>
					<setHeader name="ENTAXY_ExchangePattern">
						<xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:exchangePattern/text()
						</xpath>
					</setHeader>
					<choice>
						<when>
							<simple>${headers.ENTAXY_ExchangePattern} == 'InOut'</simple>
							<setExchangePattern pattern="InOut"/>
						</when>
						<otherwise>
							<setExchangePattern pattern="InOnly"/>
						</otherwise>
					</choice>
					<setHeader name="NTX_CustomHeaders">
						<xpath>/soap-type:packet/soap-type:message/soap-type:customHeader</xpath>
					</setHeader>
					<bean ref="headersConverter" method="xml2Json"/>
					<log message="${headers.NTX_loggingKey} Headers converted: ${headers}" loggingLevel="DEBUG"/>

					<to uri="direct:get-content-and-content-type"/>

					<when>
						<simple>${body} == null</simple>
						<log message="${headers.NTX_loggingKey} Content is invalid" loggingLevel="ERROR"/>
						<throwException ref="forced"/>
					</when>
					<removeHeaders pattern="*" excludePattern="NTX_.+|CamelJms.+|ENTAXY_.+|X-.+"/>

					<to uri="direct:exit-cascade-start" />
					<removeHeaders pattern="ENTAXY_.+|NTX_.+|X-.+"/>
					<setBody>
						<simple><![CDATA[
							<test/>
							]]>
						</simple>
					</setBody>

					<to uri="xslt:ru/entaxy/esb/platform/runtime/modules/uniform-service/xslt/PutPacketResponse.xsl?failOnNullBody=false"/>
				</otherwise>
			</choice>
		</route>

		<route id="[=objectId].get-content-and-content-type">
			<from uri="direct:get-content-and-content-type"/>

			<setHeader name="ENTAXY_ContentType">
				<xpath resultType="String">/soap-type:packet/soap-type:message/soap-type:contentType/text()
				</xpath>
			</setHeader>
			<choice>
				<when>
					<xpath>count(/soap-type:packet/soap-type:message/soap-type:content) = 1</xpath>
					<setHeader name="CamelJms_IntContentXsiType">
						<xpath resultType="String">
							/soap-type:packet/soap-type:message/soap-type:content/@xsi:type
						</xpath>
					</setHeader>
					<setHeader name="NTX_ContentHasNotEmptyText">
						<xpath resultType="Boolean">
							boolean(normalize-space(/soap-type:packet/soap-type:message/soap-type:content/text()))
						</xpath>
					</setHeader>
					<setHeader name="NTX_ContentChildNodesCount">
						<xpath resultType="Integer">
							count(/soap-type:packet/soap-type:message/soap-type:content/child::*)
						</xpath>
					</setHeader>
					<choice>
						<when>
							<simple>${headers.NTX_ContentChildNodesCount} == 1</simple>
							<setBody>
								<xpath resultType="org.w3c.dom.Document">
									/soap-type:packet/soap-type:message/soap-type:content/*
								</xpath>
							</setBody>
							<setHeader name="CamelJms_IntMessageType">
								<constant>Document</constant>
							</setHeader>
						</when>
						<when>
							<simple>${headers.NTX_ContentChildNodesCount} > 1</simple>
							<setBody>
								<xpath resultType="org.w3c.dom.NodeList">
									/soap-type:packet/soap-type:message/soap-type:content/*
								</xpath>
							</setBody>
							<setHeader name="CamelJms_IntMessageType">
								<constant>NodeList</constant>
							</setHeader>
						</when>
						<when>
							<simple>${headers.NTX_ContentChildNodesCount} == 0</simple>
							<choice>
								<when>
									<simple>${headers.NTX_ContentHasNotEmptyText}</simple>
									<setBody>
										<xpath resultType="String">
											/soap-type:packet/soap-type:message/soap-type:content/text()
										</xpath>
									</setBody>
									<setHeader name="CamelJms_IntMessageType">
										<constant>String</constant>
									</setHeader>
								</when>
								<otherwise>
									<setBody>
										<constant>
											<![CDATA[
											<emptyContent xmlns="http://www.entaxy.ru/connector/universal/internal"/>
											]]>
										</constant>
									</setBody>
								</otherwise>
							</choice>
						</when>
						<otherwise>
							<log message="${headers.NTX_loggingKey} Content is invalid" loggingLevel="ERROR"/>
							<throwException ref="forced"/>
						</otherwise>
					</choice>
					<removeHeader headerName="NTX_ContentHasNotEmptyText"/>
					<removeHeader headerName="NTX_ContentChildNodesCount"/>
				</when>
				<otherwise>
					<setBody>
						<constant>
							<![CDATA[
							<empty xmlns="http://www.entaxy.ru/connector/universal/internal"/>
							]]>
						</constant>
					</setBody>
				</otherwise>
			</choice>
		</route>
	</camelContext>
	
</blueprint>
