/*-
 * ~~~~~~licensing~~~~~~
 * jaas-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.core.management.jaas.impl;

import java.util.Collections;
import java.util.List;

import javax.management.NotCompliantMBeanException;

import org.osgi.service.component.ComponentContext;
import org.osgi.service.component.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.core.management.jaas.JaasMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeItemMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBean;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBeanFactory;
import ru.entaxy.platform.core.management.object.MBeanFactory;
import ru.entaxy.platform.core.management.object.factory.AbstractMBeanFactory;
import ru.entaxy.platform.objects.runtime.EntaxyRuntimeObject;

@MBeanFactory(id = "jaas-factory", supportedTypes = {"entaxy.security.jaas.*"})
@Component(service = EntaxyRuntimeObjectMBeanFactory.class, immediate = true)
public class JaasMBeanFactory extends AbstractMBeanFactory {

    private static final Logger log = LoggerFactory.getLogger(JaasMBeanFactory.class);

    private static final String ATTR_BACKEND_ENABLED = "backendEnabled";

    @Reference(cardinality = ReferenceCardinality.MANDATORY, policy = ReferencePolicy.DYNAMIC,
            policyOption = ReferencePolicyOption.GREEDY)
    volatile protected JaasMbeanImpl jaasMBean;

    public void activate(ComponentContext componentContext) {
//        try {
//            this.jaasMBean = new JaasMbeanImpl();
//        } catch (NotCompliantMBeanException e) {
//            log.error("Error creating JaasMbeanImpl", e);;
//        }
    }

    @Override
    public <T extends EntaxyRuntimeObjectMBean> T createMBean(EntaxyRuntimeObject entaxyRuntimeObject) {
        try {
            if (JaasMBean.TYPE_REALM.equals(entaxyRuntimeObject.getType())) {
                JaasRealmMBeanImpl mBean = new JaasRealmMBeanImpl(entaxyRuntimeObject);
                return (T) mBean;
            }
            if (JaasMBean.TYPE_LOGINMODULE.equals(entaxyRuntimeObject.getType())) {

                boolean withBackend = false;

                if (entaxyRuntimeObject.hasAttribute(ATTR_BACKEND_ENABLED)) {
                    Object value = entaxyRuntimeObject.getAttribute(ATTR_BACKEND_ENABLED);
                    if (value != null) {
                        try {
                            withBackend = Boolean.parseBoolean(value.toString());
                        } catch (Exception ignore) {
                            // ignore
                        }
                    }
                }

                if (withBackend) {
                    JaasLoginModuleWithBackingEngineMBeanImpl mBean =
                            new JaasLoginModuleWithBackingEngineMBeanImpl(entaxyRuntimeObject);
                    return (T) mBean;
                } else {
                    JaasLoginModuleMBeanImpl mBean = new JaasLoginModuleMBeanImpl(entaxyRuntimeObject);
                    return (T) mBean;
                }
            }
        } catch (Exception e) {
            log.error("Error creating MBean for [" + entaxyRuntimeObject.getObjectFullId() + "]", e);
        }
        return null;
    }

    @Override
    public List<? extends EntaxyRuntimeItemMBean> createMBeans() {
        if (jaasMBean != null)
            return Collections.singletonList(jaasMBean);
        return Collections.emptyList();
    }

}
