/*-
 * ~~~~~~licensing~~~~~~
 * jaas-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
/*
 * Licensed to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional information regarding
 * copyright ownership. The ASF licenses this file to You under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance with the License. You may obtain a
 * copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.entaxy.platform.runtime.security.jaas.modules.resources;

import java.util.Map;

import org.apache.karaf.jaas.modules.BackingEngine;
import org.apache.karaf.jaas.modules.BackingEngineFactory;
import org.apache.karaf.jaas.modules.JAASUtils;
import org.apache.karaf.jaas.modules.encryption.EncryptionSupport;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ru.entaxy.platform.runtime.security.jaas.utils.PropertiesWithCallback;
import ru.entaxy.platform.runtime.security.jaas.utils.ResourceCallback;

public class ResourceBackingEngineFactory implements BackingEngineFactory {

    private static final transient Logger log = LoggerFactory.getLogger(ResourceBackingEngineFactory.class);

    private static final String USER_FILE = "users";

    /**
     * Builds the Backing Engine
     */
    public BackingEngine build(Map<String, ?> options) {
        ResourceBackingEngine engine = null;
        String usersResource = JAASUtils.getString(options, USER_FILE);

        ResourceCallback resourceCallback = new ResourceCallback(usersResource, "# Auto created users resource");

        PropertiesWithCallback users = new PropertiesWithCallback(resourceCallback);
        try {
            users.load();
            EncryptionSupport encryptionSupport = new EncryptionSupport(options);
            engine = new ResourceBackingEngine(users, encryptionSupport);
        } catch (Exception ioe) {
            log.warn("Cannot open users file: {}", usersResource);
        }
        return engine;
    }

    /**
     * Returns the login module class, that this factory can build.
     */
    public String getModuleClass() {
        return ResourceLoginModule.class.getName();
    }

}
