/*-
 * ~~~~~~licensing~~~~~~
 * jaas-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.platform.runtime.security.jaas.utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;

import ru.entaxy.esb.resources.EntaxyResource;
import ru.entaxy.platform.runtime.security.jaas.modules.resources.ServiceHelper;

public class ResourceCallback implements PropertiesWithCallback.Callback {

    protected String resourceUrl;
    protected String defaultContent = "#";

    public ResourceCallback(String resourceUrl) {
        super();
        this.resourceUrl = resourceUrl;
    }

    public ResourceCallback(String resourceUrl, String defaultContent) {
        this(resourceUrl);
        this.defaultContent = defaultContent;
    }

    @Override
    public void load(PropertiesWithCallback properties) throws Exception {

        EntaxyResource resource = ServiceHelper.getInstance().getEntaxyResourceService().getResource(resourceUrl);
        if (resource == null)
            throw new FileNotFoundException(resourceUrl);
        if (!resource.exists()) {
            resource.save(
                    new ByteArrayInputStream(new String(defaultContent == null ? "" : defaultContent).getBytes()));
        }

        properties.load(resource.getInputStream());

    }

    @Override
    public void save(PropertiesWithCallback properties) throws Exception {
        EntaxyResource resource = ServiceHelper.getInstance().getEntaxyResourceService().getResource(resourceUrl);
        if (resource == null)
            throw new FileNotFoundException(resourceUrl);
        try (ByteArrayOutputStream bos = new ByteArrayOutputStream()) {
            properties.save(bos);
            try (ByteArrayInputStream bis = new ByteArrayInputStream(bos.toByteArray())) {
                resource.save(bis);
            }
        } catch (Exception toRethrow) {
            throw toRethrow;
        }
    }

}
