/*-
 * ~~~~~~licensing~~~~~~
 * secure-vault-runtime
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.security.vault.runtime.lookup;

import java.util.Dictionary;
import java.util.Enumeration;

import org.osgi.framework.ServiceReference;
import org.osgi.service.cm.ConfigurationPlugin;
import org.osgi.service.component.annotations.Component;

@Component(service = {ConfigurationPlugin.class}, immediate = true,
        property = {
            ConfigurationPlugin.CM_RANKING + "=100", "config.plugin.id=SecretsLookupConfigurationPlugin"})
public class SecretsLookupConfigurationPlugin implements ConfigurationPlugin {

    @Override
    public void modifyConfiguration(ServiceReference<?> arg0, Dictionary<String, Object> properties) {
        for (Enumeration<String> keys = properties.keys(); keys.hasMoreElements();) {

            String key = keys.nextElement();
            Object val = properties.get(key);

            if (val != null && val instanceof String) {
                String value = (String) val;

                if (value.startsWith(SecretsLookupProvider.PREFIX) && value.endsWith("}")) {
                    String spec = value.substring(SecretsLookupProvider.PREFIX.length(), value.length() - 1);
                    int index = spec.lastIndexOf('@');
                    if (index <= 0)
                        continue;

                    String alias = spec.substring(0, index);
                    String vault = spec.substring(index + 1);

                    String data = SecretsLookupProvider.lookupSecret(alias, vault);

                    properties.put(key, data);

                }

            }

        }
    }

}
