/*-
 * ~~~~~~licensing~~~~~~
 * keystore-management
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
package ru.entaxy.security.keystore.management;

import ru.entaxy.esb.platform.base.management.core.api.Attribute;
import ru.entaxy.esb.platform.base.management.core.api.Operation;
import ru.entaxy.esb.platform.base.management.core.api.Parameter;
import ru.entaxy.esb.platform.base.management.core.api.MBeanAnnotated;
import ru.entaxy.esb.platform.base.management.core.api.MBeanExportPolicy;
import ru.entaxy.platform.core.management.object.EntaxyRuntimeObjectMBean;
import ru.entaxy.security.keystore.runtime.EntaxyKeystore;

import javax.management.MBeanOperationInfo;
import java.util.List;

@MBeanAnnotated(policy = MBeanExportPolicy.ANNOTATED_ONLY)
public interface KeystoreMBean extends EntaxyRuntimeObjectMBean {

    String SUBQUALIFIER_KEY = "keystore";

    @Attribute(desc = "Keystore type")
    default String getKeystoreType() {
        return getRuntimeObject().getAttributeOrDefault("keystoreType", EntaxyKeystore.KEYSTORE_TYPE_DEFAULT).toString();
    }

    @Attribute(desc = "Keystore URL")
    default String getKeystoreUrl() {
        return getRuntimeObject().getAttributeOrDefault("keystoreUrl", "").toString();
    }

    @Operation(desc = "List aliases for the keystore")
    List<String> listAliases() throws Exception;

    @Operation(impact = MBeanOperationInfo.ACTION, desc = "Add new key to keystore")
    void addKey(@Parameter(name = "alias", desc = "Alias of new key") String alias,
                @Parameter(name = "keyContent", desc = "Key content as JSON string") String keyContent) throws Exception;

    @Operation(impact = MBeanOperationInfo.ACTION, desc = "Remove key by alias")
    void removeKey(@Parameter(name = "alias", desc = "Alias of key to remove") String alias) throws Exception;
}
