/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-cicd-plugin
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */

var EntaxyCICD;
(function (EntaxyCICD) {
    EntaxyCICD._module.component('entaxyCiCdApplicationItems', {
        bindings: {
            version: '<'
        },
        template:
        `
          <pf-toolbar class="entaxy-toolbar" config="$ctrl.toolbarConfig"></pf-toolbar>
          <pf-table-view class="entaxy-cicd-table"
                          config="$ctrl.tableConfig"
                          columns="$ctrl.tableColumns"
                          page-config="$ctrl.pageConfig"
                          items="$ctrl.viewedItems">
          </pf-table-view>
        `,
        controller: EntaxyCICDApplicationItemsController
    })
    .name;

    function EntaxyCICDApplicationItemsController(workspace, $scope, entaxyCICDService) {
        'ngInject';

        let ctrl = this;

        ctrl.tableConfig = {
            selectionMatchProp: 'id',
            showCheckboxes: false
        };

        ctrl.toolbarConfig = {
            filterConfig: {
                fields: [
                    {
                        id: 'id',
                        title: 'Id',
                        placeholder: 'Filter by Id...',
                        filterType: 'text'
                    },
                    {
                        id: 'type',
                        title: 'Type',
                        placeholder: 'Filter by Type...',
                        filterType: 'select',
                        filterValues: []
                    },
                    {
                        id: 'revisionable',
                        title: 'Revisionable',
                        placeholder: 'Filter by Revisionable...',
                        filterType: 'select',
                        filterValues: ['yes', 'no']
                    },
                    {
                        id: 'editable',
                        title: 'Editable',
                        placeholder: 'Filter by Editable...',
                        filterType: 'select',
                        filterValues: ['yes', 'no']
                    }
                ],
                inlineResults: true,
                appliedFilters: [],
                onFilterChange: filterChange
            },
            isTableView: true
        };

        ctrl.tableColumns = [
            { header: 'Id', itemField: 'id' },
            { header: 'Type', itemField: 'type' },
            { header: 'Revisionable', itemField: 'revisionable',
                templateFn: (value) => EntaxyCICD.getStatusTemplate(value) },
            { header: 'Editable', itemField: 'editable',
                templateFn: (value) => EntaxyCICD.getStatusTemplate(value) }
        ];

        ctrl.pageConfig = Entaxy.getStandardPageConfig();

        function filterChange(filters) {
            ctrl.viewedItems = Entaxy.applyFilters(ctrl.items, filters, matchesFilter);
            ctrl.toolbarConfig.filterConfig.resultsCount = ctrl.viewedItems.length;
        };

        function matchesFilter(item, filter) {
            let match = true;
            if (filter.id === 'id') {
                match = item.id.toLowerCase().match(filter.value.toLowerCase()) !== null;
            } else if (filter.id === 'type') {
                match = item.type === filter.value;
            } else if (filter.id === 'revisionable') {
                match = filter.value === 'yes' ? item.revisionable === true : item.revisionable === false;
            } else if (filter.id === 'editable') {
                match = filter.value === 'yes' ? item.editable === true : item.editable === false;
            }
            return match;
        };

        ctrl.items = [];
        ctrl.viewedItems = [];

        ctrl.$onInit = function() {
            entaxyCICDService.setTypesToFilterConfig(ctrl.toolbarConfig.filterConfig.fields);
        }

        $scope.$watch('$ctrl.version', function () {
            populateTable();
        });

        function populateTable() {

            const storage = EntaxyCICD.getStorage(workspace.selection);
            const application = EntaxyCICD.getProjectOrApplication(workspace.selection);

            entaxyCICDService.executeOperation('listApplicationItems', [storage, application, ctrl.version])
            .then(result => {
                let items = JSON.parse(result);
                ctrl.items = items;
                filterChange(ctrl.toolbarConfig.filterConfig.appliedFilters);
            });
        }
    }
    EntaxyCICDApplicationItemsController.$inject = ['workspace', '$scope', 'entaxyCICDService'];

})(EntaxyCICD || (EntaxyCICD = {}));
