/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-atlasmap-plugin
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyCreateResourceModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.modalTitle}}</h4>
            </div>
              <div class="modal-body-without-header">
                <div class="entaxy-create-resource-fields-container">
                  <div class="entaxy-create-resource-name-field-container" ng-class="{'has-error': $ctrl.errors.name}">
                    <label class="control-label required-pf" for="contentName">Name</label>
                    <input type="text" id="contentName" class="form-control" ng-model="$ctrl.name" set-focused="true">
                  </div>
                  <entaxy-select-from-enum id="contentType" ng-class="{'has-error': $ctrl.errors.type}"
                        values="$ctrl.selectValues" model="$ctrl.editorMode"
                        is-empty-included="false" sort-needed="true">
                  </entaxy-select-from-enum>
                </div>
                <div class="entaxy-resource-editor-container">
                  <entaxy-editor class="entaxy-editor" ng-class="{'has-error': $ctrl.errors.content}"
                        mode="$ctrl.editorMode" source-doc="$ctrl.source">
                  </entaxy-editor>
                </div>
              </div>
              <div class="modal-footer">
                <button type="submit" class="btn btn-primary" ng-click="$ctrl.save()">{{$ctrl.btnTitle}}</button>
              </div>
          </div>
        `,
        controller: entaxyCreateResourceModalController
    })
    .name;

    function entaxyCreateResourceModalController($uibModal, $scope, entaxyService, entaxyHotkeysService) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function() {
            entaxyHotkeysService.setHotkeys($scope, getHotkeysConfigs());

            ctrl.mode = Entaxy.MODAL_MODES.EDIT;
            ctrl.modalTitle = ctrl.resolve.modalTitle ? ctrl.resolve.modalTitle : 'Create resource';
            ctrl.btnTitle = 'Save';
            ctrl.selectValues = [
                'xml',
                {
                    displayValue: 'json',
                    value: { name: 'javascript', json: true }
                }
            ];
            if (ctrl.resolve.extraTypes) {
                ctrl.selectValues = ctrl.selectValues.concat(ctrl.resolve.extraTypes);
            }
            ctrl.name = ctrl.resolve.resourceName ? ctrl.resolve.resourceName : '';
            ctrl.source = '';
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.save = function() {
            if (validate()) {
                ctrl.modalInstance.close({
                    name: ctrl.name,
                    text: ctrl.source,
                    type: getType().toUpperCase()
                });
            } else {
                let fieldName = ctrl.errors.name ? 'Name' : 'Content';
                let message = ctrl.errors.type ?
                    'Selected type should match the content' : fieldName + ' value should not be empty';
                Entaxy.notificationError(message);
            }
        }

        function validate() {
            ctrl.errors = {};
            if (!ctrl.name || ctrl.name.trim().length == 0) {
                ctrl.errors.name = true;
                return false;
            }
            if (angular.isString(ctrl.source) && ctrl.source.trim().length == 0) {
                ctrl.errors.content = true;
            }
            if (!ctrl.errors.content && !isTypeFitContent()) {
                ctrl.errors.type = true;
            }
            return Object.keys(ctrl.errors).length == 0;
        }

        function getType() {
            let selectedValue = ctrl.selectValues.find(selectValue => selectValue.value === ctrl.editorMode);
            return selectedValue !== undefined ? selectedValue.displayValue : ctrl.editorMode;
        }

        function isTypeFitContent() {
            if (ctrl.editorMode === 'xml') {
                return entaxyService.isSupposedlyXml(ctrl.source);
            } else if (angular.isObject(ctrl.editorMode) && ctrl.editorMode.json) {
                return entaxyService.isSupposedlyJson(ctrl.source);
            } else {
                return true;
            }
        }

        function getHotkeysConfigs() {
            return [
                {
                    ...Entaxy.getSaveHotkeyDescription(),
                    callback: function (event) {
                        event.preventDefault();
                        ctrl.save();
                    }
                }
            ];
        }
    }
    entaxyCreateResourceModalController.$inject = ['$uibModal', '$scope', 'entaxyService', 'entaxyHotkeysService'];

})(Entaxy || (Entaxy = {}));
