/*-
 * ~~~~~~licensing~~~~~~
 * entaxy-management-plugin
 * ==========
 * Copyright (C) 2020 - 2025 EmDev LLC
 * ==========
 * You may not use this file except in accordance with the License Terms of the Copyright
 * Holder located at: https://entaxy.ru/eula . All copyrights, all intellectual property
 * rights to the Software and any copies are the property of the Copyright Holder. Unless
 * it is explicitly allowed the Copyright Holder, the User is prohibited from using the
 * Software for commercial purposes to provide services to third parties.
 * 
 * The Copyright Holder hereby declares that the Software is provided on an "AS IS".
 * Under no circumstances does the Copyright Holder guarantee or promise that the
 * Software provided by him will be suitable or not suitable for the specific purposes
 * of the User, that the Software will meet all commercial and personal subjective
 * expectations of the User, that the Software will work properly, without technical
 * errors, quickly and uninterruptedly.
 * 
 * Under no circumstances shall the Copyright Holder or its Affiliates is not liable
 * to the User for any direct or indirect losses of the User, his expenses or actual
 * damage, including, downtime; loss of bussines; lost profit; lost earnings; loss
 * or damage to data, property, etc.
 * ~~~~~~/licensing~~~~~~
 */
var Entaxy;
(function (Entaxy) {
    Entaxy._module.component('entaxyUserManagementModal', {
        bindings: {
            modalInstance: '<',
            resolve: '<'
        },
        template:
        `
          <div class="entaxy-modal-container simple-modal-content">
            <div class="modal-header">
              <button type="button" class="close" aria-label="Close" ng-click="$ctrl.cancel()">
                <span class="pficon pficon-close" aria-hidden="true"></span>
              </button>
              <h4 class="modal-title">{{$ctrl.resolve.modalTitle}}</h4>
            </div>
            <div class="modal-body">
              <form name="entaxyObjectForm" class="form-horizontal">
                <div class="form-group" ng-repeat="formField in $ctrl.formFields"
                        ng-class="{'has-error': $ctrl.errors[formField.name]}" ng-if="!formField.isHidden">
                  <div class="col-sm-3 label-col">
                    <label class="control-label" ng-class="{'required-pf': formField.required}" for="{{formField.name}}">
                      {{formField.label}}
                    </label>
                    <button type="button" class="btn btn-link label-description-popover"
                            popover-placement="auto top-left" popover-trigger="'outsideClick'"
                            uib-popover="{{formField.description}}" ng-if="formField.description">
                      <span class="pficon pficon-help"></span>
                    </button>
                  </div>
                  <div class="col-sm-8">
                    <input type="{{formField.type}}" id="{{formField.name}}" class="form-control"
                            ng-model="formField.value" ng-readonly="formField.readOnly"
                            ng-if="!formField.enum && formField.type !== 'password' && formField.type !== 'textarea'"
                            set-focused="formField.setFocused">
                    <entaxy-password-input id="{{formField.name}}" name="formField.name" model="formField.value"
                            confirmation-model="formField.confirmationValue" readonly="formField.readOnly"
                            ng-if="formField.type === 'password'" errors="$ctrl.errors"
                            set-focused="formField.setFocused"></entaxy-password-input>
                    <entaxy-select-from-enum id="{{formField.name}}" values="formField.values"
                            model="formField.value" readonly="formField.readOnly" is-empty-included="false"
                            ng-if="formField.enum"></entaxy-select-from-enum>
                    <textarea type="{{formField.type}}" id="{{formField.name}}" class="form-control"
                            ng-model="formField.value" ng-readonly="formField.readOnly"
                            ng-if="formField.type === 'textarea'" set-focused="formField.setFocused"/>
                    <span class="help-block" ng-show="$ctrl.errors[formField.name]">
                      {{$ctrl.errors[formField.name]}}
                    </span>
                  </div>
                </div>
              </form>
            </div>
            <div class="modal-footer">
              <button type="submit" class="btn btn-primary" ng-click="$ctrl.save()">Save</button>
            </div>
          </div>
        `,
        controller: entaxyUserManagementModalController
    })
    .name;

    function entaxyUserManagementModalController($scope, $uibModal, entaxyHotkeysService) {
        'ngInject';

        let ctrl = this;

        ctrl.$onInit = function() {
            entaxyHotkeysService.setHotkeys($scope, getHotkeysConfigs());

            ctrl.formFields = ctrl.resolve.formFields;
            if (!ctrl.formFields || ctrl.formFields.length === 0) {
                ctrl.cancel('Fields can not be found');
            }
        }

        ctrl.cancel = function(reason) {
            ctrl.modalInstance.dismiss(reason);
        }

        ctrl.save = function() {
            ctrl.errors = validate();
            if (Object.keys(ctrl.errors).length === 0) {
                let args = ctrl.formFields.map(formField => formField.value);
                ctrl.modalInstance.close(args);
            }
        }

        function validate() {
            let errors = {};

            ctrl.formFields.forEach(formField => {
                if (formField.type === 'password' && ctrl.errors && ctrl.errors[formField.name]) {
                    errors[formField.name] = ctrl.errors[formField.name];
                }

                if (!errors[formField.name] && formField.required
                    && (!formField.value || formField.value.trim().length === 0)) {
                    errors[formField.name] = Entaxy.ERROR_MESSAGE.EMPTY;
                }

                if (!errors[formField.name] && formField.validation && formField.validation.checkUniqueness
                        && formField.validation.values.includes(formField.value)) {
                    errors[formField.name] = Entaxy.ERROR_MESSAGE.UNIQUE;
                }
            });

            return errors;
        }

        function getHotkeysConfigs() {
            return [
                {
                    ...Entaxy.getSaveHotkeyDescription(),
                    callback: function (event) {
                        event.preventDefault();
                        ctrl.save();
                    }
                }
            ];
        }
    }
    entaxyUserManagementModalController.$inject = ['$scope', '$uibModal', 'entaxyHotkeysService'];

})(Entaxy || (Entaxy = {}));
